/* Simulated CCCId for filterd testing. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libtrap/trap.h>
#include <signal.h>
#include <time.h>
#include <getopt.h>
#include <limits.h>
#include <unistd.h>
#include <arpa/inet.h>

#include "ccci_struct/ccci_msg.h"

static int stop = 0;
TRAP_DEFAULT_SIGNAL_HANDLER(stop = 1);

trap_module_info_t module_info = {
    "Module for simulating function of the CCCId.",
    "This module generates rules and sends them to filterd.\n \
     Parameters:\n \
     -a Randomize action between adding and deleting.\n \
     -r N Generate N random rules and finish.\n \
     -c Allow user to create a rule and send it. \n", 
    0,
    1
};

static void generate_rule(int act_rand, int ip_ver, union ccci_msg *msg, union ccci_rule *rule, union ccci_ipaddr *ip)
{
    ccci_msg_setaction(msg, 0);        
    ccci_msg_setrid(msg, rand() % (UINT16_MAX + 1));
    ccci_msg_setsid(msg, rand() % (UINT16_MAX + 1));

    ccci_rule_settype(rule, 3);
    ccci_rule_setipver(rule, 4);

    ccci_rule_setipmask(rule, 32);

    ccci_ipaddr_setipv4(ip, 0x93E50E92);

    memcpy((rule->raw32 + CCCI_RULE32_IP0), ip, sizeof(union ccci_ipaddr));

    ccci_rule_setproto4(rule, 17);

    ccci_rule_setport0(rule, 80);

    memcpy((msg->raw + CCCI_MSG8_RULE), rule, sizeof(union ccci_rule));
}

static void construct_rule(char *rule_str, union ccci_msg *msg)
{
    char *tk;
    uint8_t rule_type;
    uint8_t ipver;
    uint8_t pref;
    uint8_t proto;
    uint16_t port;
    uint32_t ip;
    uint16_t sid_rid;
    unsigned char ip6[16];

    union ccci_ipaddr ccci_ip;

    union ccci_rule *rule = ccci_msg_getrule(msg);

    // RULE TYPE
    tk = strtok(rule_str, "|");

    rule_type = strtoul(tk, NULL, 0);

    ccci_rule_settype(rule, rule_type);

    //IPVER
    tk = strtok(NULL, "|");
    ipver = strtoul(tk, NULL, 0);
    ccci_rule_setipver(rule, ipver);

    //PREF LEN
    tk = strtok(NULL, "|");
    if (rule_type == 0) {
        pref = strtoul(tk, NULL, 0);
        ccci_rule_setipmask(rule, pref);
    }
 
    //SRC IP
    tk = strtok(NULL, "|");
    if (ipver == 4) {
        inet_pton(AF_INET, tk, &ip);
        ip = ntohl(ip);
        ccci_ipaddr_setipv4(&ccci_ip, ip);
    } else if (ipver == 6) {
        inet_pton(AF_INET6, tk, &ip6);
        ccci_ipaddr_setipv6(&ccci_ip, ip6);
    }
    memcpy((rule->raw32 + CCCI_RULE32_IP0), &ccci_ip, sizeof(union ccci_ipaddr));

    //DST IP
    tk = strtok(NULL, "|");
    if (rule_type == 1) {
        if (ipver == 4) {
            inet_pton(AF_INET, tk, &ip);
            ip = ntohl(ip);
            ccci_ipaddr_setipv4(&ccci_ip, ip);
        } else if (ipver == 6) {
            inet_pton(AF_INET6, tk, &ip6);
            ccci_ipaddr_setipv6(&ccci_ip, ip6);
        }
        memcpy((rule->raw32 + CCCI_RULE32_IP1), &ccci_ip, sizeof(union ccci_ipaddr));
    }

    //SRC PORT
    tk = strtok(NULL, "|");
    if (rule_type == 1 || rule_type == 3) {
        port = strtoul(tk, NULL, 0);
        ccci_rule_setport0(rule, port);
    }

    //DST PORT
    tk = strtok(NULL, "|");
    if (rule_type == 1) {
        port = strtoul(tk, NULL, 0);
        ccci_rule_setport1(rule, port);
    }

    //PROTO
    tk = strtok(NULL, "|");
    if (rule_type == 1 || rule_type == 3) {
        proto = strtoul(tk, NULL, 0);
        ccci_rule_setproto4(rule, proto);
    }

    //RID
    tk = strtok(NULL, "|");
    sid_rid = strtoul(tk, NULL, 0);
    ccci_msg_setsid(msg, sid_rid); 
    ccci_msg_setrid(msg, sid_rid);

}

int main(int argc, char** argv)
{
    if (argc != 5) {
        exit(2);
    }

    TRAP_DEFAULT_INITIALIZATION(argc, argv, module_info);
    TRAP_REGISTER_DEFAULT_SIGNAL_HANDLER();

    union ccci_msg msg = ccci_msg_new();
    union ccci_rule rule;
    union ccci_ipaddr ip;

    if (*argv[1] == 'A') {
        ccci_msg_setaction(&msg, 0);
    } else if (*argv[1] == 'D') {
        ccci_msg_setaction(&msg, 1);
    }
    
    trap_ifcctl(TRAPIFC_OUTPUT,0,TRAPCTL_BUFFERSWITCH,0);

    construct_rule(argv[2], &msg);

    trap_send_data(0, &msg, sizeof(union ccci_msg), TRAP_WAIT);

    TRAP_DEFAULT_FINALIZATION();

    return 0;
}
