/**
 * \file filter_types.h
 * \brief Header file with data types for filtering table.
 * \author Roman Vrana, xvrana20@stud.fit.vutbr.cz
 * \date 2014
 */


#ifndef FILTER_TYPES_H
#define FILTER_TYPES_H

#include <sdm.h>

/* Consatnts for switching flow key creation method */
#define MSG_CCCI    1
#define MSG_SZE     2

/* L4 protocol numbers */
#define L4_TCP      0x06
#define L4_UDP      0x11

/**
 * Flow key for hash filter. See filter_control.c for key creating
 * method.
 */
typedef union {
    struct item_key {
        /*@{*/
        sdm_ip_t ip_1; /**< IP address 1. */
        sdm_ip_t ip_2; /**< IP Address 2. */
        uint16_t port_1; /**< Port 1. */
        uint16_t port_2; /**< Port 2. */
        uint8_t proto; /**< L4 protocol */
        uint8_t ipver; /**< Version of IP protocol */
        /*@}*/
    } items;
    char key[sizeof(struct item_key)];
} item_key_t;

/**
 * Structure with sending flags stored in hash filter.
 */
//typedef struct item_param_s {
    /*@{*/
//    uint32_t rid; /**< RID of the flow. */
//    uint8_t send_flags; /**< Flags for frame sending. */
    /*@}*/
//} item_param_t;

/**
 * Full structure of the flow filter item for use both in hash table and stash.
 */
typedef struct filter_item_s {
    /*@{*/
    sdm_flow_identifier_t key; /**< Flow ID key. */
    uint32_t rid; /**< Report parameters of the flow. */
    /*@}*/
} filter_item_t;

//typedef struct filter_item {
    /*@{*/
//    item_key_t key; /**< Flow ID key. */
//    item_param_t rule_info; /**< Report parameters of the flow. */
    /*@}*/
//} filter_item_old_t;

/**
 * Structure with flow data for use in stash.
 */
typedef struct drop_item_s {
    /*@{*/
    filter_item_t *item; /**< Item payload (same as hash filter).*/
    struct drop_item_s *next; /**< Next item in stash. */
    /*@}*/
} drop_item_t;

/**
 * Structure for filter match reporting.
 */
typedef struct match_s {
    /*@{*/
    uint8_t match; /**< Match flag */
    uint32_t src_rid; /**< RID for matching source */
    uint32_t dst_rid; /**< RID for matching destination */
    /*@}*/
} match_t;

/**
 * Structure with offsets for identifiers extraction.
 */ 
typedef struct offsets_s {
    /*@{*/
    uint8_t ip_ver; /**< IP version.*/
    uint8_t iph_start; /**< Start of L3 header. */
    uint8_t ip_start; /**< Start of source ip address. */
    uint8_t l4_start; /**< Start of L4 header. */
    uint8_t protocol; /**< L4 protocol offset. */
    uint8_t frame_data; /**< Offset to actual frame in SZE segment. */
    /*@}*/
} offsets_t;

#endif
