/*
 * Soubor:  iri-iif-adb.c
 * Datum:   27.05.2014
 * Autor:   Stanislav Bárta, xbarta29@stud.fit.vutbr.cz
 * Projekt: IRI-IIF pro SDM
 * Popis:   soubor obsahujici funkce pro spravu aplikacniho zpozdovaciho bufferu
 */

#include "adb.h"

__thread int threadId; // identifikator vlakna, ktory je preneseny z filtrd pri inicializacii
__thread char interfaceData[BYTES_PER_RECORD]; // pole pre prijem sprav z kniznice pre medzivlaknovu komunikaciu

// fronty pozadavku z IRI-Core a filtr demonu
itc_ptr interfaceFromFilterD;
itc_ptr interfaceFromIriCore;



/*
 * funkce pro ulozeni noveho paketu do ADB
 * @param flows_hash_table_item_t * ukazatel na zaznam toku do jehoz ADB se bude ukladat
 * @param iri_iif_offsets_t * ukazatel na strukturu obsahujici offsety
 * @param ini3payload_t * ukazatel na paket ukladany do ADB
 */
void iri_iif_adb_insert_packet(iri_iif_hash_table_item_t *input_hashTableRecord, iri_iif_offsets_t *input_offsets, ini3payload_t *input_packet)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: iri_iif_adb_insert_packet()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	packetBufferItem_ptr bufferItem = NULL; // ukazatel na ulozeny paket
	
	// pre dany zaznam sa uz nebudu ukladat dalsie pakety
	if( input_hashTableRecord->additionalData->adb_store == 0 )
		return;
	
	// ulozenie paketu a ziskanie ukazatela na ulozeny paket
	iri_iif_packet_buffer_packet_save( input_hashTableRecord, input_offsets, input_packet, &bufferItem );
	
	// do ulozeneho paketu si nastavim priznak, ze ADB pouziva dany paket
	bufferItem->adb_inUse = TRUE;
	
	// jedna sa o prvy paket ulozeny pre ADB daneho toku
	if( input_hashTableRecord->additionalData->adb_first == NULL )
	{
		input_hashTableRecord->additionalData->adb_first = bufferItem;
	}
	// tok uz obsahuje nejaky ADB pakety, novy paket umiestnim na konec
	else
	{
        #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
            printf("%d %s %d: // TODO: tady to padalo (1)\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
        #endif
		input_hashTableRecord->additionalData->adb_last->adb_next = bufferItem;
        #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
            printf("%d %s %d: // TODO: tady to padalo (2)\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
        #endif
	}
	
	// ulozeny paket vlozim na koniec ADB
	input_hashTableRecord->additionalData->adb_last = bufferItem;
}

/*
 * funkce pro uvolneni celeho aplikacniho zpozdovaciho bufferu
 * @param adb_t * ukazatel na uvolnovane ADB
 */
// zmazani celeho ADB k nejakemu toku (ide sa mazat)
void iri_iif_adb_free(iri_iif_hash_table_item_t * input_hashTableRecord)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: iri_iif_adb_free()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	packetBufferItem_ptr nextRecord; // pomocny ukazatel na nasledujuci prvok
	
	// zmazem vsetky pakety
	while( input_hashTableRecord->additionalData->adb_first != NULL )
	{
		// zalohujem si ukazatel na dalsi prvok
		nextRecord = input_hashTableRecord->additionalData->adb_first->adb_next;
		
		// oznacim paket ako nepouzivany v ADB
		input_hashTableRecord->additionalData->adb_first->adb_inUse = FALSE;
		
		// odstranim aktualny paket
		iri_iif_packet_buffer_packet_free( input_hashTableRecord, input_hashTableRecord->additionalData->adb_first );
		
		// prejdem na zalohovany ukazatel
		input_hashTableRecord->additionalData->adb_first = nextRecord;
	}
	
	// vynulujem ukazatel na posledny paket v ADB zozname
	input_hashTableRecord->additionalData->adb_last = NULL;
}

void iri_iif_adb_send_to_ini3( iri_iif_hash_table_item_t * input_hashTableRecord, uint32_t input_rid )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: iri_iif_adb_send_to_ini3()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	packetBufferItem_ptr nextRecord; // pomocny ukazatel na nasledujuci prvok
	ini3payload_t packetFromAdb; // paket, ktory sa odosle na ini3
	int trapReturnCode; // navratova hodnota
	
	while( input_hashTableRecord->additionalData->adb_first != NULL )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
			printf("%d %s %d: odoslanie paketu na INI3\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		packetFromAdb = input_hashTableRecord->additionalData->adb_first->packet;
		packetFromAdb.parts.header.rid[0] = input_rid; // k paketu sa prida ziskane RID
		packetFromAdb.parts.header.rid[1] = input_rid; // k paketu sa prida ziskane RID
		
        // odoslanie paketu na INI3:
		trapReturnCode = trap_send(0, packetFromAdb.raw, sizeof(ini3payload_t));
		TRAP_DEFAULT_SEND_ERROR_HANDLING(trapReturnCode, 0, 0);
		
		// najprv si zalohujem ukazatel na dalsi prvok
		nextRecord = input_hashTableRecord->additionalData->adb_first->adb_next;
		
		// aby som vedel ze dany prvok uz ADB nevyuziva (potrebne ked rovnaky prvok vyuziva viac ako 1 blok)
		input_hashTableRecord->additionalData->adb_first->adb_inUse = FALSE;
		
		// odstranenie prvku
		iri_iif_packet_buffer_packet_free( input_hashTableRecord, input_hashTableRecord->additionalData->adb_first );
		
		// prejdenie na dalsi prvok obnovenim zalohovaneho odkazu
		input_hashTableRecord->additionalData->adb_first = nextRecord;
	}
	
	// vynulujem ukazatel na posledny paket v ADB zozname
	input_hashTableRecord->additionalData->adb_last = NULL;
}

/*
 * funkce pro zpracovani textoveho retezce prijateho jako odpoved z IRI-core
 * @param char * ukazatel na prijaty retezec
 * @param int delka prijateho retezce
 * @param item_key_t * ukazatel na klic, ktery ma byt na zaklade retezce vytvoren
 * @return int identifikator vlakna kteremu vytvoreny klic patri
 */
int iri_iif_adb_parse_reply_from_iri_core(char *data, int length, item_key_t *key)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: iri_iif_adb_parse_reply_from_iri_core()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	int ip_ver = 99;
	int index = 0;
	int cast = 0;
	char id[20]; // id vlakna
	char proto[20];
	char ip_1_str[INET6_ADDRSTRLEN];
	char ip_2_str[INET6_ADDRSTRLEN];
	char port_1_str[20];
	char port_2_str[20];
	
	memset(&id[0], 0, sizeof(id));
	memset(&proto[0], 0, sizeof(proto));
	memset(&ip_1_str[0], 0, sizeof(ip_1_str));
	memset(&ip_2_str[0], 0, sizeof(ip_2_str));
	memset(&port_1_str[0], 0, sizeof(port_1_str));
	memset(&port_2_str[0], 0, sizeof(port_2_str));
	
	for(int i = 1 ; i < length ; i++)
	{
		if(cast == 0) // id vlakna
		{
			if(data[i] == ',')
			{
				cast = 1; // protocol
				index = 0;
			}
			else
			{
				id[index] = data[i];
				index++;
			}
		}
		else if(cast == 1) // protokol
		{
			if(data[i] == ',')
			{
				cast = 20; // ip 1
				index = 0;
			}
			else if(data[i] == ' ')
			{
				continue;
			}
			else
			{
				proto[index] = data[i];
				index++;
			}
		}
		else if(cast == 20) // ip 1
		{
			if(data[i] == '\'' || data[i] == '"')
			{
				cast = 2; // ip 1
				index = 0;
			}
		}
		else if(cast == 2) // ip 1
		{
			if(data[i] == '\'' || data[i] == '"')
			{
				cast = 30; // port 1
				index = 0;
			}
			else
			{
				if(data[i] == ':')
				{
					ip_ver = 6;
				}
				else if(data[i] == '.')
				{
					ip_ver = 4;
				}
				ip_1_str[index] = data[i];
				index++;
			}
		}
		else if(cast == 30) // port 1
		{
			if(data[i] == ',')
			{
				cast = 3; // port 1
				index = 0;
			}
		}
		else if(cast == 3) // port 1
		{
			if(data[i] == ',')
			{
				cast = 40; // ip 2
				index = 0;
			}
			else if(data[i] == ' ')
			{
				continue;
			}
			else
			{
				port_1_str[index] = data[i];
				index++;
			}
		}
		else if(cast == 40) // ip 2
		{
			if(data[i] == '\'' || data[i] == '"')
			{
				cast = 4; // ip 2
				index = 0;
			}
		}
		else if(cast == 4) // ip 2
		{
			if(data[i] == '\'' || data[i] == '"')
			{
				cast = 50; // port 2
				index = 0;
			}
			else if(data[i] == ' ')
			{
				continue;
			}
			else
			{
				ip_2_str[index] = data[i];
				index++;
			}
		}
		else if(cast == 50) // port 2
		{
			if(data[i] == ',')
			{
				cast = 5; // port 2
				index = 0;
			}
		}
		else if(cast == 5) // port 2
		{
			if(data[i] == ')')
			{
				break;
			}
			else if(data[i] == ' ')
			{
				continue;
			}
			else
			{
				port_2_str[index] = data[i];
				index++;
			}
		}
	}
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: vyparsovane udaje id - %s, proto - %s, ip_ver - %d, ip1 - %s, port1 - %s, ip2 - %s, port2 - %s\n", (unsigned int)pthread_self(), __FILE__, __LINE__, id, proto, ip_ver, ip_1_str, port_1_str, ip_2_str, port_2_str);
    #endif

	
	if(ip_ver == 4)
	{
		inet_pton(AF_INET, ip_1_str, key->items.ip_1.v4_long.data.d8);
		inet_pton(AF_INET, ip_2_str, key->items.ip_2.v4_long.data.d8);
	}
	else if(ip_ver == 6)
	{
		inet_pton(AF_INET6, ip_1_str, key->items.ip_1.v6.d8);
		inet_pton(AF_INET6, ip_2_str, key->items.ip_2.v6.d8);
	}
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
	else
	{
		printf("%d %s %d: INY PROTOKOL\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	}
	#endif
	
	key->items.port_1 = atoi(port_1_str);
	key->items.port_2 = atoi(port_2_str);
	key->items.proto = atoi(proto);
	key->items.ipver = ip_ver; // na nekerych systemoch: items.ip_ver
	
	return atoi(id);
}

/*
 * @param void * ukazatel na strukturu obsahujici parametry vytvareneho vlakna
 * @return void * 
 */
void *iri_iif_adb_listener(void *args)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: iri_iif_adb_listener()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	iri_iif_adb_args_t *adb = (iri_iif_adb_args_t *) args;
	intercept_identifier_t intercept_id; // prijate data su ukladane do struktury
	int thread_index; // identifkator vlakna pre ktore patri prijata IRI odpoved
	char data[REPLY_LEN]; //  jedna sprava zo socketu
    char dataBuffer[REPLY_LEN * 2]; //  data prijate zo socketu
	char intercept_data[BYTES_PER_RECORD]; // data odoslanie jednotlivym vlaknam
	int itc_ret; // navratova hodnota z rozhrania
	int received; // premenna oznamujuca pocet prijatych bajtov
    int bufferLength; // ukazal na zaciatok dalsej spravy zo socketu (pocas 1 citania sa moze nacitat viacero sprav)
    char * endOfMessageIndex;
    int messageLength;
	
	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL); // disabling interuption of thread
    
    bzero(data, REPLY_LEN);
    bzero(dataBuffer, REPLY_LEN * 2);
    bufferLength = 0;
    
    // hlavni smycka spravy ADB
	while(1)
	{
        received = read(adb->iri_iif_socket, data, REPLY_LEN); // prijeti odpovedi ze socketu
        
		// neprijal som ziadne data
		if(received <= 0)
		{
            #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
                printf("%d %s %d: chyba pri prijimani dat z IRI-core socketu\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
            #endif
			break;
		}
        
        memcpy( dataBuffer + bufferLength, data, received ); // pridat prijatu spravu do bufferu
        bufferLength += received;
        
        while(1) // pokial je co spracovavat
        {
            endOfMessageIndex = strchr(dataBuffer, 10);
                       
            if( dataBuffer[0] == '\'' && bufferLength >= 12 ) // length("'HEARTBEAT'\n") = 12
            {
                memmove( dataBuffer, dataBuffer + 12, 12);
                bufferLength -= 12;
            }
            else if( endOfMessageIndex != NULL)
            {
                #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
                    printf("%d %s %d: prijata sprava od iri-core\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
                #endif
                
                messageLength = endOfMessageIndex - dataBuffer + 1;
                
                // vynulovanie struktury kde bude ulozeny kluc
                memset(&intercept_id, 0x0, sizeof(intercept_identifier_t) );
                
                // jedna sa o negativne potvrdovani, takze RID nie je dostupne z IRI-core
                intercept_id.rid = 0;
                
                // zpracovani prijate textove odpovedi
                thread_index = iri_iif_adb_parse_reply_from_iri_core(dataBuffer, bufferLength, &(intercept_id.key));
                
                // skopirovanie struktury s klucom a cislom odposlechu do bajtoveho pola
                memcpy(intercept_data, (char *)(&intercept_id), sizeof(intercept_identifier_t));
                
                // vlozeni zaznamu do struktury mezivlaknove komunikace
                itc_ret = itc_write(adb->interface_for_adb, thread_index, intercept_data);
                
                memmove( dataBuffer, dataBuffer + messageLength, REPLY_LEN * 2 - messageLength);
                bufferLength -= messageLength;
            }
            else
            {
                break;
            }
        }
    }
    #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
        printf("%d %s %d: ukoncenie ADB vlakna\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
    #endif
}


/*
 * funkce pro precteni pozadavku z bloku ze spravy ADB (oznamuje sa ze, na zadany tok neni nastaveny odposlech)
 */
void iri_iif_check_iri_reply_interface()
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: iri_iif_check_iri_reply_interface()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	intercept_identifier_t *interceptId; // premenna obsahujuca paticu a RID
	iri_iif_hash_table_item_t *flowRecord; // premenna obsahujuca polozku z uloziska
    
    // spracuje vsechny zpravy z IRI-Core
	while( itc_read( interfaceFromIriCore, threadId, interfaceData ) )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
			printf("%d %s %d: prijate data z iri-core\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// prekonvertovanie prijatych bajtov do struktury
		interceptId = (intercept_identifier_t *) interfaceData;
		
		// kontrola ci sa jedna o spravu obsahujucu uzitocne data alebo o keepalive
		if( interceptId->key.items.port_1 == 0 && interceptId->key.items.port_2 == 0 )
		{
			continue;
		}
		
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1 || IRI_IIF_DEBUG_COMMUNICATION == 1
			printf("%d %s %d: prijata sprava z iri-core\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// vyhladanie toku v ulozisku
		flowRecord = iri_iif_find_flow_record( &( interceptId->key ) );
		
		// tok na ktory prisla odpoved sa nachadza v hash tabulke
		if(flowRecord != NULL &&
		// prvek nie je zmazany v prvej faze
			flowRecord->additionalData != NULL )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
				printf("%d %s %d: odpoved na iri\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			
			// poznacim si ze uz nemusim cekat na odpoved pro danu spravu
			flowRecord->additionalData->modules_waitingForReplies = flowRecord->additionalData->modules_waitingForReplies - 1;
			
			// odstranenie paketov z ADB
			iri_iif_adb_free( flowRecord );
			
			// dalej uz neukladat do ADB
			flowRecord->additionalData->adb_store = 0;
			
                        #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
                                printf("%d %s %d: flowRecord->additionalData->modules_waitingForReplies=%d,flowRecord->timeDelete=%u\n", (unsigned int)pthread_self(), __FILE__, __LINE__, flowRecord->additionalData->modules_waitingForReplies, flowRecord->timeDelete);
                        #endif
			
			
			// pokial neocakavam odpovede na dalsie spravy
			if( flowRecord->additionalData->modules_waitingForReplies == 0 &&
			// pokial tok uz skoncil (este pred prijatim odpovede) tak uz ho mozem vymazat
				flowRecord->timeDelete != 4294967295 )
			{
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
					printf("%d %s %d: odstranim zaznam pretoze sa cekalo na prijati poslednej odpovede\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
				#endif
				
				// oznacenie zaznamu pre zmazanie + odstranenie ADB, TCPr a oznamenie modulom o ukonceni toku
				iri_iif_hash_table_destroy_record_phase_1( flowRecord, 0 );
			}
		}
        #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
        else
        {
            printf("%d %s %d: tok na kery dosel block z IRI-Core sa nenachadza v ulozisku\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
        }
        #endif
	}
}


/*
 * funkce pro kontrolu pozadavku z filter demona
 */
void iri_iif_check_filterd_interface()
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: iri_iif_check_filterd_interface()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	intercept_identifier_t *interceptId; // premenna obsahujuca paticu a RID
	iri_iif_hash_table_item_t *flowRecord; // premenna obsahujuca polozku z hash tabulky
	
	// citam data pokial sa daju nejake citat
	while( itc_read( interfaceFromFilterD, threadId, interfaceData ) )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1 || IRI_IIF_DEBUG_COMMUNICATION == 1
			printf("%d %s %d: prijata sprava z filterd\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// prekonvertovanie prijatych bajtov do struktury
		interceptId = (intercept_identifier_t *) interfaceData;
		
		
        // prehodeni big a little endian
        if (interceptId->key.items.ipver == 4)
        {
            interceptId->key.items.ip_1.v4_long.data.d32[0] =  htobe32( interceptId->key.items.ip_1.v4_long.data.d32[0] );
            interceptId->key.items.ip_2.v4_long.data.d32[0] =  htobe32( interceptId->key.items.ip_2.v4_long.data.d32[0] );
        }
        else if (interceptId->key.items.ipver == 6)
        {
            uint32_t temp0, temp1, temp2, temp3;
            temp0 = interceptId->key.items.ip_1.v6.d32[0];
            temp1 = interceptId->key.items.ip_1.v6.d32[1];
            temp2 = interceptId->key.items.ip_1.v6.d32[2];
            temp3 = interceptId->key.items.ip_1.v6.d32[3];
            interceptId->key.items.ip_1.v6.d32[0] = htonl(temp3);
            interceptId->key.items.ip_1.v6.d32[1] = htonl(temp2);
            interceptId->key.items.ip_1.v6.d32[2] = htonl(temp1);
            interceptId->key.items.ip_1.v6.d32[3] = htonl(temp0);
            
            temp0 = interceptId->key.items.ip_2.v6.d32[0];
            temp1 = interceptId->key.items.ip_2.v6.d32[1];
            temp2 = interceptId->key.items.ip_2.v6.d32[2];
            temp3 = interceptId->key.items.ip_2.v6.d32[3];
            interceptId->key.items.ip_2.v6.d32[0] = htonl(temp3);
            interceptId->key.items.ip_2.v6.d32[1] = htonl(temp2);
            interceptId->key.items.ip_2.v6.d32[2] = htonl(temp1);
            interceptId->key.items.ip_2.v6.d32[3] = htonl(temp0);
        }
		
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1 || IRI_IIF_DEBUG_COMMUNICATION == 1
			printf("%d %s %d: hledam na zaklade %u %u %u %u\n", (unsigned int)pthread_self(), __FILE__, __LINE__, interceptId->key.items.port_1, interceptId->key.items.port_2, interceptId->key.items.ipver , interceptId->key.items.proto);
		#endif
		
		// vyhladanie toku v ulozisku
		flowRecord = iri_iif_find_flow_record( &( interceptId->key ) );
		
		// tok na ktory prisla odpoved sa nachadza v hash tabulke
		if(flowRecord != NULL &&
		// prvek nie je zmazany v prvej faze
			flowRecord->additionalData != NULL )
		{
			// poznacim si ze uz nemusim cekat na odpoved pro danu spravu
			flowRecord->additionalData->modules_waitingForReplies = flowRecord->additionalData->modules_waitingForReplies - 1;
			
			// odoslanie dat z ADB do INI3
			iri_iif_adb_send_to_ini3( flowRecord , interceptId->rid);
			
			// dalej uz neukladat do ADB
			flowRecord->additionalData->adb_store = 0;
			
			// pokial neocakavam odpovede na dalsie spravy
			if( flowRecord->additionalData->modules_waitingForReplies == 0 &&
			// pokial tok uz skoncil (este pred prijatim odpovede) tak uz ho mozem vymazat
				flowRecord->timeDelete != 4294967295 )
			{
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
					printf("%d %s %d: odstranim zaznam pretoze sa cekalo na prijati poslednej odpovede\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
				#endif
				
				// oznacenie zaznamu pre zmazanie + odstranenie ADB, TCPr a oznamenie modulom o ukonceni toku
				iri_iif_hash_table_destroy_record_phase_1( flowRecord, 0 );
			}
		}
        #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
        else
        {
            printf("%d %s %d: tok na kery dosel odposlech z filterd sa nenachadza v ulozisku\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
        }
        #endif
	}
}


void iri_iif_adb_check_iterfaces( )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: iri_iif_adb_check_iterfaces()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// precteni pozadavku z filter demona
    iri_iif_check_filterd_interface();
    
	// precteni pozadavku ze spravy adb
	iri_iif_check_iri_reply_interface();
}

void iri_iif_adb_init( int input_threadId, itc_ptr input_interfaceFromFilterD, itc_ptr input_interfaceFromIriCore )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_ADB == 1
		printf("%d %s %d: iri_iif_adb_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// ulozenie ukazatelov na rozhrania
    interfaceFromFilterD = input_interfaceFromFilterD;
	interfaceFromIriCore = input_interfaceFromIriCore;
}

