#! /usr/bin/env python

import json
import re
import os
pathDirectory = ""
settingsFileName = "settings"

def filter_vectors_include( data ):
	f = open(pathDirectory + 'filter-vectors.include', 'w')
	
	index = 1
	for line in data:
		if line["enabled"] == "yes":
			for protocol in line["protocols"]:
				for port in protocol["ports"]:
					if protocol["name"] == "TCP":
						f.write( "tcpPortMap["+str(port)+"] = tcpPortMap["+str(port)+"] + "+str(index)+";\n")
					if protocol["name"] == "UDP":
						f.write( "udpPortMap["+str(port)+"] = udpPortMap["+str(port)+"] + "+str(index)+";\n")
			index = index * 2
	
	f.close()

def modules_init_include( data ):
	f = open(pathDirectory + 'modules-init.include', 'w')
	
	index = 0
	for line in data:
		if line["enabled"] == "yes":
			#f.write( "" + line["name"] + "_init(input_iriCoreSocket);\n")
			f.write( "module_process_packet_functions[" + str(index) + "] = &" + line["name"] + "_process_packet;\n")
			f.write( "module_destroy_data_functions[" + str(index) + "] = &" + line["name"] + "_destroy_data;\n")
			for protocol in line["protocols"]:
				if protocol["name"] == "TCP":
					f.write( "module_process_end_of_flow[" + str(index) + "] = &" + line["name"] + "_process_end_of_flow;\n")
			index = index + 1
	
	f.write( "\n")
	f.close()

def modules_includes_include( data, slisData ):
	f = open(pathDirectory + 'modules-includes.include', 'w')
	
	for line in data:
		if line["enabled"] == "yes":
			f.write( "#include \"../" + line["name"] + "/" + line["name"] + ".h\"\n")
	
	f.write( "\n")
	
	f.write( '#define MODULES_CONNECTION_TO_IRI_CORE_IP "' + str(slisData["IP"]) + '"\n')
	f.write( '#define MODULES_CONNECTION_TO_IRI_CORE_PORT "' + str(slisData["port"]) + '"\n')
	
	f.write( "\n")
	f.close()

def tcpreassembling_includes_include( data ):
	f = open(pathDirectory + 'tcpreassembling-defines.include', 'w')
	
	f.write( "#define TCP_REASSEMBLING_OLD_GAP_IN_SECONDS " + str( data["difference-in-seconds"] ) + "\n")
	f.write( "#define TCP_REASSEMBLING_OLD_GAP_IN_BYTES " + str( data["difference-in-seqnum"] ) + "\n")
	
	f.write( "\n")
	f.close()

def shared_defines_include( modulesData , itcData, hashTableData, pbData ):
	f = open(pathDirectory + 'shared-defines.include', 'w')
	
	moduleCount = 0
	for line in modulesData:
		if line["enabled"] == "yes":
			moduleCount = moduleCount + 1
	
	f.write( "#define MODULE_COUNT " + str(moduleCount) + "\n")
	
	f.write( "#define ITC_RECORDS_PER_CIRCLE " + str( itcData["records-per-circle"] ) + "\n")
	
	f.write( "#define HASH_TABLE_SECONDS_IN_PHASE_1 " + str( hashTableData["seconds-in-phase-1"] ) + "\n")
	f.write( "#define HASH_TABLE_CLEAN_UP_NUMBER_OF_RECORDS " + str( hashTableData["clean-up-number-of-records"] ) + "\n")
	f.write( "#define HASH_TABLE_CLEAN_UP_INACTIVE_TIMEOUT " + str( hashTableData["clean-up-inactive-timeout"] ) + "\n")
	f.write( "#define HASH_TABLE_SIZE_OF_THE_TABLE " + str( hashTableData["size-of-the-table"] ) + "\n")
	f.write( "#define HASH_TABLE_SIZE_OF_THE_STACK " + str( hashTableData["size-of-the-stack"] ) + "\n")
	
	f.write( "#define PACKET_BUFFER_BIG_FLOW_LEVEL " + str( pbData["big-flow-level"] ) + "\n")
	f.write( "#define PACKET_BUFFER_ALMOST_FULL_LEVEL_1 " + str( pbData["almost-full-level-1"] ) + "\n")
	f.write( "#define PACKET_BUFFER_ALMOST_FULL_LEVEL_2 " + str( pbData["almost-full-level-2"] ) + "\n")
	f.write( "#define PACKET_BUFFER_DELETE_PACKETS " + str( pbData["delete-packets"] ) + "\n")
	
	f.write( "\n")
	f.close()


if __name__ == "__main__":
	#global pathDirectory
	print "Processing of the settings file..."
	print os.getcwd()
	
	# otvorim si subor kde su vsetky nastavenia
	try:
		filePointer = open( settingsFileName, "r" )
	except:
		pathDirectory = "configs/"
		filePointer = open( pathDirectory + settingsFileName, "r" )
	
	# pomocou premennej budem vediet v ktorej casti konfiguraku sa nachadzam
	configPart = 0
	
	# inicializujem si data pre jednotlive casti konfigu
	modulesData = []
	itcData = []
	pbData = []
	hashTableData = {}
	tcpReassData = {}
	slisData = []
	
	# subor zacnem spracovavat po riadkoch
	for line in filePointer:
		# zahodim radkovani na konci radku
		line = line.strip()
		
		# prazdne a zakomentovane radky ignorujem
		if len(line) > 2 and line[0] != "#":
			# kontrola ci neprechadzam do inej casti konfigu
			if line == "MODULES:":
				configPart = 1
			elif line == "SLIS:":
				configPart = 6
			elif line == "HASH-TABLE:":
				configPart = 2
			elif line == "ITC:":
				configPart = 3
			elif line == "PACKET-BUFFER:":
				configPart = 4
			elif line == "TCP-REASSEMBLING:":
				configPart = 5
			else:
				# cast moduly
				if configPart == 1:
					decodedLine = json.loads(line)
					modulesData.append(decodedLine)
				# cast hash tabulka
				elif configPart == 2:
					decodedLine = json.loads(line)
					hashTableData = dict( hashTableData.items() + decodedLine.items() )
				elif configPart == 3:
					itcData = json.loads(line)
				elif configPart == 4:
					pbData = json.loads(line)
				elif configPart == 5:
					tcpReassData = json.loads(line)
				elif configPart == 6:
					slisData = json.loads(line)
	
	# zatvorenie suboru
	filePointer.close()
	
	# z nacitanych dat vygenerujem prislusne subory
	filter_vectors_include( modulesData )
	modules_init_include( modulesData )
	modules_includes_include( modulesData, slisData )
	tcpreassembling_includes_include( tcpReassData )
	shared_defines_include( modulesData , itcData, hashTableData, pbData )

