/*
 * Soubor:  iri-iif-controller.c
 * Datum:   27.05.2014
 * Autor:   Stanislav Barta, xbarta29@stud.fit.vutbr.cz
 * Projekt: IRI-IIF pro SDM
 * Popis:   soubor obsahujici funkce tvorici rozhrani mezi CC-IIF a IRI-IIF a
 *          funkce vykonavajici cinnost IRI-IIF kontroleru
 */

#include "controller.h"

extern __thread int threadId;

/*
 * inicializacni funkce kterou je nutno zavolat po vytvoreni vlakna pred zpracovanim prvniho paketu
 * @param int hodnota urcujici velikost hashovaci tabulky
 * @param int socket na ktery se budou zasilat vysledne zpravy IRI
 * @param itc_ptr rozhrani pro spravu ADB ze strany filtr demona
 * @param itc_ptr rozhrani pro spravu ADB ze strany komponenty sprava ADB
 * @return int hodnota urcujici uspesnost inicializace
 */
int iri_iif_init( int input_flowsHashTableSize, int input_iriCoreSocket, itc_ptr input_interfaceFromFilterD, itc_ptr input_interfaceFromIriCore , int input_threadId )
{
	setbuf(stdout, NULL);
    
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: iri_iif_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
    int output_returnCode;
    
    // ulozenie cisla vlakna pre neskorsie znackovanie IRI sprav
    threadId = input_threadId;
    
    // inicializacia jednotlivych casti systemu
    output_returnCode = iri_iif_hash_table_init( HASH_TABLE_SIZE_OF_THE_TABLE , HASH_TABLE_SIZE_OF_THE_STACK);
    iri_iif_adb_init( input_threadId, input_interfaceFromFilterD, input_interfaceFromIriCore );
    iri_iif_filter_init( input_threadId );
    iri_iif_modules_init( input_iriCoreSocket );
    iri_iif_packet_buffer_init();
    
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: aktualne vlakno ma index %d\n", (unsigned int)pthread_self(), __FILE__, __LINE__, input_threadId);
	#endif
	
	return output_returnCode;
}


/*
 * funkce ktera pripravi strukturu iri_iif_offsets_t pro predani do modulu a vyuziti pri skladani tcp toku
 * @param iri_iif_offsets_t * ukazatel na strukturu ktera je vyplnovana hodnotami
 * @param offsets_t * ukazatel na srukturu offsetů získaných z filt deamonu
 * @param uint8_t * ukazatel na data paketu pro dopocitani dalsich udaju
 */
void prepare_iri_iif_offsets(iri_iif_offsets_t *iri_offsets, offsets_t *offsets, uint8_t *payload)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: prepare_iri_iif_offsets()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	int ip_payload_size = 0;
	
	uint8_t *protocol = (payload+((offsets->protocol)));
    
	iri_offsets->ip_ver = offsets->ip_ver;
	iri_offsets->iph_start = offsets->iph_start;
	iri_offsets->ip_start = offsets->ip_start;
	iri_offsets->l4_start = offsets->l4_start;
	iri_offsets->protocol = offsets->protocol;
	
	if( *(payload + iri_offsets->protocol) == TCP_PROTOCOL )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
			printf("%d %s %d: jedna sa o TCP protokol\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		struct tcphdr *tcp_header = (struct tcphdr *) (payload + iri_offsets->l4_start);
		
		// vypocitani velikosti tcp hlavicky v bajtech
		int tcp_header_size = ( payload[iri_offsets->l4_start + 12] >> 4 ) * 4;
        printf("TODO 75: %d %d %d %d %d \n", offsets->ip_ver, offsets->iph_start, offsets->ip_start, offsets->l4_start, offsets->protocol);
        printf("TODO 80: %d %d;%d %d %d \n", iri_offsets->l4_start, iri_offsets->l4_start + 12, payload[iri_offsets->l4_start + 12], ( payload[iri_offsets->l4_start + 12] >> 4 ), ( payload[iri_offsets->l4_start + 12] >> 4 ) * 4);
		
		// urceni delky dat v ip (pro urceni delky payloadu v tcp) -> nelze pouzit celkovou delku kvuli ethernet padding
		if(offsets->ip_ver == 4)
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
				printf("%d %s %d: jedna sa o IPv4 protokol\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			struct ip *ipv4_packet = (struct ip *) (payload + offsets->iph_start);
			ip_payload_size = ntohs(ipv4_packet->ip_len) - ((ipv4_packet->ip_hl)*4);
		}
		else if(offsets->ip_ver == 6)
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
				printf("%d %s %d: jedna sa o IPv6 protokol\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			struct ip6_hdr *ipv6_packet = (struct ip6_hdr *) (payload + iri_offsets->iph_start);
			ip_payload_size = ntohs(ipv6_packet->ip6_plen);
		}
		
		// prirazeni hodnot do struktury podle tcp dat
		iri_offsets->data_offset = offsets->l4_start + tcp_header_size;
		iri_offsets->data_length = ip_payload_size - tcp_header_size;
        printf("TODO 100: %d %d (%d %d %d %d)\n", iri_offsets->data_offset, iri_offsets->data_length, offsets->l4_start , tcp_header_size, ip_payload_size , tcp_header_size);
	}
	else if( *(payload + iri_offsets->protocol) == UDP_PROTOCOL)
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
			printf("%d %s %d: jedna sa o UDP protokol\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// prirazeni do struktury podle udp dat
		iri_offsets->data_offset = iri_offsets->l4_start + UDP_HEADER_LENGTH;
		iri_offsets->data_length = (( *((uint8_t *)(payload + iri_offsets->l4_start) + 4) << 8 ) + *((uint8_t *)(payload + iri_offsets->l4_start) + 5)) - UDP_HEADER_LENGTH;
	}
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
	else
	{
		printf("%d %s %d: nejedna sa o TCP/UDP protokol\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	}
	#endif
	
}

/*
 * funkce pro zpracovani noveho paketu
 * @param flow_identifier_t * ukazatel na strukturu obsahujici predpocitane hodnoty hash a klic ze ktereho byly pocitany
 * @param offsets_t * ukazatel na strukturu predpocitanych offsetu
 * @param ini3payload_t * ukazatel na strukturu obsahujici hlavicku load_head_t a payload paketu
 * @return int hodnota urcujici zda je o tok zajem. 0 znamena filtrovat a 1 ze je zajem tok zpracovavat
 */

int iri_iif_process_new_packet(flow_identifier_t *input_hashAndKey, offsets_t *input_offsets, ini3payload_t *input_packet)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("\n%d :: Spracovanie paketu v IRI-IIF\n", (unsigned int)pthread_self());
	#endif
    
	iri_iif_hash_table_item_t *item = NULL; // zaznam v hashovaci tabulke
	iri_iif_offsets_t iriOffsets; // zistenie offsetov, ktore zatial nie su dostupne z filterd
	unsigned short int modulesVector; // premenna pre vektor modulov, ktore maju o dany tok zaujem
	
	// osetrenie paketov vacsich ako je miesto v packet buffery
	if ( input_packet->parts.header.blob_size >= 1600 )
		return RETAIN_FLOW;
	
	
	// blok ADB
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: Spustenie bloku ADB\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// kontrola odpovedi z IRI-core a filterd na IRI spravy
    iri_iif_adb_check_iterfaces( );
    
    
	// kontrola neaktivnych tokov a tokov oznacenych pre zmazanie
	iri_iif_hash_table_clean_up( input_packet->parts.header.timestamp_sec );
	
    
	// kontrola fake paketov
	if ( input_offsets == NULL )
		return RETAIN_FLOW;
	
    
	// prehodeni big a little endian
    if (input_offsets->ip_ver == 4)
    {
        input_hashAndKey->key->items.ip_1.v4_long.data.d32[0] =  htobe32( input_hashAndKey->key->items.ip_1.v4_long.data.d32[0] );
        input_hashAndKey->key->items.ip_2.v4_long.data.d32[0] =  htobe32( input_hashAndKey->key->items.ip_2.v4_long.data.d32[0] );
	}
    else if (input_offsets->ip_ver == 6)
    {
        uint32_t temp0, temp1, temp2, temp3;
        temp0 = input_hashAndKey->key->items.ip_1.v6.d32[0];
        temp1 = input_hashAndKey->key->items.ip_1.v6.d32[1];
        temp2 = input_hashAndKey->key->items.ip_1.v6.d32[2];
        temp3 = input_hashAndKey->key->items.ip_1.v6.d32[3];
        input_hashAndKey->key->items.ip_1.v6.d32[0] = htonl(temp3);
        input_hashAndKey->key->items.ip_1.v6.d32[1] = htonl(temp2);
        input_hashAndKey->key->items.ip_1.v6.d32[2] = htonl(temp1);
        input_hashAndKey->key->items.ip_1.v6.d32[3] = htonl(temp0);
        
        temp0 = input_hashAndKey->key->items.ip_2.v6.d32[0];
        temp1 = input_hashAndKey->key->items.ip_2.v6.d32[1];
        temp2 = input_hashAndKey->key->items.ip_2.v6.d32[2];
        temp3 = input_hashAndKey->key->items.ip_2.v6.d32[3];
        input_hashAndKey->key->items.ip_2.v6.d32[0] = htonl(temp3);
        input_hashAndKey->key->items.ip_2.v6.d32[1] = htonl(temp2);
        input_hashAndKey->key->items.ip_2.v6.d32[2] = htonl(temp1);
        input_hashAndKey->key->items.ip_2.v6.d32[3] = htonl(temp0);
    }
    
    
	
    // blok filter
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: Spustenie bloku FILTER\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// zistenie vektoru modulov
	modulesVector = iri_iif_filter_flow( input_hashAndKey );
	
	// ked je vektor nulovy (ziadny modul nema zaujem), je hodnota premennej 0
    if( modulesVector == NOBODY_WANTS_TO_ANALYZE_THIS_FLOW )
    {
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
			printf("%d %s %d: Blokovanie toku, pretoze ziadny aplikacny modul nema o dany protokol/port zaujem\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		return BLOCK_FLOW;
	}
	
	
	// blok hash-table
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: Spustenie bloku HASH-TABLE\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// vyhladanie zaznamu v hash tabulke
	item = iri_iif_hash_table_find_or_create_record( input_hashAndKey, modulesVector );
	if( item == NULL) // hodnota NULL sa vrati ak bol dany tok zmazany
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
			printf("%d %s %d: Blokovanie toku, pretoze tok je v tabulke oznaceny ako vymazany\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		return BLOCK_FLOW;
	}
	
	
	// vlozenie casovej znacky z paketu do policka oznamujuce cas posledneho spracovavneho paketu
	item->timeLastPacket = input_packet->parts.header.timestamp_sec;
	
	
	// blok packet buffer
	iri_iif_packet_buffer_update_cache( NULL);  // oznacenie ze paket zatial nebol ulozeny
	
	// docasny blok pre vypocet offsetu
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: Spustenie bloku OFFSET\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// dopocitam ukazatel na data a velkost aplikacnych dat
	prepare_iri_iif_offsets(&iriOffsets, input_offsets, input_packet->parts.data);
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: hledam na zaklade: %u %u %d %d\n", (unsigned int)pthread_self(), __FILE__, __LINE__,
		input_hashAndKey->key->items.port_1,
		input_hashAndKey->key->items.port_2,
		input_hashAndKey->key->items.ipver,
		input_hashAndKey->key->items.proto);
	#endif
	
	// jedna sa o tok protokolu TCP
	if( input_hashAndKey->key->items.proto == TCP_PROTOCOL)
	{
		// blok TCP reassembling
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
			printf("%d %s %d: Spustenie bloku TCP-REASSEMBLING\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		char reassemblingStatus;
		packetBufferItem_ptr deleteItem = NULL;
		char flowDirection = 0;
		
		// skontrolujem ci je packet v poradi a ked ne tak ho ulozim
		reassemblingStatus = iri_iif_tcp_reassembling_process_new_packet( item, &iriOffsets, &input_packet, &deleteItem, &flowDirection );
		
		// odeslem pakety kere su v poradi
		while( reassemblingStatus == TCP_REASSEMBLING_SEND_TO_MODULES )
		{
			// blok aplikacne moduly
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
				printf("%d %s %d: Spustenie bloku MODULY\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			
			// odoslem paket modulom, ktore o neho maju zaujem a z navratovej hodnoty zistim novy vektor modulov
			modulesVector = iri_iif_modules_send_packet( item, &iriOffsets, input_packet);
			
			// vektor modulov je nulovy, nikto uz nema zaujem spracovavat dany tok
			if ( modulesVector == NOBODY_WANTS_TO_ANALYZE_THIS_FLOW )
			{
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
					printf("%d %s %d: Blokovanie toku, pretoze ziadny aplikacny modul o neho nema zaujem\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
				#endif
				
				// zmazat record v tabulke (prva faza)
				iri_iif_hash_table_destroy_record_phase_1( item, 1 );
				
				return BLOCK_FLOW;
			}
			
			// skontrolujem ci sa v TCP nenachadza dalsi paket v poradi
			reassemblingStatus = iri_iif_tcp_reassembling_check_for_more_packets( item, &iriOffsets, &input_packet, &deleteItem, flowDirection );
		}
		
		// ak sme zachytili ukoncenie spojenia, oznamime to modulom
		if( reassemblingStatus == TCP_REASSEMBLING_END_OF_THE_FLOW )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
				printf("%d %s %d: TCP reassembling detekovat ukoncenie toku\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			
			// oznameni modulom ze skoncil tok (je to nutne ked nasleduje destroy_phase_1)
			iri_iif_modules_notice_end_of_flow( item );
			
			// zmazat record v tabulke (prva faza)
			iri_iif_hash_table_destroy_record_phase_1( item, 0 );
			
			return BLOCK_FLOW;
		}
	}
	// NEjedna sa o tok protokolu TCP
	else
	{
		// blok aplikacne moduly
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
			printf("%d %s %d: Spustenie bloku MODULY\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// odoslem paket modulom, ktore o neho maju zaujem a z navratovej hodnoty zistim novy vektor modulov
		modulesVector = iri_iif_modules_send_packet( item, &iriOffsets, input_packet);
		
		// vektor modulov je nulovy, nikto uz nema zaujem spracovavat dany tok
		if ( modulesVector == NOBODY_WANTS_TO_ANALYZE_THIS_FLOW )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
				printf("%d %s %d: Blokovanie toku, pretoze ziadny aplikacny modul o neho nema zaujem\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			
			// zmazat record v tabulke (prva faza)
			iri_iif_hash_table_destroy_record_phase_1( item, 1 );
			
			return BLOCK_FLOW;
		}
	}
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: Koniec spracovavania paketu, ponechanie toku, vektor modulov %d\n", (unsigned int)pthread_self(), __FILE__, __LINE__, item->additionalData->modules_vector2);
	#endif
	
	return RETAIN_FLOW;
}

/*
 * funkce pro kompletni zruseni IRI-IIF dat pri ukonceni aplikace
 */
void iri_iif_destroy()
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_CONTROLLER == 1
		printf("%d %s %d: iri_iif_destroy()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// odstranim celu hash tabulku (aj data co maju ulozene moduly)
	iri_iif_hash_table_destroy();
	
	// odstranim plny aj prazdny packet buffer
	iri_iif_packet_buffer_destroy();
	
	printf("IRI-IIF done\n");
}
