
#define FILTER_FLOWS_BY_PORT_NUMBERS

#include "filter.h"

// dve riedke polia pre vyhladanie vektoru modulov, indexom je cislo portu a hodnota je vektor
unsigned short int tcpPortMap[65536];
unsigned short int udpPortMap[65536];


void iri_iif_filter_init( char input_threadId )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_FILTER == 1
		printf("%d %s %d: iri_iif_filter_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
    // inicializaciu je nutne spustit iba z jedneho vlakna
    if( input_threadId != 0 )
    {
        return;
    }
    
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_FILTER == 1
		printf("%d %s %d: jedna sa o vlakno s ID=0\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	unsigned int i; // pomocna premenna pre cislo portu
	
	// prednastavim si cely rozsah portov
	for( i = 0; i < 65536; i++)
	{
		// najprv nastavim vsetky hodnoty na 0 -> ziadny modul nema zaujem o dany port
		tcpPortMap[i] = 0;
		udpPortMap[i] = 0;
	}
	
	// subor aktualizuje hodnoty premennych tcpPortMap/udpPortMap, pricom nastavuje bitovu masku ktore moduly maju zaujem
	#include "configs/filter-vectors.include"
}

/*
 * funkce pro kontrolu zda existuje nejaky protokol, kery dany tok analyzuje a kdyz ne, tok muzeme ignorovat
 * @param flow_identifier_t * ukazatel na strukturu obsahujici predpocitane hodnoty hash a klic ze ktereho byly pocitany
 * @return int vraci 1 pokud je o protokol zajem jinak vraci 0
 */
unsigned short int iri_iif_filter_flow( flow_identifier_t *input_hashesAndKey )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_FILTER == 1
		printf("%d %s %d: iri_iif_filter_flow()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// premenna obsahujuca vektor modulov, ktore maju zaujem o tok
	unsigned short int output_modulesMap = 0;

	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_FILTER == 1
	printf("%d %s %d: hodnota %d\n", (unsigned int)pthread_self(), __FILE__, __LINE__, input_hashesAndKey->key->items.proto);
	#endif
	// moduly si mozu ziadat len o porty z protokolov TCP a UDP
	if( input_hashesAndKey->key->items.proto == TCP_PROTOCOL )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_FILTER == 1
			printf("%d %s %d: TCP_PROTOCOL\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
	
		// pomocou bitovej operacie OR spojim vektory zo zdrojoveho a cieloveho portu zaroven
		output_modulesMap = tcpPortMap[ input_hashesAndKey->key->items.port_1 ] | tcpPortMap[ input_hashesAndKey->key->items.port_2 ];
	}
	else if( input_hashesAndKey->key->items.proto == UDP_PROTOCOL )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_FILTER == 1
			printf("%d %s %d: UDP_PROTOCOL\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// pomocou bitovej operacie OR spojim vektory zo zdrojoveho a cieloveho portu zaroven
		output_modulesMap = udpPortMap[ input_hashesAndKey->key->items.port_1 ] | udpPortMap[ input_hashesAndKey->key->items.port_2 ];
	}
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_FILTER == 1
		printf("%d %s %d: vysledny vektor je %u\n", (unsigned int)pthread_self(), __FILE__, __LINE__, output_modulesMap);
	#endif
	
	return output_modulesMap;
}
