
#include "header-files.h"

#ifndef _IRI_IIF_FAST_HASH_TABLE_
#define _IRI_IIF_FAST_HASH_TABLE_
#include "../fast_hash_table/fast_hash_table.h"
#endif

__thread fht_table_t * flowsHashTable; // struktura hashovaci tabulky, lokalni pro dane vlakno
__thread fht_iter_t * cleanUpIterator; // iterator ktory postupne bude prechadzat celu tabulku a cistit tabulku


int iri_iif_hash_table_init( int input_flowsHashTableSize, int input_flowsStashSize )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: iri_iif_hash_table_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// inicializacia hlavnej struktury pre hash tabulku
	flowsHashTable = fht_init( input_flowsHashTableSize, sizeof(item_key_t), sizeof(iri_iif_hash_table_item_t), input_flowsStashSize );
	if( flowsHashTable == NULL )
	{
		return IRI_IIF_RETURN_GENERAL_ERROR;
	}
	
	// inicializacia iteratoru pre cistenie tabulky
	cleanUpIterator = fht_init_iter( flowsHashTable );
	
    printf("%u    Hash table: %p, Iterator: %p\n", (unsigned) pthread_self(), flowsHashTable, cleanUpIterator);
    
	return IRI_IIF_RETURN_OK;
}

void iri_iif_hash_table_new_record_init( iri_iif_hash_table_item_t *input_newItem, flow_identifier_t *input_hashesAndKey, unsigned short int input_modulesMap )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: iri_iif_hash_table_new_record_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	input_newItem->key = *(input_hashesAndKey->key);
	
	// do casovej znacky reprezentujucej kedy sa ma dany tok zmazat nastavim cas ktory nikdy nenastane
	input_newItem->timeDelete = 4294967295; // vlozenie maximalnej moznej hodnoty na 32 bitoch
	
	//////////////////////////////////////////////// blok packet buffer ////////////////////////////////////////////////
	input_newItem->additionalData->pb_storedBytes = 0;
	/////////////////////////////////////////////////// blok modules ///////////////////////////////////////////////////
	input_newItem->additionalData->modules_vector2 = input_modulesMap;
	for(int i = 0 ; i < MODULE_COUNT ; i++)
	{
		input_newItem->additionalData->modules_data[i] = NULL;
	}
	input_newItem->additionalData->modules_waitingForReplies = 0;
	///////////////////////////////////////////////////// blok ADB /////////////////////////////////////////////////////
	input_newItem->additionalData->adb_store = 1;
	input_newItem->additionalData->adb_first = NULL;
	input_newItem->additionalData->adb_last = NULL;
	///////////////////////////////////////////////// TCP reassembling /////////////////////////////////////////////////
	input_newItem->additionalData->tcpr_seqNumB = 0; // priradenie hodnoty 0 nema ziadny vyznam, len to utisi valgrind
	input_newItem->additionalData->tcpr_firstPacketB = NULL;
	input_newItem->additionalData->tcpr_lastPacketB = NULL;
	input_newItem->additionalData->tcpr_stateB = TCP_STATE_INIT;
	input_newItem->additionalData->tcpr_timestampSecB = 0;
	
	input_newItem->additionalData->tcpr_seqNumS = 0; // priradenie hodnoty 0 nema ziadny vyznam, len to utisi valgrind
	input_newItem->additionalData->tcpr_firstPacketS = NULL;
	input_newItem->additionalData->tcpr_lastPacketS = NULL;
	input_newItem->additionalData->tcpr_stateS = TCP_STATE_INIT;
	input_newItem->additionalData->tcpr_timestampSecS = 0;
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}

iri_iif_hash_table_item_t * iri_iif_hash_table_create_new_record( flow_identifier_t *input_hashesAndKey, unsigned short int input_modulesMap )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: iri_iif_hash_table_create_new_record()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	iri_iif_hash_table_item_t *newItem; // pomocna struktura do kerej sa nakopiruju secky data kere sa potom vloza do tabulky
	iri_iif_hash_table_item_t *deletedItem; // struktura pre ulozenie ukazatela na data, ktore su z hash tabulky odstranene
	
	// alokacia noveho prvku
	newItem = (iri_iif_hash_table_item_t*) malloc(sizeof(iri_iif_hash_table_item_t));
	newItem->additionalData = (iri_iif_hash_table_additional_data_t*) malloc(sizeof(iri_iif_hash_table_additional_data_t));
	
	// alokacia miesta pre zmazany prvok
	deletedItem = (iri_iif_hash_table_item_t*) malloc(sizeof(iri_iif_hash_table_item_t));
	
	// inicializacia hodnot noveho prvku
	iri_iif_hash_table_new_record_init( newItem, input_hashesAndKey, input_modulesMap );
	
	
	
	// vlozenie prvku do hash tabulky (pripadne do pretokoveho seznamu)
	int returnCode = fht_insert_with_stash(flowsHashTable, (char *) input_hashesAndKey->key, newItem, NULL, deletedItem);
	if( returnCode == FHT_INSERT_STASH_LOST)
	{
        char key_ip_1_str[INET6_ADDRSTRLEN]; // pomocne premenna pre skonvertovanie IP adresy do textovej podoby
        char key_ip_2_str[INET6_ADDRSTRLEN]; // podla Linux/include/linux/inet.h INET6_ADDRSTRLEN = 48B
    
        if(deletedItem->key.items.ipver == 4)
        {
            inet_ntop(AF_INET, &(deletedItem->key.items.ip_1.v4_long.data.d8), key_ip_1_str, INET6_ADDRSTRLEN);
            inet_ntop(AF_INET, &(deletedItem->key.items.ip_2.v4_long.data.d8), key_ip_2_str, INET6_ADDRSTRLEN); 
        }
        else
        {
            inet_ntop(AF_INET6, &(deletedItem->key.items.ip_1.v6.d8), key_ip_1_str, INET6_ADDRSTRLEN);
            inet_ntop(AF_INET6, &(deletedItem->key.items.ip_2.v6.d8), key_ip_2_str, INET6_ADDRSTRLEN);
        }
        
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
			printf("%d %s %d: hash tabulka je plna, musim odstranit nastarsi zaznam v stacku - %d, \"%s\", %d, \"%s\", %d\n", (unsigned int)pthread_self(), __FILE__, __LINE__, deletedItem->key.items.proto, key_ip_1_str, deletedItem->key.items.port_1, key_ip_2_str, deletedItem->key.items.port_2);
		#endif
		
		// ak este nezacal byt mazany
		if( deletedItem->additionalData != NULL )
		{
			iri_iif_hash_table_destroy_record_phase_1( deletedItem, 1 );
		}
		
		iri_iif_hash_table_destroy_record_phase_2( deletedItem );
	}
    #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
	else if( returnCode == FHT_INSERT_FAILED)
	{
		printf("%d %s %d: FHT_INSERT_LOST 3\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	}
    #endif

	
	// uvolnenie struktury ktora sa skopirovala do hash tabulky a struktury pre vyhodeny zaznam
	free( newItem );
	free( deletedItem );
	
	// po vytvoreni zaznamu musim ziskat odkaz na nu vyhladanim
	newItem = (iri_iif_hash_table_item_t *) fht_get_data_with_stash( flowsHashTable, (char *) input_hashesAndKey->key);
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: vytvoreny novy zaznam na %p\n", (unsigned int)pthread_self(), __FILE__, __LINE__, newItem);
	#endif
	
	return newItem;
}

iri_iif_hash_table_item_t * iri_iif_find_flow_record( item_key_t *key )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: iri_iif_find_flow_record()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		if( key != NULL )
        {
            char key_ip_1_str[INET6_ADDRSTRLEN]; // pomocne premenna pre skonvertovanie IP adresy do textovej podoby
            char key_ip_2_str[INET6_ADDRSTRLEN]; // podla Linux/include/linux/inet.h INET6_ADDRSTRLEN = 48B
            
            if (key->items.ipver == 4)
            {
                inet_ntop(AF_INET, &(key->items.ip_1.v4_long.data.d8), key_ip_1_str, INET6_ADDRSTRLEN);
                inet_ntop(AF_INET, &(key->items.ip_2.v4_long.data.d8), key_ip_2_str, INET6_ADDRSTRLEN);
            }
            else if (key->items.ipver == 6)
            {
                inet_ntop(AF_INET6, &(key->items.ip_1.v6.d8), key_ip_1_str, INET6_ADDRSTRLEN);
                inet_ntop(AF_INET6, &(key->items.ip_2.v6.d8), key_ip_2_str, INET6_ADDRSTRLEN); 
            }
            
			printf("%d %s %d: hledam na zaklade: %d, \"%s\", %d, \"%s\", %d\n", (unsigned int)pthread_self(), __FILE__, __LINE__, key->items.proto, key_ip_1_str, key->items.port_1, key_ip_2_str, key->items.port_2);
        }
		else
			printf("%d %s %d: hledam na zaklade: NULL\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	iri_iif_hash_table_item_t * output;
	
	// vyhladanie prvku v hash tabulke (pretokovom sezname)
	output = (iri_iif_hash_table_item_t *) fht_get_data_with_stash( flowsHashTable, (char *) key);

	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		if( output == NULL )
			printf("%d %s %d: polozka nenalezena -- menim klic\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		else
			printf("%d %s %d: polozka nalezena\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// v pripade, ze sa nenasiel zaznam, prehodim kluc a opakujem este raz
	if( output == NULL)
	{
		uint16_t port;
		port = key->items.port_1;
		key->items.port_1 = key->items.port_2;
		key->items.port_2 = port;
		
        uint32_t ip;
        if(key->items.ipver == 4)
        {
            ip = key->items.ip_1.v4_long.data.d32[0];
            key->items.ip_1.v4_long.data.d32[0] = key->items.ip_2.v4_long.data.d32[0];
            key->items.ip_2.v4_long.data.d32[0] = ip;
        }
        else
        {
            for(char i = 0; i < 4; i++)
            {
                ip = key->items.ip_1.v6.d32[i];
                key->items.ip_1.v6.d32[i] = key->items.ip_2.v6.d32[i];
                key->items.ip_2.v6.d32[i] = ip;
            }
        }
        
		output = (iri_iif_hash_table_item_t *) fht_get_data_with_stash( flowsHashTable, (char *) key);
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		if( output == NULL )
			printf("%d %s %d: polozka nenalezena\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	}
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: nasel sem zaznam - %p\n", (unsigned int)pthread_self(), __FILE__, __LINE__, output);
	#endif
	
	return output;
}

iri_iif_hash_table_item_t * iri_iif_hash_table_find_or_create_record( flow_identifier_t *input_hashesAndKey, unsigned short int input_modulesMap )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: iri_iif_hash_table_find_or_create_record()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	iri_iif_hash_table_item_t * output_record; // vysledna premenna do ktorej sa ulozi ukazatel na najdene/nenajdene data
	
	// vyhladanie prvku v hash tabulke (pretokovom sezname)
	output_record = iri_iif_find_flow_record( input_hashesAndKey->key );
	
	// novy tok ktery jeste neni v hashovaci tabulce
	if(output_record == NULL)
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
			printf("%d %s %d: Jedna sa o novy tok, kery este neni v hash tabulke\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// vytvorenie zaznamu v hash tabulke
		output_record = iri_iif_hash_table_create_new_record( input_hashesAndKey, input_modulesMap );
	}
	// prvek je zmazany v prvej fazy
	else if( output_record->additionalData == NULL )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
			printf("%d %s %d: prisiel paket pre zmazany tok (faza 1)\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// vratenim NULL hodnoty oznamujem ze tok sa nema spracovavat
		return NULL;
	}
	/////////////////////////////////////////////////// blok modules ///////////////////////////////////////////////////
	// ak ziadny modul nema o tok zaujem, nie je nutne tok dalej spracovavat
	else if ( output_record->additionalData->modules_vector2 == NOBODY_WANTS_TO_ANALYZE_THIS_FLOW )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
			printf("%d %s %d: ziadny modul nema o tok zaujem, nie je nutne tok dalej spracovavat\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// vratenim NULL hodnoty oznamujem ze tok sa nema spracovavat
		return NULL;
	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	return output_record;
}


// zaznam sa nemoze nachadzat ve fazi 1
void iri_iif_hash_table_destroy_record_phase_1( iri_iif_hash_table_item_t *item, char forced )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: iri_iif_hash_table_destroy_record_phase_1()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// zaznam uz je vymazany v prvej fazi
	if( item->additionalData == NULL )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
			printf("%d %s %d: zaznam uz je vymazany v prvej fazi\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		return;
	}
	// zaznam nemozem zmazat lebo sa ceka na IRI spravy
	else if( item->additionalData->modules_waitingForReplies != 0 && forced == 0 )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
			printf("%d %s %d: zaznam nemozem zmazat lebo sa ceka na %u odpovedi\n", (unsigned int)pthread_self(), __FILE__, __LINE__, item->additionalData->modules_waitingForReplies);
		#endif
		
		// zapisanie casovej znacky kedy sa ma zmazat packet
		item->timeDelete = item->timeLastPacket + HASH_TABLE_SECONDS_IN_PHASE_1; // v timeLastPacket je ulozena casova znacka aktualne spracovavaneho paketu
		
		return;
	}
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: mazem zaznam %p\n", (unsigned int)pthread_self(), __FILE__, __LINE__, item);
	#endif
	
	// odstranenie paketov z ADB
	iri_iif_adb_free( item );
	
	// odstranenie paketov z TCP reassemblingu
	iri_iif_tcp_reassembling_free( item );
	
	// oznamenie modulom, ze tok sa ide vymazat
	iri_iif_modules_destroy( item );
	
	// zapisanie casovej znacky kedy sa ma zmazat packet
	item->timeDelete = item->timeLastPacket + HASH_TABLE_SECONDS_IN_PHASE_1; // v timeLastPacket je ulozena casova znacka aktualne spracovavaneho paketu
	
	// uvolenie dat jednotlivych blokov (moduly si svoje data odstranili uz v prvej faze)
	free( item->additionalData );
	item->additionalData = NULL;
}

void iri_iif_hash_table_destroy_record_phase_2( iri_iif_hash_table_item_t *item )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: iri_iif_hash_table_destroy_record_phase_2()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// uvolenie dat jednotlivych blokov (moduly si svoje data odstranili uz v prvej faze)
	free( item->additionalData );
	
	// definitivne vymazanie prvku
	fht_remove_with_stash( flowsHashTable, (char *) &item->key);
}

void iri_iif_hash_table_print()
{
	printf("%d %s %d: iri_iif_hash_table_print()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
    
    fht_iter_t * iterator;
    iterator = fht_init_iter( flowsHashTable );
    char key_ip_1_str[INET6_ADDRSTRLEN]; // pomocne premenna pre skonvertovanie IP adresy do textovej podoby
    char key_ip_2_str[INET6_ADDRSTRLEN]; // podla Linux/include/linux/inet.h INET6_ADDRSTRLEN = 48B
    unsigned int i = 0;
    int returnCode;
    
    while(1)
    {
		// posuniem sa na dalsi prvok
		returnCode = fht_get_next_iter( iterator );
        
        if( returnCode != FHT_ITER_RET_OK )
            break;
        
        if(( (iri_iif_hash_table_item_t *) ( iterator->data_ptr ) )->key.items.ipver == 4)
        {
            inet_ntop(AF_INET, &(( (iri_iif_hash_table_item_t *) ( iterator->data_ptr ) )->key.items.ip_1.v4_long.data.d8), key_ip_1_str, INET6_ADDRSTRLEN);
            inet_ntop(AF_INET, &(( (iri_iif_hash_table_item_t *) ( iterator->data_ptr ) )->key.items.ip_2.v4_long.data.d8), key_ip_2_str, INET6_ADDRSTRLEN);
        }
        else
        {
            inet_ntop(AF_INET6, &(( (iri_iif_hash_table_item_t *) ( iterator->data_ptr ) )->key.items.ip_1.v6.d8), key_ip_1_str, INET6_ADDRSTRLEN);
            inet_ntop(AF_INET6, &(( (iri_iif_hash_table_item_t *) ( iterator->data_ptr ) )->key.items.ip_2.v6.d8), key_ip_2_str, INET6_ADDRSTRLEN);
        }
        
        printf("%d: %d, \"%s\", %d, \"%s\", %d\n", i, ( (iri_iif_hash_table_item_t *) ( iterator->data_ptr ) )->key.items.proto, key_ip_1_str, ( (iri_iif_hash_table_item_t *) ( iterator->data_ptr ) )->key.items.port_1, key_ip_2_str, ( (iri_iif_hash_table_item_t *) ( iterator->data_ptr ) )->key.items.port_2);
        
        i++;
    }
}

void iri_iif_hash_table_clean_up( uint32_t input_timestamp )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: iri_iif_hash_table_clean_up()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	unsigned int count; // pomocna premenna pre pocitadlo poctu skontrolovanych paketov
	uint32_t tempInactiveTimestamp; // premenna do ktorej si ulozim cas, ktory budem nasledne porovnavat s casmi v tokoch
    int returnCode;
	
	// ak bude cas zaznamu toku starsi ako cas vypocitany z aktualneho minus neaktivny timeout, bude sa jednat o neaktivny tok
	tempInactiveTimestamp = input_timestamp - HASH_TABLE_CLEAN_UP_INACTIVE_TIMEOUT;
	
	// prejdem N poloziek v hash tabulke
	for( count = 0; count < HASH_TABLE_CLEAN_UP_NUMBER_OF_RECORDS; count++)
	{
		// posuniem sa na dalsi prvok
		returnCode = fht_get_next_iter( cleanUpIterator );
		
		// prisiel som na konec tabulky, pri dalcom cykle zacnem od zaciatku
		if( returnCode != FHT_ITER_RET_OK )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
				printf("%d %s %d: prisiel som na koniec tabulky, obnovim iterator\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			
			fht_destroy_iter( cleanUpIterator );
			
			// inicializacia iteratoru pre cistenie tabulky
			cleanUpIterator = fht_init_iter( flowsHashTable );
            
			continue;
		}
        
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
        	        printf("%d %s %d: kontrola prvku: %d < %d, %p, %u < %u\n", (unsigned int)pthread_self(), __FILE__, __LINE__, ( (iri_iif_hash_table_item_t *) ( cleanUpIterator->data_ptr ) )->timeLastPacket, tempInactiveTimestamp, ( (iri_iif_hash_table_item_t *) ( cleanUpIterator->data_ptr ) )->additionalData, ( (iri_iif_hash_table_item_t *) ( cleanUpIterator->data_ptr ) )->timeDelete, input_timestamp);
	    #endif

		// nasiel sa neaktivny tok, ktoremu dlho neprisel zadny paket
		if( ( (iri_iif_hash_table_item_t *) ( cleanUpIterator->data_ptr ) )->timeLastPacket < tempInactiveTimestamp
		&&
		// a zaroven sa nejedna o tok, ktory uz je mazany
		( (iri_iif_hash_table_item_t *) ( cleanUpIterator->data_ptr ) )->additionalData != NULL )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
				 printf("%d %s %d: %u %u\n",  (unsigned int)pthread_self(), __FILE__, __LINE__, ( (iri_iif_hash_table_item_t *) ( cleanUpIterator->data_ptr ) )->timeLastPacket, tempInactiveTimestamp);
			#endif
			// prva faza zmazania prvku
			iri_iif_hash_table_destroy_record_phase_1( (iri_iif_hash_table_item_t *) ( cleanUpIterator->data_ptr ), 1 );
		}
		
		// aktualny cas je vacsi nez cas kedy sa ma dany zaznam vymazat
		if( ( (iri_iif_hash_table_item_t *) ( cleanUpIterator->data_ptr ) )->timeDelete < input_timestamp )
		{
                        #if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
                                 printf("%d %s %d: ",  (unsigned int)pthread_self(), __FILE__, __LINE__);
                        #endif
			// dokoncenie odstranenia prvku
			iri_iif_hash_table_destroy_record_phase_2( (iri_iif_hash_table_item_t *) ( cleanUpIterator->data_ptr ) );
		}
	}
}

void iri_iif_hash_table_destroy( )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_HASH_TABLE == 1
		printf("%d %s %d: iri_iif_hash_table_destroy()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	printf("%u    Hash table: %p, Iterator: %p\n", (unsigned) pthread_self(), flowsHashTable, cleanUpIterator);
	fht_iter_t * iterator; // premenna pre iteraciu v ramci hash tabulky
	
	// odstranenie iteratoru pre cistenie tabulky
	fht_destroy_iter( cleanUpIterator );
	
	// vytvorenie iteratoru pre prejdenie celej tabulky
	iterator = fht_init_iter( flowsHashTable );
	
	// prejdem kazdu polozku v hash tabulke
	while( fht_get_next_iter( iterator ) == FHT_ITER_RET_OK )
	{
		// pokial zaznam este nie je v prvej fazy vymazania, spravim to teraz
		if( ( (iri_iif_hash_table_item_t*) iterator->data_ptr )->additionalData != NULL )
		{
			// z daneho toku odstranim data kere maju ulozene moduly
			iri_iif_modules_destroy( (iri_iif_hash_table_item_t*) iterator->data_ptr );
			
			// pre kazdu polozku v tabulke alokujem vsetky potrebne data pre ulozenie kompletneho zaznamu k polozke
			free( ( (iri_iif_hash_table_item_t*) iterator->data_ptr )->additionalData );
		}
	}
	
	// odstraneni iteratoru
	fht_destroy_iter( iterator );
	
	// odstraneni celej hash tabulky
	fht_destroy( flowsHashTable );
}
