/*
 * Soubor:  instant_messaging.c
 * Datum:   27.05.2014
 * Autor:   Stanislav Bárta, xbarta29@stud.fit.vutbr.cz
 * Projekt: IM modul pro SDM
 * Popis:   soubor obsahujici funkce rozhrani aplikacniho IM modulu
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <pcap.h>
#include <getopt.h>

#include "instant_messaging.h"
#include "detekce_protokolu.h"
#include "xmpp.h"
#include "irc.h"
#include "oscar.h"
#include "ymsg.h"
#include "iri.h"
#include "sprava.h"
#include "my_string.h"



int iri_iif_socket;
int index_to_data_vector;

/*
 * inicializacni funkce
 */
void instant_messaging_init(int socket, int index)
{
	iri_iif_socket = socket;
	index_to_data_vector = index;
}

int test_port(int zdroj, int cil)
{
	// XMPP komunikace
	if(zdroj == 5222)
	{
		return ACCEPT;
	}

	if(cil == 5222)
	{
		return ACCEPT;
	}

	// IRC komunikace
	if(zdroj == 6665 || zdroj == 6666 || zdroj == 6667 || zdroj == 6668 || zdroj == 6669)
	{
		return ACCEPT;
	}

	if(cil == 6665 || cil == 6666 || cil == 6667 || cil == 6668 || cil == 6669)
	{
		return ACCEPT;
	}

	// OSCAR komunikace
	if(zdroj == 5190 || zdroj == 5191 || zdroj == 5192 || zdroj == 5193)
	{
		return ACCEPT;
	}

	if(cil == 5190 || cil == 5191 || cil == 5192 || cil == 5193)
	{
		return ACCEPT;
	}

	// YMSG komunikace
	if(zdroj == 5050)
	{
		return ACCEPT;
	}

	if(cil == 5050)
	{
		return ACCEPT;
	}

	// komunikace ktera nas nezajima
	return FILTER;
}

/*
 * callback funkce volana po zachyceni packetu
 */
int instant_messaging_process_packet(flows_hash_table_item_t *table_item, iri_iif_offsets_t *offsets, ini3payload_t *packet, int flag)
{

	if(flag == PACKET_TO_ANALYZE)
	{
		if(table_item->key.proto != TCP_PROTOCOL)
		{
			return FILTER;
		}
		int ret = test_port(table_item->key.port_1, table_item->key.port_2);
		if(ret == FILTER)
		{
			return FILTER;
		}
	}

	struct data_packetu obsah = ziskej_obsah(offsets, packet);

	struct sezeni *aktualni_sezeni = NULL;
	// ziskani sezeni
	if(table_item->modules_data[index_to_data_vector] != NULL)
	{
		aktualni_sezeni = (struct sezeni *) (table_item->modules_data[index_to_data_vector]);
	}
	else
	{
		table_item->modules_data[index_to_data_vector] = malloc(sizeof(struct sezeni));
		aktualni_sezeni = (struct sezeni *) (table_item->modules_data[index_to_data_vector]);
		inicializace_sezeni(aktualni_sezeni);
		
		aktualni_sezeni->transportni_protokol = obsah.transportni_protokol;
		if(obsah.verze_ip == IPverze4)
		{
			aktualni_sezeni->typ_ip = IPverze4;
			aktualni_sezeni->ipv4_adresa = obsah.ipv4_cile;
			aktualni_sezeni->ipv4_adresa_2 = obsah.ipv4_zdroje;
		}
		else if(obsah.verze_ip == IPverze6)
		{
			aktualni_sezeni->typ_ip = IPverze6;
			aktualni_sezeni->ipv6_adresa = obsah.ipv6_cile;
			aktualni_sezeni->ipv6_adresa_2 = obsah.ipv6_zdroje;
		}
		aktualni_sezeni->port_2 = obsah.port_zdroje;
		aktualni_sezeni->port = obsah.port_cile;
		aktualni_sezeni->protokol = obsah.typ;
	}

	// podle typu zjisteneho protokolu zpracujeme obsah
	if((obsah.typ == XMPP))
	{
		if(obsah.stav_tcp == ZPRACOVAT)
		{
			zpracuj_xmpp(&obsah, aktualni_sezeni, iri_iif_socket, &(table_item->key));
			free(obsah.data);
			if(aktualni_sezeni->ukonceno)
			{
				smazani_sezeni(aktualni_sezeni);
				free(table_item->modules_data[index_to_data_vector]);
				table_item->modules_data[index_to_data_vector] = NULL;
			}
		}
		else if(obsah.stav_tcp == UKONCENO)
		{
			ukonci_xmpp(&obsah, aktualni_sezeni, iri_iif_socket, &(table_item->key));
			smazani_sezeni(aktualni_sezeni);
			free(table_item->modules_data[index_to_data_vector]);
			table_item->modules_data[index_to_data_vector] = NULL;
		}
	}
	else if((obsah.typ == IRC))
	{
		if(obsah.stav_tcp == ZPRACOVAT)
		{
			zpracuj_irc(&obsah, aktualni_sezeni, iri_iif_socket, &(table_item->key));
			free(obsah.data);
			if(aktualni_sezeni->ukonceno)
			{
				smazani_sezeni(aktualni_sezeni);
				free(table_item->modules_data[index_to_data_vector]);
				table_item->modules_data[index_to_data_vector] = NULL;
			}
		}
		else if(obsah.stav_tcp == UKONCENO)
		{
			ukonci_irc(&obsah, aktualni_sezeni, iri_iif_socket, &(table_item->key));
			smazani_sezeni(aktualni_sezeni);
			free(table_item->modules_data[index_to_data_vector]);
			table_item->modules_data[index_to_data_vector] = NULL;
		}
	}
	else if((obsah.typ == OSCAR))
	{
		if(obsah.stav_tcp == ZPRACOVAT)
		{
			zpracuj_oscar(&obsah, aktualni_sezeni, iri_iif_socket, &(table_item->key));
			if(aktualni_sezeni->ukonceno)
			{
				smazani_sezeni(aktualni_sezeni);
				free(table_item->modules_data[index_to_data_vector]);
				table_item->modules_data[index_to_data_vector] = NULL;
			}
		}
		else if(obsah.stav_tcp == UKONCENO)
		{
			ukonci_oscar(&obsah, aktualni_sezeni, iri_iif_socket, &(table_item->key));
			smazani_sezeni(aktualni_sezeni);
			free(table_item->modules_data[index_to_data_vector]);
			table_item->modules_data[index_to_data_vector] = NULL;
		}
	}
	else if((obsah.typ == YMSG))
	{
		if(obsah.stav_tcp == ZPRACOVAT)
		{
			zpracuj_ymsg(&obsah, aktualni_sezeni, iri_iif_socket, &(table_item->key));
			if(aktualni_sezeni->ukonceno)
			{
				smazani_sezeni(aktualni_sezeni);
				free(table_item->modules_data[index_to_data_vector]);
				table_item->modules_data[index_to_data_vector] = NULL;
			}
		}
		else if(obsah.stav_tcp == UKONCENO)
		{
			ukonci_ymsg(&obsah, aktualni_sezeni, iri_iif_socket, &(table_item->key));
			smazani_sezeni(aktualni_sezeni);
			free(table_item->modules_data[index_to_data_vector]);
			table_item->modules_data[index_to_data_vector] = NULL;
		}
	}
	
	if((obsah.typ == NEZNAME))
	{
		return FILTER;
	}
	else
	{
		return ACCEPT;
	}
}



