/*
 * Soubor:  itc2.c
 * Autor:   Martin Holkovič, xholko00@stud.fit.vutbr.cz
 * Úpravy:  Stanislav Bárta, xbarta29@stud.fit.vutbr.cz
 * Projekt: IRI-IIF pro SDM
 * Popis:   soubor obsahujici funkce pro vymenu dat mezi vlakny
 */

#include "itc2.h"

/*
 * funkce inicializujici jeden kruh struktury pro vymenu dat mezi vlakny
 * @param circle_ptr ukazatel na inicializovany kruh
 * @param unsigned short int pocet zaznamu v kruhu
 * @return int indikace chyby
 */
int itc_init_circle(circle_ptr input, unsigned short int recordsPerCircle, unsigned short int bytesPerRecord)
{
	unsigned short int j; // pomocna premenna pre iteraciu blokov v ramci kruhu
    
    // alokacia priestoru pre zaznamy
	input->data = malloc( sizeof(char) * ( recordsPerCircle) * bytesPerRecord );
	input->valid = malloc( sizeof(char) * ( recordsPerCircle) );
	input->last = malloc( sizeof(char) * ( recordsPerCircle) );
	input->next = NULL;
    
    // kontrola spravnosti alokacie
    if(input->data == NULL || input->valid == NULL || input->last == NULL)
		return MALLOC_ERROR;
    
    // inicializacia flagov pre vsetky zaznamy
	for (j = 0; j < recordsPerCircle; j++)
	{
		input->valid[j] = 0;
		input->last[j] = 0;
	}
    
    return NO_ERROR;
}

/*
 * funkce inicializujici vsechny kruhy struktury pro vymenu dat mezi vlakny
 * @param circle_ptr ukazatel na inicializovany kruh
 * @param unsigned short int pocet zaznamu v kruhu
 * @return int indikace chyby
 */
int itc_init_circles(circles_ptr input, unsigned short int recordsPerCircle, unsigned short int bytesPerRecord)
{
	circle_ptr newCircle; // ukazatel na kruh
	
	// inicializacia indexov v kruhoch
	input->readShift = 0;
	input->writeShift = 0;
	
	// inicializacia ukazatelov na kruhy
	input->readCircle = NULL;
	input->writeCircle = NULL;
	
	// alokacia kruhu
	newCircle = malloc( sizeof(struct circleS) );
	if(newCircle == NULL)
		return MALLOC_ERROR;
	
	// vlozenie novo vytvoreneho kruhu
	input->readCircle = newCircle;
	input->writeCircle = newCircle;
	
	// inicializacia kruhu
	if(itc_init_circle(newCircle, recordsPerCircle, bytesPerRecord) != NO_ERROR)
		return MALLOC_ERROR;
	
    return NO_ERROR;
}

/*
 * funkce inicializujici strukturu pro vymenu dat mezi vlakny
 * @param itc_ptr * ukazatel na inicializovanou strukturu
 * @param unsigned short int pocet kruhu
 * @param unsigned short int pocet zaznamu v jednom kruhu
 * @param unsigned short int pocet bytu na jeden zaznam v kruhu
 * @return int indikace chyby
 */
int itc_init(itc_ptr * input, unsigned short int numberOfCircles, unsigned short int recordsPerCircle, unsigned short int bytesPerRecord)
{
    circles_ptr newCircles; // ukazatel na sady kruhov
    circle_ptr newCircle; // ukazatel na kruh
    unsigned short int i; // pomocna premenna pre iteraciu medzi sadami kruhov
    unsigned short int j; // pomocna premenna pre iteraciu
    
    // alokovanie celkovej struktury
    *input = malloc( sizeof( struct itc ) );
    if( input == NULL )
        return MALLOC_ERROR;
    
    // inicializacia celkovej struktury - ulozenie parametrov zadanej pri inicializacii 
    (*input)->numberOfCircles = numberOfCircles;
    (*input)->recordsPerCircle = recordsPerCircle;
    (*input)->bytesPerRecord = bytesPerRecord;
    
    // inicializacia celkovej struktury - vytvorenie pola ukazatelov na sady kruhov 
    (*input)->circles = malloc( sizeof( circles_ptr ) * numberOfCircles );
    if( (*input)->circles == NULL )
        return MALLOC_ERROR;
    
    // inicializacia celkovej struktury - inicializovanie ukazatelov na sady kruhov
    for (i = 0; i < numberOfCircles; i++)
		(*input)->circles[i] = NULL;
	
	// vytvorenie sad kruhov
	for (i = 0; i < numberOfCircles; i++)
    {
		// alokacia sady kruhov
        newCircles = malloc( sizeof( struct circlesS ) );
		if( newCircles == NULL )
			return MALLOC_ERROR;
        
        // vlozenie novo vytvorenej sady kruhov
        (*input)->circles[i] = newCircles;
        
        // inicializacia sady kruhov
		if( itc_init_circles( newCircles, recordsPerCircle, bytesPerRecord) != NO_ERROR )
            return MALLOC_ERROR;
	}
	
	return NO_ERROR;
}

/*
 * funkce pro zruseni struktury pro vymenu dat mezi vlakny
 * @param itc_ptr * ukazatel na rusenou strukturu
 */
void itc_destroy(itc_ptr * input)
{
	unsigned short int i; // pomocna premenna pre iteraciu medzi sadami kruhov
	circle_ptr temp1, temp2; // pomocne ukazatele na kruhy
    
    // kontrola alokacie celkovej struktury
	if( *input == NULL )
		return;
	
	// prejdem vsetky sady kruhov
	for( i = 0; i < (*input)->numberOfCircles; i++ )
	{
		// kontrola alokacie jednej sady kruhov
		if( (*input)->circles[i] == NULL )
			break;
		
		// nastavenie pomocneho ukazatela na prvy kruh v ramci sady
		temp1 = ( (*input)->circles[i] )->readCircle;
		
		// prejdem vsetky kruhy v ramci sady
		while( temp1 != NULL )
		{
			// do pomocneho ukazatela si ulozim ukazatel na dalsi kruh
			temp2 = temp1->next;
			
			// uvolnenie kruhu
			free( temp1->data );
			free( temp1->valid );
			free( temp1->last );
			free( temp1 );
			
			// prejdenie na dalsi kruh v poradi
			temp1 = temp2;
		}
		
		// uvolnenie sady kruhov
		free ( (*input)->circles[i] );
	}
	
	// uvolnenie pola ukazatelov na sadu kruhov
	free( (*input)->circles );
	
	// uvolnenie celkovej struktury
	free( *input );
	*input = NULL;
}

/*
 * funkce pro rozsireni poctu kruhu ve strukture pro vymenu dat mezi vlakny
 * @param itc_ptr ukazatel na strukturu do ktere se kruh pridava
 * @param unsigned short int index sady kruhu do ktere je kruh pridavan
 * @return circle_ptr ukazatel na novy kruh
 */
circle_ptr itc_circles_extend(itc_ptr input, unsigned short int index)
{
	circle_ptr newCircle; // pomocny ukazatel na vytvaranu strukturu
	
	// alokovanie struktury pre novy kruh
	newCircle = malloc( sizeof( struct circleS ) );
	if(newCircle == NULL)
		return NULL;
	
	// inicializacia hodnot v novom kruhu
	itc_init_circle(newCircle, input->recordsPerCircle, input->bytesPerRecord);
	
	// prepojenie aktualneho kruhu s novo vytvorenym
	((input->circles[index])->writeCircle)->next = newCircle;
	
	// zapisovacie vlakno bude ukazovat na novo vytvoreny kruh
	(input->circles[index])->writeCircle = newCircle;
	
	return newCircle;
}

/*
 * funkce pro zapis novych dat do struktury pro vymenu dat mezi vlakny
 * @param itc_ptr ukazatel na strukturu do ktere je zapisovano
 * @param unsigned short int index sady kruhu do ktere je zapisovano
 * @param char * ukazatel na zapisovana data
 * @return int indikace chyby
 */
int itc_write(itc_ptr input, unsigned short int index, char *data)
{
	circles_ptr circles; // ukazatel na sadu kruhov
	circle_ptr circle; // ukazatel na kruh
	
	circles = input->circles[index];
	circle = circles->writeCircle;
	
	// v aktualnom kruhu sa nachadza este volny blok pre zapis dalsich dat
    if( circle->valid[circles->writeShift] == 0 )
    {
        // skopirovanie dat
        memcpy(circle->data + sizeof(char) * circles->writeShift * input->bytesPerRecord, data, input->bytesPerRecord);
        
        // pametova bariera
        asm volatile("" ::: "memory");
        
        // po zapisani dat oznacim data za platne
        circle->valid[circles->writeShift] = 1;
        
        // posuniem index na blok v ramci kruhu
        circles->writeShift = ( circles->writeShift + 1 ) % input->recordsPerCircle;
    }
    // aktualny kruh je plny a preto treba vytvorit novy a stary s novym prepojit
    else
    {
		circle_ptr newCircle; // pomocny ukazatel na novo vytvorenu strukturu
		
		// vytvorenie noveho kruhu a prepojenie s aktualnym
		newCircle = itc_circles_extend(input, index);
		
		// v pripade ak nebolo mozne alokovat novu strukturu vratim chybu
		if(newCircle == NULL)
            return MALLOC_ERROR;
        
        // skopirovanie dat
        memcpy( newCircle->data, data, input->bytesPerRecord );
        
        // pametova bariera
        asm volatile("" ::: "memory");
        
        // po zapisani dat oznacim data za platne (pri novom kruhu sa ako prve zapisuje na index 0)
        newCircle->valid[0] = 1;
        
        // po zapisani dat nastavim v prvom kruhu aby sa preslo na dalsi kruh
        circle->last[circles->writeShift] = 1;
        
        // posuniem index na blok v ramci kruhu (zacina sa vzdy na indexy 0, preto dalsim indexom v poradi je 1)
        circles->writeShift = 1;
    }
    
	return NO_ERROR;
}

/*
 * funkce pro cteni dat ze struktury pro vymenu dat mezi vlakny
 * @param itc_ptr ukazatel na strukturu ze ktere je cteno
 * @param unsigned short int index sady kruhu ze ktere je cteno
 * @param char * ukazatel do pameti kam zapsat prectena data
 * @return int indikace chyby
 */
int itc_read(itc_ptr input, unsigned short int index, char *data)
{
	circles_ptr circles; // ukazatel na sadu kruhov
	circle_ptr circle; // ukazatel na kruh
	
	circles = input->circles[index];
	circle = circles->readCircle;
	
	// v kruhu su ulozene nejake data
	if( circle->valid[circles->readShift] == 1 )
    {
        // skopirovanie dat
        memcpy(data, circle->data + sizeof(char) * circles->readShift * input->bytesPerRecord, input->bytesPerRecord);
        
        // blok kruhu z ktoreho sa precitali data oznacime ako volny pre zapis dalsich dat
        circle->valid[circles->readShift] = 0;
        
        // posuniem ukazatel na blok v ramci kruhu
        circles->readShift = ( circles->readShift + 1 ) % input->recordsPerCircle;
    }
    // v aktualnom kruhu sa sice nenachadzaju ziadne data, ale bol nastaveny flag aby sa preslo na dalsi kruh v ktorom nejake data su
    else if( circle->last[circles->readShift] == 1 )
    {
        circle_ptr nextCircle; // pomocny ukazatel na kruh, ktory nasleduje za aktualnym
		nextCircle = circle->next;
		
		// uvolnenie kruhu
        free(circle->data);
        free(circle->valid);
        free(circle->last);
        free(circle);
        
        // ukazatel na kruh z ktoreho sa cita presuniem na dalsi kruh v poradi
        circles->readCircle = nextCircle;
        
        // skopirovanie dat
		memcpy(data, nextCircle->data, input->bytesPerRecord);
        
        // blok kruhu z ktoreho sa precitali data oznacime ako volny pre zapis dalsich dat (prve data sa vzdy citaju z indexu cislo 0)
        nextCircle->valid[0] = 0;
        
        // posuniem ukazatel na blok v ramci kruhu (prve data sa vzdy zapisuju na 0-indexm takze dalsim indexom je cislo 1)
        circles->readShift = 1;
    }
    // ziadne data nie su na citanie
    else
        return NOTHING_TO_READ;
    
	return READ_SUCCESSFUL;
}
