/*
 * Soubor:  itc2.h
 * Datum:   27.05.2014
 * Autor:   Martin Holkovič, xholko00@stud.fit.vutbr.cz
 * Úpravy:  Stanislav Bárta, xbarta29@stud.fit.vutbr.cz
 * Projekt: IRI-IIF pro SDM
 * Popis:   hlavickovy soubor pro funkce pro vymenu dat mezi vlakny
 */

#ifndef _IRI_IIF_ITC2_
#define _IRI_IIF_ITC2_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// RETURN USER CODES
#define NOTHING_TO_READ	0
#define READ_SUCCESSFUL	1

// RETURN ERROR CODES
#define NO_ERROR		0
#define MALLOC_ERROR	1




typedef struct circleS { 
    char * data;
    char * valid;
    char * last;
    struct circleS *next;
} *circle_ptr;

typedef struct circlesS {
    circle_ptr readCircle;
    circle_ptr writeCircle;
    unsigned short int readShift;
    unsigned short int writeShift;
} *circles_ptr;

typedef struct itc {
    unsigned short int numberOfCircles;
	unsigned short int recordsPerCircle;
	unsigned short int bytesPerRecord;
    circles_ptr * circles;
} *itc_ptr;


/*
 * funkce inicializujici jeden kruh struktury pro vymenu dat mezi vlakny
 * @param circle_ptr ukazatel na inicializovany kruh
 * @param unsigned int pocet zaznamu v kruhu
 * @param unsigned short int pocet bytu na jeden zaznam v kruhu
 * @return int indikace chyby
 */
int itc_init_circle(circle_ptr input, unsigned short int recordsPerCircle, unsigned short int bytesPerRecord);

/*
 * funkce inicializujici vsechny kruhy struktury pro vymenu dat mezi vlakny
 * @param circle_ptr ukazatel na inicializovany kruh
 * @param unsigned int pocet zaznamu v kruhu
 * @param unsigned short int pocet bytu na jeden zaznam v kruhu
 * @return int indikace chyby
 */
int itc_init_circles(circles_ptr input, unsigned short int recordsPerCircle, unsigned short int bytesPerRecord);

/*
 * funkce inicializujici strukturu pro vymenu dat mezi vlakny
 * @param itc_ptr * ukazatel na inicializovanou strukturu
 * @param unsigned short int pocet kruhu
 * @param unsigned short int pocet zaznamu v jednom kruhu
 * @param unsigned short int pocet bytu na jeden zaznam v kruhu
 * @return int indikace chyby
 */
int itc_init(itc_ptr * input, unsigned short int numberOfCircles, unsigned short int recordsPerCircle, unsigned short int bytesPerRecord);

/*
 * funkce pro zruseni struktury pro vymenu dat mezi vlakny
 * @param itc_ptr * ukazatel na rusenou strukturu
 */
void itc_destroy(itc_ptr * input);

/*
 * funkce pro rozsireni poctu kruhu ve strukture pro vymenu dat mezi vlakny
 * @param itc_ptr ukazatel na strukturu do ktere se kruh pridava
 * @param unsigned int index sady kruhu do ktere je kruh pridavan
 * @return circle_ptr ukazatel na novy kruh
 */
circle_ptr itc_circles_extend(itc_ptr input, unsigned short int index);

/*
 * funkce pro zapis novych dat do struktury pro vymenu dat mezi vlakny
 * @param itc_ptr ukazatel na strukturu do ktere je zapisovano
 * @param unsigned int index sady kruhu do ktere je zapisovano
 * @param char * ukazatel na zapisovana data
 * @return int indikace chyby
 */
int itc_write(itc_ptr input, unsigned short int index, char *data);

/*
 * funkce pro cteni dat ze struktury pro vymenu dat mezi vlakny
 * @param itc_ptr ukazatel na strukturu ze ktere je cteno
 * @param unsigned int index sady kruhu ze ktere je cteno
 * @param char * ukazatel do pameti kam zapsat prectena data
 * @return int indikace chyby
 */
int itc_read(itc_ptr input, unsigned short int index, char *data);

#endif
