
#include "modules.h"
#include <fcntl.h>

// pole ukazatelu na funkce modulu pro zpracovani paketu
module_process_packet_function_t module_process_packet_functions[MODULE_COUNT];
module_process_end_of_flow_function_t module_process_end_of_flow[MODULE_COUNT];
module_destroy_data_function_t module_destroy_data_functions[MODULE_COUNT];

// socket pre spojenie s iri-core
int iriCoreSocket;

void iri_iif_modules_init( int input_iriCoreSocket )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
		printf("%d %s %d: iri_iif_modules_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// vytvorenie socketoveho spojenia s iri-core
	iriCoreSocket = input_iriCoreSocket;
    
	// automaticky generovany subor na zaklade konfigu (modules.config)
	#include "configs/modules-init.include"
}

/*
 * funkce pro otevreni TCP spojeni s MF ktere se budou zasilat vytvorene zpravy IRI
 * @return int ukazatel na socket pro komunikaci s IRI-Core
 */
int iri_iif_modules_open_socket_to_iri_core()
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
			printf("%d %s %d: iri_iif_modules_open_socket_to_iri_core()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
    int sockfd;
    struct addrinfo hints, *serv_info, *p;
    int rv;
	
    memset(&hints, 0x0, sizeof(struct addrinfo));
    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;

    rv = getaddrinfo(MODULES_CONNECTION_TO_IRI_CORE_IP,
                     MODULES_CONNECTION_TO_IRI_CORE_PORT,
                     &hints, &serv_info);

    if (rv != 0) 
    {
        fprintf(stderr,"ERROR, no such host\n");
        exit(2);
    }
    

    for (p = serv_info; p != NULL; p = p->ai_next) {
        // Create a socket file descriptor
        sockfd = socket(p->ai_family, p->ai_socktype, p->ai_protocol);
        if (sockfd < 0) {
            perror("ERROR opening socket for IRI-Core");
            rv = 1;
            continue;
        }
        
        
        // connect to IRI-Core socket    
        if (connect(sockfd, p->ai_addr, p->ai_addrlen) < 0) {
            close(sockfd);
            perror("ERROR connecting to IRI-Core");
            rv = 3;
            continue;
        }
        break;
    }
    
    if (p == NULL) {
        freeaddrinfo(serv_info);
        exit(rv);
    } else {
        freeaddrinfo(serv_info);
        return sockfd;
    }
    fcntl(sockfd, F_SETFD, O_NONBLOCK);
}

void iri_iif_modules_send_iri_message( const char* input_text , char input_flag, iri_iif_hash_table_item_t *input_tableItem, char input_ipVersion)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
		printf("%d %s %d: iri_iif_modules_send_iri_message(%d)\n", (unsigned int)pthread_self(), __FILE__, __LINE__, input_flag);
	#endif
	
	char sendMessage[20150]; // pomocne pole pre vytvorenie kompletnej IRI spravy, 20000 je maximalna velkost z modulu
    int returnValue;
	
	// skopirujem povodnu spravu co odosiela modul
	strcpy(sendMessage, "(\"SDM\", "); // identifikator pre IRI-core
	strcat(sendMessage, input_text);
	
	// modul oznamuje, ze na danu IRI spravu by sa mala vratit odpoved
	if( input_flag == 1)
	{
		char feedback[130]; // pomocne pole pre dalsiu polozku do IRI spravy
		
		char key_ip_1_str[INET6_ADDRSTRLEN]; // pomocne premenna pre skonvertovanie IP adresy do textovej podoby
		char key_ip_2_str[INET6_ADDRSTRLEN]; // podla Linux/include/linux/inet.h INET6_ADDRSTRLEN = 48B
		
		// poznacim si ze odosielam IRI spravu pre ktoru musim cakat na odpoved
		input_tableItem->additionalData->modules_waitingForReplies = input_tableItem->additionalData->modules_waitingForReplies + 1;
		
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
			printf("%d %s %d: ostava cakat na %u odpovedi\n", (unsigned int)pthread_self(), __FILE__, __LINE__, input_tableItem->additionalData->modules_waitingForReplies);
		#endif
		
		// skonvertovanie IP adries do textovej podoby na zaklade verzie adres
		if(input_ipVersion == 4)
		{
			inet_ntop(AF_INET, &(input_tableItem->key.items.ip_1.v4_long.data.d8), key_ip_1_str, INET6_ADDRSTRLEN);
			inet_ntop(AF_INET, &(input_tableItem->key.items.ip_2.v4_long.data.d8), key_ip_2_str, INET6_ADDRSTRLEN);
		}
		else
		{
			inet_ntop(AF_INET6, &(input_tableItem->key.items.ip_1.v6.d8), key_ip_1_str, INET6_ADDRSTRLEN);
			inet_ntop(AF_INET6, &(input_tableItem->key.items.ip_2.v6.d8), key_ip_2_str, INET6_ADDRSTRLEN);
		}
		
		// vytvorenie dalsej polozky v IRI sprave
		sprintf(feedback, ", (%d, %d, \"%s\", %d, \"%s\", %d)", 0, input_tableItem->key.items.proto, key_ip_1_str, input_tableItem->key.items.port_1, key_ip_2_str, input_tableItem->key.items.port_2);
		
		// pripojenie vzniknuteho retazca na koniec IRI spravy
		strcat(sendMessage, feedback);
	}
	
	// na konci kazdej IRI spravy musi byt ukoncenie riadku
	strcat(sendMessage, ") )\n");
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1 || IRI_IIF_DEBUG_COMMUNICATION == 1
		printf("%d %s %d: odosielam IRI spravu:%s", (unsigned int)pthread_self(), __FILE__, __LINE__, sendMessage);
	#endif
	
	
	// odoslanie spravy na socket
    if ( write(iriCoreSocket, sendMessage, strlen(sendMessage)) < 0)
    {
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
			printf("%d %s %d: data neboli uspesne odoslane\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
	}
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
	else
	{
		printf("%d %s %d: data boli uspesne odoslane\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	}
	#endif
}

/*
 * Funkce pro predani paketu do modulu ke zpracovani
 * @param iri_iif_hash_table_collision_item_t * ukazatel za zaznam toku obsahujici infrmaci o tom ktere moduly dany tok zadajici
 * @param iri_iif_offsets_t * ukazatel na strukturu obsahujici offsety 
 * @param ini3payload_t * ukazatel na strukturu obsahujici paket a data tvorici hlavicku
 * @return int urcujici zda maji moduly o tok zajem nebo ne
 */

int iri_iif_modules_send_packet( iri_iif_hash_table_item_t * input_hashTableRecord, iri_iif_offsets_t * input_offsets, ini3payload_t * input_packet)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
		printf("%d %s %d: iri_iif_modules_send_packet()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	int i; // pomocna premenna pre iteraciu v ramci pola modulov
	unsigned short int binaryIndex = 1; // binarny index do vektora modulov, reprezentujuci prislusny modul
	char returnValue; // navratova hodnota z modulu
	
	// zavolani funkci modulu pro analyzu paketu
	for(i = 0 ; i < MODULE_COUNT; i++)
	{
		// binarne nasobenie vektoru s indexem modulu, pri nenulovej hodnote viem ze modul ma zaujem
		if( input_hashTableRecord->additionalData->modules_vector2 & binaryIndex )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
				printf("%d %s %d: Modul is indexem %d chce dany tok spracovat\n", (unsigned int)pthread_self(), __FILE__, __LINE__, i);
			#endif
			
			// odoslem modulu paket a zistim si ci chce dalej spracovavat pakety z daneho toku
			returnValue = module_process_packet_functions[i](input_hashTableRecord, input_offsets, input_packet->parts.data, input_packet->parts.header.blob_size, &input_hashTableRecord->additionalData->modules_data[i] );
			
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
                printf("%d %s %d: Modul is indexem %d skoncil s spracovanim toku\n", (unsigned int)pthread_self(), __FILE__, __LINE__, i);
            #endif

			if( returnValue == MODULE_WANTS_TO_FILTER_THIS_FLOW )
			{
				// pomocou operacie XOR odstranim dany modul z vektoru modulov
				input_hashTableRecord->additionalData->modules_vector2 = input_hashTableRecord->additionalData->modules_vector2 ^ binaryIndex;
			}
		}
		
		// posuniem index modulu v ramci vektoru (operacia je ekvivalentna s operaciou *2 )
		binaryIndex = binaryIndex << 1;
	}
	
	// ulozeni paketu do ADB pokud ma nejaky aplikacni modul zaujem (t.j. vektor modulov je nenulovy)
	if( input_hashTableRecord->additionalData->modules_vector2 != NOBODY_WANTS_TO_ANALYZE_THIS_FLOW || input_hashTableRecord->additionalData->modules_waitingForReplies > 0 )
	{
		iri_iif_adb_insert_packet( input_hashTableRecord, input_offsets, input_packet);
	}
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
		printf("%d %s %d: Vektor modulov po spracovani paketu modulmi:  %d\n", (unsigned int)pthread_self(), __FILE__, __LINE__, input_hashTableRecord->additionalData->modules_vector2);
	#endif
	
	// vracim pocet modulu kere maju stale zaujem o dany tok
	return input_hashTableRecord->additionalData->modules_vector2;
}

void iri_iif_modules_notice_end_of_flow( iri_iif_hash_table_item_t * input_hashTableRecord)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
		printf("%d %s %d: iri_iif_modules_notice_end_of_flow()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	int i; // pomocna premenna pre iteraciu v ramci pola modulov
	unsigned short int binaryIndex = 1; // binarny index do vektora modulov, reprezentujuci prislusny modul
	
	// zavolani funkci modulu pro analyzu paketu
	for(i = 0 ; i < MODULE_COUNT; i++)
	{
		// binarne nasobenie vektoru s indexem modulu, pri nenulovej hodnote viem ze modul ma zaujem
		if( input_hashTableRecord->additionalData->modules_vector2 & binaryIndex )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
				printf("%d %s %d: Modul is indexem %d chce dany tok spracovat\n", (unsigned int)pthread_self(), __FILE__, __LINE__, i);
			#endif
			
			// odoslem modulu paket a zistim si ci chce dalej spracovavat pakety z daneho toku
			module_process_end_of_flow[i](input_hashTableRecord, &input_hashTableRecord->additionalData->modules_data[i]);
		}
		
		// posuniem index modulu v ramci vektoru (operacia je ekvivalentna s operaciou *2 )
		binaryIndex = binaryIndex << 1;
	}
}

void iri_iif_modules_destroy( iri_iif_hash_table_item_t * input_hashTableRecord )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
		printf("%d %s %d: iri_iif_modules_destroy()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	int i; // pomocna premenna pre iteraciu v ramci pola modulov
	
	// zavolani funkci modulu pro analyzu paketu
	for(i = 0 ; i < MODULE_COUNT; i++)
	{
		// dany modul ma ulozene nejake data
		if( input_hashTableRecord->additionalData->modules_data[i] != NULL )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_MODULES == 1
				printf("%d %s %d: Modul is indexem %d ide zmazat jeho data k toku\n", (unsigned int)pthread_self(), __FILE__, __LINE__, i);
			#endif
			
			// zmazat data daneho modulu
			module_destroy_data_functions[i](input_hashTableRecord, &input_hashTableRecord->additionalData->modules_data[i] );
		}
	}
}
