#include "new_sip.h"

/**
 * Init flow info.
 */
void initUser(userInfoPtr newUser, char * ip1, char * ip2, char * uri1) {
  newUser->ipAddress1 = (char *) malloc(strlen(ip1) + 1);
  strcpy(newUser->ipAddress1, ip1);
  newUser->ipAddress2 = (char *) malloc(strlen(ip2) + 1);
  strcpy(newUser->ipAddress2, ip2);
  newUser->sipUri1 = (char *) malloc(strlen(uri1) + 1);
  strcpy(newUser->sipUri1, uri1);
  newUser->registerStatus = NO_IRI_SENT;
  newUser->callStatus = NO_IRI_SENT;
  // init first transaction in list of transactions
  newUser->transactions = NULL;
  newUser->sipUri2 = (char *) calloc(1, sizeof(char));
  for(int i = 0; i < 5; i++){
    newUser->rtpPort1[i] = 0;
    newUser->rtpPort2[i] = 0;
  }
}

/**
 * Insert new transaction.
 */
void insertNewTransaction(userInfoPtr newUser, char * uri2, char * callId, short unsigned int seqNum, methodType method, methodType status) {
  transactionInfo * currentTransaction = newUser->transactions;
  transactionInfo * prevTransaction = NULL;
  // find end of list
  while (currentTransaction != NULL) {
    prevTransaction = currentTransaction;
    currentTransaction = currentTransaction->next;
  }
  // create new transaction
  transactionInfo * newTransaction = (transactionInfo *) malloc(sizeof (transactionInfo));
  newTransaction->uri2 = (char *) malloc(strlen(uri2) + 1);
  strcpy(newTransaction->uri2, uri2);
  newTransaction->callId = (char *) malloc(strlen(callId) + 1);
  strcpy(newTransaction->callId, callId);
  newTransaction->sequenceNumber = seqNum;
  newTransaction->method = method;
  newTransaction->status = status;
  newTransaction->waitForAck = false;
  newTransaction->next = NULL;
  // add transaction to the list
  if (prevTransaction == NULL)
    newUser->transactions = newTransaction;
  else
    prevTransaction->next = newTransaction;
}

/**
 * Find transaction.
 */
transactionInfo * findTransaction(userInfoPtr newUser, char * callId, short unsigned int seqNum, methodType method) {
  transactionInfo * currentTransaction = newUser->transactions;
  while (currentTransaction != NULL) {
    if (strcmp(currentTransaction->callId, callId) == 0 && currentTransaction->sequenceNumber == seqNum && currentTransaction->method == method)
      return (currentTransaction);
    currentTransaction = currentTransaction->next;
  }
  return (NULL);
}

/**
 * Find and delete transaction.
 */
void findAndDeleteTransaction(userInfoPtr newUser, char * callId, short unsigned int seqNum, methodType method) {
  //printf("pattern: %s %d %d\n", callId, seqNum, method);
  transactionInfo * currentTransaction = newUser->transactions;
  transactionInfo * prevTransaction = NULL;
  while (currentTransaction != NULL) {
    //printf("%s %d %d\n", currentTransaction->callId, currentTransaction->sequenceNumber, currentTransaction->method);
    if (strcmp(currentTransaction->callId, callId) == 0 && currentTransaction->sequenceNumber == seqNum && currentTransaction->method == method) {
      break;
    }
    prevTransaction = currentTransaction;
    currentTransaction = currentTransaction->next;
  }
  if (currentTransaction == NULL)
    return;
  if (prevTransaction == NULL)
    newUser->transactions = currentTransaction->next;
  else
    prevTransaction->next = currentTransaction->next;

  free(currentTransaction->uri2);
  free(currentTransaction->callId);
  free(currentTransaction);
  //printf("transaction deleted\n");
}

/**
 * Delete first transaction.
 */
void deleteFirstTransaction(userInfoPtr newUser) {
  transactionInfo * deleteTransaction = newUser->transactions;
  newUser->transactions = newUser->transactions->next;

  free(deleteTransaction->uri2);
  free(deleteTransaction->callId);
  free(deleteTransaction);
  //printf("transaction deleted\n");
}

/**
 * Count transactions.
 */
int countTransactions(userInfoPtr newUser) {
  int cnt = 0;
  transactionInfo * currentTransaction = newUser->transactions;
  while (currentTransaction != NULL) {
    cnt++;
    currentTransaction = currentTransaction->next;
  }
  return (cnt);
}

void printTransaction(transactionInfo * currentTransaction) {
  printf("cseq: %d\n", currentTransaction->sequenceNumber);
  printf("id: %s\n", currentTransaction->callId);
  if (currentTransaction->method == REQUEST_REGISTER)
    printf("method: REGISTER\n");
  else if (currentTransaction->method == REQUEST_INVITE)
    printf("method: INVITE\n");
  else if (currentTransaction->method == REQUEST_BYE)
    printf("method: BYE\n");
  else if (currentTransaction->method == REQUEST_OPTIONS)
    printf("method: OPTIONS\n");
  else
    printf("method: OTHER\n");
  if (currentTransaction->status == NO_METHOD)
    printf("status: REQUEST\n");
  else if (currentTransaction->status == RESPONSE_SUCCESS)
    printf("status: 200 OK\n");
  else if (currentTransaction->status == RESPONSE_ERROR)
    printf("status: 3xx, 4xx, ERROR\n");
  else if (currentTransaction->status == RESPONSE_SESSION_PROGRESS)
    printf("status: 183 SESSION PROGRESS\n");
  else
    printf("status: OTHER RESPONSE\n");
  printf("to: %s\n", currentTransaction->uri2);
}

/**
 * Print transactions.
 */
void printTransactions(userInfoPtr newUser) {
  printf("======================\n");
  transactionInfo * currentTransaction = newUser->transactions;
  while (currentTransaction != NULL) {
    printTransaction(currentTransaction);
    currentTransaction = currentTransaction->next;
  }
  printf("======================\n");
}

/**
 * Get source SIP URI.
 */
void getSrcSipUri(char * sipMsg, char * uriFrom) {
  char * begin = strstr(sipMsg, "From:");
  char * end = NULL;
  // SIP URI was not found in message
  if (begin == NULL) {
    //printf("Address not found.\n");
    uriFrom[0] = '\0';
    return;
  }    // cut out SIP URI between <>
  else {
    if (strstr(begin, "sip:") != NULL) {
      begin = strstr(begin, "sip:");
    } else if (strstr(begin, "tel:") != NULL) {
      begin = strstr(begin, "tel:");
    } else {
      uriFrom[0] = '\0';
      return;
    }
    begin = begin + 4;
    if (strstr(begin, ";") != NULL)
      end = strstr(begin, ";");
    if (strstr(begin, ">") != NULL)
      end = strstr(begin, ">");
  }
  
  memcpy(uriFrom, begin, end - begin);
  uriFrom[end - begin] = '\0';
  //printf("FROM: %s\n", res);
}

/**
 * Get call ID.
 */
void getCallId(char * sipMsg, char * id) {
  char * begin = strstr(sipMsg, "Call-Id:");
  char * end = NULL;

  if (strstr(sipMsg, "Call-Id:") != NULL) {
    begin = strstr(sipMsg, "Call-Id:");
  } else if (strstr(sipMsg, "Call-ID:") != NULL) {
    begin = strstr(sipMsg, "Call-ID:");
  }
  if (begin == NULL) {
    //printf("Call ID not found.\n");
    id[0] = '\0';
    return;
  } else {
    begin = begin + 9;
    end = strstr(begin, "\n");
  }
 
  memcpy(id, begin, end - begin);
  id[end - begin] = '\0';
  //printf("CALL-ID: %s\n", res);
}

/**
 * Get destination SIP URI.
 */
void getDstSipUri(char * sipMsg, char * uriTo) {
  char * begin = strstr(sipMsg, "To:");
  char * end = NULL;
  // SIP URI was not found in message
  if (begin == NULL) {
    //printf("Address not found.\n");
    uriTo[0] = '\0';
    return;
  }
  if (strstr(begin, "sip:") != NULL) {
    begin = strstr(begin, "sip:");
  } else if (strstr(begin, "tel:") != NULL) {
    begin = strstr(begin, "tel:");
  } else {
    uriTo[0] = '\0';
    return;
  }
  begin = begin + 4;
  if (strstr(begin, ";") != NULL) {
    end = strstr(begin, ";");
    end--;
  }
  if (strstr(begin, ">") != NULL) {
    end = strstr(begin, ">");
  }
  if (end == NULL) {
    end = strstr(begin, "\n");
    if (end == NULL)
      uriTo[0] = '\0';
      return;
  }
  //char * res = malloc(end - begin + 1);
  memcpy(uriTo, begin, end - begin);
  uriTo[end - begin] = '\0';
  //printf("TO: %s\n", res);
}

/**
 * Get RTP port(s) - "audio", "video", "text", "application", and "message"
 */
void getMediaPort(char * sipMsg, short unsigned int * portList) {
  char * begin;
  char * end = NULL;
  char res[128];
  const char *ports[5];
  ports[0] = "m=audio";
  ports[1] = "m=video";
  ports[2] = "m=text";
  ports[3] = "m=application";
  ports[4] = "m=message";

  //memset(portList, 0, 5);

  for (int i = 0; i < 5; i++) {
    begin = strstr(sipMsg, ports[i]);
    if (begin != NULL) {
      //printf("port found\n");
      begin = begin + strlen(ports[i]);
      end = strstr(begin, "\n");
      memcpy(res, begin, end - begin);
      res[end - begin] = '\0';
      portList[i] = atoi(res);
    }
//    else
//      printf("port not found\n");
  }
}

/**
 * Get request sequence number.
 */
unsigned short int getSeqNumber(char * sipMsg) {
  char * begin = strstr(sipMsg, "CSeq: ");
  char * end = NULL;

  if (begin == NULL) {
//    printf("CSeq not found.\n");
    return (0);
  } else {
    begin = begin + strlen("CSeq: ");
    end = strstr(begin, " ");
  }
  char res[128];
  memcpy(res, begin, end - begin);
  res[end - begin] = '\0';
  int seqNum = atoi(res);
  return (seqNum);
}

/**
 * Check expitation in REGISTER message (if 0, message type is UNREGISTER)
 */
char * checkUnregister(char * sipMsg) {
  char * found = NULL;
  found = strstr(sipMsg, "Expires: 0");
  if (found == NULL)
    found = strstr(sipMsg, ";expires=0");
  return (found);
}

/**
 * Find message type substring.
 */
bool checkMsgType(char * sipMsg, char * substring) {
  char * firstLine;
  if (strstr(sipMsg, "\n") != NULL) {
    firstLine = strstr(sipMsg, "\n");
    char * type = strstr(sipMsg, substring);
    if (type != NULL && firstLine - type > 0)
      return (true);
  } else {
    if (strstr(sipMsg, substring) != NULL)
      return (true);
  }
  return (false);
}

/**
 * Get message type.
 */
int getMessageType(char * sipMsg) {
  if (checkMsgType(sipMsg, "REGISTER")) {
    if (checkUnregister(sipMsg) != NULL) {
      //printf("UNREGISTER\n");
      return (REQUEST_UNREGISTER);
    }
    //printf("REGISTER\n");
    return (REQUEST_REGISTER);
  }
  if (checkMsgType(sipMsg, "INVITE")) {
    //printf("INVITE\n");
    return (REQUEST_INVITE);
  }
  if (checkMsgType(sipMsg, "PRACK")) {
    //printf("PRACK\n");
    return (REQUEST_PRACK);
  }
  if (checkMsgType(sipMsg, "ACK")) {
    //printf("ACK\n");
    return (REQUEST_ACK);
  }
  if (checkMsgType(sipMsg, "CANCEL")) {
    //printf("CANCEL\n");
    return (REQUEST_CANCEL);
  }
  if (checkMsgType(sipMsg, "BYE")) {
    //printf("BYE\n");
    return (REQUEST_BYE);
  }
  if (checkMsgType(sipMsg, "OPTIONS")) {
    return (REQUEST_OPTIONS);
  }
  if (checkMsgType(sipMsg, "SUBSCRIBE")) {
    return (REQUEST_SUBSCRIBE);
  }
  if (checkMsgType(sipMsg, "NOTIFY")) {
    return (REQUEST_NOTIFY);
  }
  if (checkMsgType(sipMsg, "PUBLISH")) {
    return (REQUEST_PUBLISH);
  }
  if (checkMsgType(sipMsg, "SIP/2.0 183")) {
    //printf("183\n");
    return (RESPONSE_SESSION_PROGRESS);
  }
  if (checkMsgType(sipMsg, "SIP/2.0 1")) {
    //printf("1xx PROVISIONAL\n");
    return (RESPONSE_PROVISIONAL);
  }
  if (checkMsgType(sipMsg, "SIP/2.0 2")) {
    //printf("2xx OK\n");
    return (RESPONSE_SUCCESS);
  }
  if (checkMsgType(sipMsg, "SIP/2.0 3")) {
    //printf("3xx ERROR\n");
    return (RESPONSE_ERROR);
  }
  if (checkMsgType(sipMsg, "SIP/2.0 4")) {
    //printf("4xx ERROR\n");
    return (RESPONSE_ERROR);
  }
  if (checkMsgType(sipMsg, "SIP/2.0 5")) {
    //printf("5xx ERROR\n");
    return (RESPONSE_ERROR);
  }
  if (checkMsgType(sipMsg, "SIP/2.0 6")) {
    //printf("6xx ERROR\n");
    return (RESPONSE_ERROR);
  }
  return (-1);
}

methodType getSeqMethod(char * sipMsg) {
  char * begin = strstr(sipMsg, "CSeq: ");
  char * end = NULL;

  if (begin == NULL) {
//    printf("CSeq not found.\n");
    return (0);
  } else {
    begin = begin + strlen("CSeq: ");
    begin = strstr(begin, " ");
    begin++;
    end = strstr(begin, "\n");
  }
  char res[128];
  memcpy(res, begin, end - begin);
  res[end - begin] = '\0';
  int type = getMessageType(res);
  return (type);
}

/**
 * Send register message to IRI Core
 */
void printIriMessageRegister(char * type, char * details, char * ipAddress1, char * ipAddress2, char * sipUri, iri_iif_hash_table_item_t *table_item) {
  char msg[2048];
  char addressType[5];
  // check address type
  if (strchr(ipAddress1, ':') == NULL)
    sprintf(addressType, "IPv4");
  else
    sprintf(addressType, "IPv6");

  struct timeval tv;
  double time_in_micros;
  gettimeofday(&tv, NULL);
  time_in_micros = tv.tv_sec + 1000000.0 / tv.tv_usec;
  // create IRI message
  //sprintf(msg, "('SIP', %.2f, '%s', '', [('UDP', ('%d.%d.%d.%d', %d, '%d.%d.%d.%d', %d)), ('SIP URI', '%s')], [], []", time_in_micros, type, table_item->key.items.ip_2.v4_long.data.d8[0], table_item->key.items.ip_2.v4_long.data.d8[1], table_item->key.items.ip_2.v4_long.data.d8[2], table_item->key.items.ip_2.v4_long.data.d8[3], table_item->key.items.port_2, table_item->key.items.ip_1.v4_long.data.d8[0], table_item->key.items.ip_1.v4_long.data.d8[1], table_item->key.items.ip_1.v4_long.data.d8[2], table_item->key.items.ip_1.v4_long.data.d8[3], table_item->key.items.port_1, sipUri);
  sprintf(msg, "('SIP', %.2f, '%s', '%s', [('UDP', ('%s', %d, '%s', %d)), ('SIP URI', '%s')], [], []", time_in_micros, type, details, ipAddress1, table_item->key.items.port_2, ipAddress2, table_item->key.items.port_1, sipUri);
  //sprintf(msg, "('SIP', %.2f, '%s', '%s', [('UDP', ('%s', %d, '%s', %d)), ('SIP URI', '%s')], [], []", time_in_micros, type, details, ipAddress1, 0, ipAddress2, 0, sipUri);
  //printf("REGISTER: %s\n", msg);
  //printf("SIP, %s, %s, %s\n", type, ipAddress, sipUri);
  if(strcmp(type, "END") == 0)
      iri_iif_modules_send_iri_message(msg, 0, table_item, 4);
  else
      iri_iif_modules_send_iri_message(msg, 1, table_item, 4);
}

/**
 * Send call message to IRI Core
 */
void printIriMessageCall(char * type, char * message, userInfoPtr curr, iri_iif_hash_table_item_t *table_item) {
  char msg[2048];
  msg[0] = '\0';
  char srcAddressType[5];
  char dstAddressType[5];
  // check source address type
  if (strchr(curr->ipAddress1, ':') == NULL)
    sprintf(srcAddressType, "IPv4");
  else
    sprintf(srcAddressType, "IPv6");

  // check destination address type
  if (strchr(curr->ipAddress2, ':') == NULL)
    sprintf(dstAddressType, "IPv4");
  else
    sprintf(dstAddressType, "IPv6");

  struct timeval tv;
  double time_in_micros;
  gettimeofday(&tv, NULL);
  time_in_micros = tv.tv_sec + 1000000.0 / tv.tv_usec;
  // create IRI message
  if (strcmp(type, "REPORT") == 0) {
    sprintf(msg, "('SIP', %.2f, '%s', '%s', [('SIP URI', '%s'), ('SIP URI', '%s')], [], []",
        time_in_micros, type, message, curr->sipUri1, curr->sipUri2);
  } else {
    for (int i = 0; i < 5; i++) {
      if(curr->rtpPort1[i] != 0 && curr->rtpPort2 != 0){
        sprintf(msg, "('SIP', %.2f, '%s', '%s', [('UDP', ('%s', %d, '%s', %d)), ('SIP URI', '%s'), ('SIP URI', '%s')], [], []",
          time_in_micros, type, message, curr->ipAddress1, curr->rtpPort1[i], curr->ipAddress2, curr->rtpPort2[i], curr->sipUri1, curr->sipUri2);
      }
      else if(curr->rtpPort1[i] != 0){
        sprintf(msg, "('SIP', %.2f, '%s', '%s', [('UDP3', '%s_%d'), ('SIP URI', '%s'), ('SIP URI', '%s')], [], []",
          time_in_micros, type, message, curr->ipAddress1, curr->rtpPort1[i], curr->sipUri1, curr->sipUri2);
      }
      else if(curr->rtpPort2[i] != 0){
        sprintf(msg, "('SIP', %.2f, '%s', '%s', [('UDP3', '%s_%d'), ('SIP URI', '%s'), ('SIP URI', '%s')], [], []",
          time_in_micros, type, message, curr->ipAddress2, curr->rtpPort2[i], curr->sipUri1, curr->sipUri2);
      }
    }
  }
  if(strlen(msg) > 0)
    //printf("CALL: %s\n", msg);
    iri_iif_modules_send_iri_message(msg, 0, table_item, 4);
}

void processSip(char * srcAddress, char * dstAddress, char * sipMsg, void * extPointer, iri_iif_hash_table_item_t *table_item) {
//void processSip(char * srcAddress, char * dstAddress, char * sipMsg, void * extPointer) {
  //static int count = 0;
  //printf("%d\n", ++count);
  // get basic info from message
  char uriFrom[128];
  char uriTo[128];
  char id[128];
  getSrcSipUri(sipMsg, uriFrom);
  getDstSipUri(sipMsg, uriTo);
  getCallId(sipMsg, id);
  int messageType = getMessageType(sipMsg);
 
  // probably not SIP packet
  if (strlen(uriFrom) == 0 || strlen(id) == 0)
    return;

  if (strlen(uriTo) == 0)
    uriTo[0] = '\0';
  //printf("%s\n", uriFrom);
  //printf("%s\n", uriTo);
  userInfoPtr * dataPointer = (userInfoPtr *) extPointer;

  // flow unknown, create record
  if (*dataPointer == NULL) {
    userInfoPtr newUser = (userInfoPtr) malloc(sizeof (struct userInfoPtr));
    initUser(newUser, srcAddress, dstAddress, uriFrom);
    * dataPointer = newUser;
  }

  userInfoPtr currentUser = *dataPointer;
  transactionInfo * trans = NULL;

  // new message is REQUEST and there is too much old crap -> delete oldest crap
  if (countTransactions(currentUser) >= 10) {
//    printf("deleting transaction\n");
    deleteFirstTransaction(currentUser);
  }

  // get RTP ports
  if (messageType == REQUEST_INVITE) {
    getMediaPort(sipMsg, currentUser->rtpPort1);
  }
  if (getSeqMethod(sipMsg) == REQUEST_INVITE && (messageType == RESPONSE_SUCCESS || messageType == RESPONSE_SESSION_PROGRESS)) {
    getMediaPort(sipMsg, currentUser->rtpPort2);
  }
  
  // update destionation SIP URI
  if (strcmp(currentUser->sipUri2, uriTo) != 0 && strcmp(uriFrom, uriTo) != 0) {
    currentUser->sipUri2 = (char *) malloc(strlen(uriTo) + 1);
    strcpy(currentUser->sipUri2, uriTo);
  }
  // update source SIP URI
  if(strcmp(currentUser->sipUri1, uriFrom) != 0){
    currentUser->sipUri1 = (char *) malloc(strlen(uriFrom) + 1);
    strcpy(currentUser->sipUri1, uriFrom);
  }

  // send BYE report
  if (messageType == REQUEST_BYE) {
    printIriMessageCall("REPORT", "BYE was sent", currentUser, table_item);

  }

  trans = findTransaction(currentUser, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg));
  if(trans != NULL && trans->status == NO_METHOD && messageType < RESPONSE_SESSION_PROGRESS){
    trans = NULL;
  }
  if (trans != NULL) {
    //printf("---->transaction found\n");
    // find out current status
    methodType status;
    if(trans->status == NO_METHOD){
      status = messageType;
    }
    else {
      status = trans->status;
    }
//    printf("status: %d\n", status);
//    printTransaction(trans);
    // sort message types
    if (trans->method == REQUEST_REGISTER) {
      if(status == RESPONSE_SUCCESS){
        if (currentUser->registerStatus == NO_IRI_SENT || currentUser->registerStatus == IRI_END_SENT) {
          printIriMessageRegister("BEGIN", "" , currentUser->ipAddress1, currentUser->ipAddress2, currentUser->sipUri1, table_item);
          currentUser->registerStatus = IRI_BEGIN_SENT;
        }
        else{
          printIriMessageRegister("CONTINUE", "", currentUser->ipAddress1, currentUser->ipAddress2, currentUser->sipUri1, table_item);
        }
      }
      else if (status == RESPONSE_ERROR){
        if (currentUser->registerStatus == IRI_BEGIN_SENT && currentUser->callStatus != IRI_BEGIN_SENT) {
          printIriMessageRegister("END", "", currentUser->ipAddress1, currentUser->ipAddress2, currentUser->sipUri1, table_item);
          currentUser->registerStatus = IRI_END_SENT;
        }
      }
    } /// INVITE
    else if (trans->method == REQUEST_INVITE) {
      //printTransaction(trans);
      if(status == RESPONSE_SUCCESS){
        if(currentUser->rtpPort1[0] == 0){
          trans->waitForAck = true;
          if (currentUser->callStatus == NO_IRI_SENT || currentUser->callStatus == IRI_END_SENT){
            printIriMessageCall("BEGIN", "200 OK, waiting for ACK", currentUser, table_item);
            currentUser->callStatus = IRI_THREE_SENT;
          }
        }
        if(!trans->waitForAck){
          if (currentUser->callStatus != IRI_BEGIN_SENT) {
            printIriMessageCall("BEGIN", "200 OK", currentUser, table_item);
            currentUser->callStatus = IRI_BEGIN_SENT;
          } else {
            printIriMessageCall("CONTINUE", "200 OK", currentUser, table_item);
          }
        }
      }
      else if(status == RESPONSE_SESSION_PROGRESS){
        if(currentUser->callStatus == IRI_THREE_SENT){
          if(messageType == REQUEST_INVITE){
            int tmp = currentUser->rtpPort1[0];
            currentUser->rtpPort1[0] = 0;
            printIriMessageCall("END", "UDP3 end, full RTP begins", currentUser, table_item);
            currentUser->rtpPort1[0] = tmp;
          }
          currentUser->callStatus = IRI_END_SENT;
        }
        // start RTP capturing
        if (currentUser->callStatus == NO_IRI_SENT || currentUser->callStatus == IRI_END_SENT) {
          printIriMessageCall("BEGIN", "183 Session progress", currentUser, table_item);
          if(currentUser->rtpPort1[0] == 0 || currentUser->rtpPort2[0] == 0)
            currentUser->callStatus = IRI_THREE_SENT;
          else
            currentUser->callStatus = IRI_BEGIN_SENT;
        } 
      }
      /*else if(status == RESPONSE_PROVISIONAL){
        printf("provisional\n");
      }*/
    }
    else if(trans->method == REQUEST_BYE){
      if (currentUser->callStatus == IRI_BEGIN_SENT) {
          printIriMessageCall("END", "", currentUser, table_item);
          currentUser->callStatus = IRI_END_SENT;
        }
    }
    else if(trans->method == REQUEST_NOTIFY){
      printIriMessageRegister("REPORT", "Notify", currentUser->ipAddress1, currentUser->ipAddress2, currentUser->sipUri1, table_item);
    }
    else if(trans->method == REQUEST_SUBSCRIBE){
      printIriMessageRegister("REPORT", "Subscribe", currentUser->ipAddress1, currentUser->ipAddress2, currentUser->sipUri1, table_item);
    }
    else if(trans->method == REQUEST_UNREGISTER){
      // update registration
        if(currentUser->registerStatus == IRI_BEGIN_SENT){
          printIriMessageRegister("END", "Unregister", currentUser->ipAddress1, currentUser->ipAddress2, currentUser->sipUri1, table_item);
          currentUser->registerStatus = IRI_END_SENT;
        }
    }
    if(status != RESPONSE_SESSION_PROGRESS && status != RESPONSE_PROVISIONAL && !trans->waitForAck)
      findAndDeleteTransaction(currentUser, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg));
  }
  else {
    if(messageType == REQUEST_ACK){
      transactionInfo * waitingForAck = currentUser->transactions;
      transactionInfo * found = NULL;
      while (waitingForAck != NULL) {
        if(waitingForAck->sequenceNumber == getSeqNumber(sipMsg) && waitingForAck->waitForAck)
          found = waitingForAck;
        waitingForAck = waitingForAck->next;
      }
      if(found != NULL){
//        printTransaction(found);
        if(currentUser->callStatus == IRI_THREE_SENT){
          printIriMessageCall("END", "ACK, new begin should start", currentUser, table_item);
          currentUser->registerStatus = IRI_END_SENT;
        }
        getMediaPort(sipMsg, currentUser->rtpPort1);
        if (currentUser->callStatus != IRI_BEGIN_SENT) {
          printIriMessageCall("BEGIN", "200 OK", currentUser, table_item);
          currentUser->callStatus = IRI_BEGIN_SENT;
        } else {
          printIriMessageCall("CONTINUE", "200 OK", currentUser, table_item);
        }
        findAndDeleteTransaction(currentUser, found->callId, found->sequenceNumber, found->method);
      }
    }
//    printf("---->inserting new transaction\n");
    if (messageType == RESPONSE_SUCCESS && getSeqMethod(sipMsg) == REQUEST_REGISTER){
      if (currentUser->registerStatus == NO_IRI_SENT || currentUser->registerStatus == IRI_END_SENT) {
        printIriMessageRegister("BEGIN", "" , currentUser->ipAddress1, currentUser->ipAddress2, currentUser->sipUri1, table_item);
        currentUser->registerStatus = IRI_BEGIN_SENT;
      }
    }
    if(messageType < REQUEST_UNREGISTER && messageType != REQUEST_ACK)
      insertNewTransaction(currentUser, uriTo, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg), NO_METHOD);
    else if(messageType == REQUEST_UNREGISTER)
      insertNewTransaction(currentUser, uriTo, id, getSeqNumber(sipMsg), REQUEST_UNREGISTER, NO_METHOD);
    else{
      if(messageType != RESPONSE_PROVISIONAL && getSeqMethod(sipMsg) != REQUEST_ACK)
        insertNewTransaction(currentUser, uriTo, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg), messageType);
/*      else
        printf("NOT ADDING TO LIST\n");*/
    }
  }

//  printTransactions(currentUser);
//  printf("---------------------------------------------------------------------------\n");
}

/**
 * Process packet
 */
int new_sip_process_packet(iri_iif_hash_table_item_t *table_item, iri_iif_offsets_t *offsets, unsigned char *packet, uint16_t packet_size,  void * input_dataPointer)
//int new_sip_process_packet(unsigned char *packet, uint16_t packet_size,  void * input_dataPointer)
{
  // Allocate space for an address
  char src_address[ADDRESS_SIZE];
  char dst_address[ADDRESS_SIZE];
  // UDP header
  const struct udphdr *udp = NULL;
  static int cnt = 0;
  //
  char L3_type[4];
  u_int16_t type_link_proto;
  // Sizes
  int size_link_proto = 0;
  int size_ip = 0;
  int size_udp = sizeof(struct udphdr);
  
  /// Ethernet
  const struct ether_header * ether = (struct ether_header*) packet;
  size_link_proto = sizeof (struct ether_header);
  type_link_proto = ntohs(ether->ether_type);
  
  /// IPv4
  if (type_link_proto == 0x0800) {
    const struct ip *ip = (struct ip*) (packet + size_link_proto);
    size_ip = sizeof (struct ip);
    /// Check if the packet is UDP or TCP
    if (ip->ip_p == IPPROTO_UDP) {
      strcpy(L3_type, "udp");
      udp = (struct udphdr*) (packet + size_link_proto + size_ip);
    }
    else
      return 0;
    /// get IP address
    if (inet_ntop(AF_INET, &(ip->ip_src), src_address, 64) == NULL) {
      fprintf(stderr, "Cannot get source IP address\n");
      return 0;
    }
    if (inet_ntop(AF_INET, &(ip->ip_dst), dst_address, 64) == NULL) {
	  fprintf(stderr, "Cannot get destination IP address\n");
	  return 0;
    }
  }
   /// IPv6
  else if (type_link_proto == 0x86dd) {
    const struct ip6_hdr *ip = (struct ip6_hdr*) (packet + size_link_proto);
    size_ip = sizeof (struct ip6_hdr);
    /// Check if the packet is UDP
    if (ip->ip6_ctlun.ip6_un1.ip6_un1_nxt == IPPROTO_UDP){
	  strcpy(L3_type, "udp");
      udp = (struct udphdr*) (packet + size_link_proto + size_ip);
	}
	else
	  return 0;
    
    if (inet_ntop(AF_INET6, &(ip->ip6_src), src_address, 64) == NULL) {
      fprintf(stderr, "Cannot get IP address\n");
      return 0;
	}
  }
  else {
    fprintf(stderr, "Unknown Ethernet type\n");
    return 0;
  }
  char * sipRequest = NULL;
  
  if (strcmp(L3_type, "udp") == 0) {
    cnt = cnt + 1;
    sipRequest = (char *) udp + size_udp;
  }
  processSip(src_address, dst_address, sipRequest, input_dataPointer, table_item);
  return 0;
}

int new_sip_destroy_data(iri_iif_hash_table_item_t *table_item, void * data) {
//int new_sip_destroy_data(void * data) {
  userInfoPtr * dataPointer = (userInfoPtr *) data;
  // nothing to delete
  if (*dataPointer == NULL) {
    return(0);
  }
  // delete all transactions
  userInfoPtr currentUser = *dataPointer;
  while (currentUser->transactions != NULL) {
    transactionInfo * deleteTransaction = currentUser->transactions;
    currentUser->transactions = currentUser->transactions->next;

    free(deleteTransaction->callId);
    free(deleteTransaction->uri2);
    free(deleteTransaction);
  }
  if (currentUser->callStatus == IRI_BEGIN_SENT) {
    printIriMessageCall("END", "SDM timeout", currentUser, table_item);
    //printIriMessageCall("END", "SDM timeout", currentUser);
  }
  if (currentUser->registerStatus == IRI_BEGIN_SENT) {
    printIriMessageRegister("END", "SDM timeout", currentUser->ipAddress1, currentUser->ipAddress2, currentUser->sipUri1, table_item);
    //printIriMessageRegister("END", "SDM timeout", currentUser->ipAddress1, currentUser->ipAddress2, currentUser->sipUri1);
  }

  // delete flow info
  if(currentUser->ipAddress1 != NULL)
    free(currentUser->ipAddress1);
  if(currentUser->ipAddress2 != NULL)
    free(currentUser->ipAddress2);
  if(currentUser->sipUri1 != NULL)
    free(currentUser->sipUri1);
  if(currentUser->sipUri2 != NULL)
    free(currentUser->sipUri2);
  free(*dataPointer);
  *dataPointer = NULL;
  return(0);
}
