
#include "packet-buffer.h"

__thread packetBufferItem_ptr pointerToSavedPacket; // ukazatel na ulozeny paket v ramci jedneho zivotneho cyklu (zabranuje duplicitnym kopiam)

__thread packetBuffer_ptr packetBufferFilled; // zoznam ulozenych paketov
__thread packetFreeBuffer_ptr packetBufferFree; // zasobnik volnych paketov pripravenych pre pouzitie

// pri prijati noveho packetu si zmazem pointer na ulozeny predosly packet, aby sem ten novy mohel znovu ulozit
void iri_iif_packet_buffer_update_cache( packetBufferItem_ptr input_newValue )
{
	pointerToSavedPacket = NULL;
}

int packet_delete_me()
{
	packetBufferItem_ptr bufferItem; // pomocne premenne
	unsigned int i = 0;
	
	bufferItem = packetBufferFilled->first;
	while( bufferItem != NULL )
	{
		i++;
		bufferItem = bufferItem->next;
	}
	return i;
}

int iri_iif_packet_buffer_free_add()
{
	// tady bych informaciu o spusteni funkcie rozhodne nedaval :)
	
	packetBufferItem_ptr bufferItem; // prvok paket bufferu
	
	// vytvorenie prvku
	bufferItem = malloc( sizeof(struct packetBufferItem ) ); // alokace jednoho prvku seznamu
	if( bufferItem == NULL )
		return IRI_IIF_RETURN_MALLOC_ERROR;
    
	// prvok bude ukazovat na posledny prvok v zasobniku
	bufferItem->next = packetBufferFree->last;
	
	// prvok sa stane prvym prvkov v zasobniku
	packetBufferFree->last = bufferItem;
	
	// aktualizujem pocitadlo volnych paketov
	packetBufferFree->length = packetBufferFree->length + 1;
	
	return IRI_IIF_RETURN_OK;
}

int iri_iif_packet_buffer_free_init()
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: packet_buffer_free_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	int i; // pomocna premenna pre iteraciu podla poctu vytvaranych paketov
	
	// alokace struktury pro metainformace k seznamu
	packetBufferFree = malloc( sizeof( struct packetFreeBuffer ) );
	if( packetBufferFree == NULL )
		return IRI_IIF_RETURN_MALLOC_ERROR;
	
	packetBufferFree->last = NULL; // ukazatel na posledni vlozeny prvek
	packetBufferFree->length = 0; // aktualni pocet volne dostupnych paketu = pocet prvku seznamu
	
	// vytvorim si specifikovane mnozstvo volnych paketov
	for( i = 0; i < AMOUNT_OF_THE_PACKETS; i++ )
	{
		// funkcia na vytvorenie prvku a pripojenie do bufferu volnych prvkov
		if( iri_iif_packet_buffer_free_add() != IRI_IIF_RETURN_OK )
			return IRI_IIF_RETURN_MALLOC_ERROR;
	}
	
	return IRI_IIF_RETURN_OK;
}

int iri_iif_packet_buffer_filled_init()
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: iri_iif_packet_buffer_filled_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// alokace struktury pro metainformace k seznamu
	packetBufferFilled = malloc( sizeof(struct packetBuffer ) );
	if( packetBufferFilled == NULL )
	{
		return IRI_IIF_RETURN_MALLOC_ERROR;
	}
	
	packetBufferFilled->first = NULL; // inicializovany zoznam neobsahuje ziadny prvok
	packetBufferFilled->last = NULL; // inicializovany zoznam neobsahuje ziadny prvok
	
	return IRI_IIF_RETURN_OK;
}

int iri_iif_packet_buffer_init()
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: packet_buffer_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// inicializacia zasobniku volnych prvkov
	if( iri_iif_packet_buffer_free_init(  ) != IRI_IIF_RETURN_OK )
	{
		return IRI_IIF_RETURN_MALLOC_ERROR;
	}
	
	// inicializacia zoznamu pre ulozene pakety
	if( iri_iif_packet_buffer_filled_init(  ) != IRI_IIF_RETURN_OK )
	{
		return IRI_IIF_RETURN_MALLOC_ERROR;
	}
	
	return IRI_IIF_RETURN_OK;
}


void iri_iif_packet_buffer_destroy_free_buffer(  )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: iri_iif_packet_buffer_destroy_free_buffer()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	packetBufferItem_ptr bufferItem, nextBufferItem; // pomocne premenne
	
	// zasobnik byl alokovany
	if( packetBufferFree != NULL )
	{
		// zacnem od posledneho prvku (prvok na vrchu zasobniku)
		bufferItem = (packetBufferFree)->last;
		
		// prejdem cely zasobnik
		while( bufferItem != NULL )
		{
			// zalohujem si odkaz na dalsi prvok
			nextBufferItem = bufferItem->next;
			
			// odstranim jeden prvok
			free( bufferItem );
			
			// obnovenie zalohy ukazatela
			bufferItem = nextBufferItem;
		}
		
		// odstranim cely buffer
		free( packetBufferFree );
		packetBufferFree = NULL;
	}
}

void iri_iif_packet_buffer_destroy_filled_buffer(  )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: iri_iif_packet_buffer_destroy_filled_buffer()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	packetBufferItem_ptr bufferItem, nextBufferItem; // pomocne premenne
	
	// nenulovy zeznam
	if( packetBufferFilled != NULL )
	{
		// zacnem od prveho prvku
		bufferItem = packetBufferFilled->first;
		
		// prejdem cely zeznam prvku
		while( bufferItem != NULL )
		{
			// zalohujem si odkaz na dalsi prvok
			nextBufferItem = bufferItem->next;
			
			// odstranim jeden prvok
			free( bufferItem );
			
			// obnovenie zalohy ukazatela
			bufferItem = nextBufferItem;
		}
		// odstranim cely buffer
		free( packetBufferFilled );
		packetBufferFilled = NULL;
	}
}

void iri_iif_packet_buffer_destroy()
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: packet_buffer_destroy()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// odstranenie volnych paketov
	iri_iif_packet_buffer_destroy_free_buffer( );
	
	// odstranenie ulozenych paketov
	iri_iif_packet_buffer_destroy_filled_buffer( );
}


int iri_iif_packet_buffer_free_get( packetBufferItem_ptr * bufferItemOutput )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: iri_iif_packet_buffer_free_get()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// kontrola ci tam je vubec nejaky prvek
	if( packetBufferFree->last == NULL )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
			printf("%d %s %d: plny packet buffer\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		return IRI_IIF_RETURN_NO_SPACE;
	}
	
	// zoberem prvy prvek ze zeznamu
	*bufferItemOutput = packetBufferFree->last;
	
	// posunem ukazatel na dalsi prvek
	packetBufferFree->last = packetBufferFree->last->next;
	
	// aktualizujem pocitadlo volnych paketov
	packetBufferFree->length = packetBufferFree->length - 1;
	
	// inicializaciu hodnot (ani ukazatelov) prvku sa riesi ve funkcii kera tuto funkciu vola
	
	return IRI_IIF_RETURN_OK;
}

void iri_iif_packet_buffer_item_init( iri_iif_hash_table_item_t *input_hashTableRecord, packetBufferItem_ptr input_item, iri_iif_offsets_t *offsets, ini3payload_t *packet )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: packet_buffer_item_init()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// nastavenie ukazatelov daneho prvku
	input_item->prev = packetBufferFilled->last;
	input_item->next = NULL;
	
	// skopirovanie obsahu paketu
	memcpy( &(input_item->key), &(input_hashTableRecord->key), sizeof(item_key_t) );
	memcpy( &(input_item->offsets), offsets, sizeof(iri_iif_offsets_t) );
	memcpy( &(input_item->packet.parts.header), &(packet->parts.header) , sizeof(ini3_head_t) );
	memcpy( input_item->packet.parts.data, packet->parts.data, sizeof(uint8_t) * packet->parts.header.blob_size );
	
	
	// inicializacia prvku
	input_item->uses = 1;
	///////////////////////////////////////////////////// blok ADB /////////////////////////////////////////////////////
	input_item->adb_inUse = FALSE;
	input_item->adb_next = NULL;
	///////////////////////////////////////////////// TCP reassembling /////////////////////////////////////////////////
	input_item->tcpr_inUse = FALSE;
	input_item->tcpr_seqNum = 0;
	input_item->tcpr_seqNumNext = 0;
	input_item->tcpr_prev = NULL;
	input_item->tcpr_next = NULL;
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}

void iri_iif_packet_buffer_packet_free_first( packetBufferItem_ptr * output_bufferItem )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: iri_iif_packet_buffer_packet_free_first()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	iri_iif_hash_table_item_t *flowRecord; // premenna obsahujuca polozku z uloziska
	
	// vyberiem si najstarsi ulozeny paket
	*output_bufferItem = packetBufferFilled->first;
	
	// vyhledam si polozku v tabulke tokov ku ktorej patri tento paket
	flowRecord = iri_iif_find_flow_record( & ((*output_bufferItem)->key) );
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: nasel sa zaznam %p\n", (unsigned int)pthread_self(), __FILE__, __LINE__, flowRecord);
	#endif
	
	// jednotlivym blokom musim oznamit, ze prvok sa ide odstranit
	///////////////////////////////////////////////////// blok ADB /////////////////////////////////////////////////////
	// paket je pouzivany v ADB
	if( (*output_bufferItem)->adb_inUse == TRUE )
	{
		// mazem posledny prvek v ADB
		if( flowRecord->additionalData->adb_last == *output_bufferItem )
		{
			flowRecord->additionalData->adb_last = NULL;
		}
		
		// ukazatel na prvy ADB paket posuniem na dalsi paket
		flowRecord->additionalData->adb_first = (*output_bufferItem)->adb_next;
	}
	///////////////////////////////////////////////// TCP reassembling /////////////////////////////////////////////////
	// paket je pouzivany v TCP reassemblingu
	if( (*output_bufferItem)->tcpr_inUse == TRUE )
	{
		// vytvorit rozhrani pro diry v TCP tokoch // TODO (TCP reassembling): odeslat paket modulom
		
		// jedna sa o prvy prvok v zozname paketov
		if( (*output_bufferItem)->tcpr_prev == NULL )
		{
			// zistim si do ktoreho smeru patri dany paket a aktualizujem ukazatel na prvy paket
			if( *output_bufferItem == flowRecord->additionalData->tcpr_firstPacketB )
			{
				flowRecord->additionalData->tcpr_firstPacketB = (*output_bufferItem)->tcpr_next;
			}
			else
			{
				flowRecord->additionalData->tcpr_firstPacketS = (*output_bufferItem)->tcpr_next;
			}
		}
		// prvok ma nejakeho predchodcu
		else
		{
			(*output_bufferItem)->tcpr_prev->tcpr_next = (*output_bufferItem)->tcpr_next;
		}
		
		// jedna sa o posledny prvok v zozname paketov
		if( (*output_bufferItem)->tcpr_next == NULL )
		{
			// zistim si do ktoreho smeru patri dany paket a aktualizujem ukazatel na posledny paket
			if( *output_bufferItem == flowRecord->additionalData->tcpr_lastPacketB )
			{
				flowRecord->additionalData->tcpr_lastPacketB = (*output_bufferItem)->tcpr_prev;
			}
			else
			{
				flowRecord->additionalData->tcpr_lastPacketS = (*output_bufferItem)->tcpr_prev;
			}
		}
		// prvok ma nejakeho naslednika
		else
		{
			(*output_bufferItem)->tcpr_next->tcpr_prev = (*output_bufferItem)->tcpr_prev;
		}
	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	// znizim pocitadlo alokovanych bajtov
	flowRecord->additionalData->pb_storedBytes -= DEFAULT_PACKET_SIZE;
	
	// posuniem ukazatel na prvy ulozeny paket
	packetBufferFilled->first = packetBufferFilled->first->next;
	
	// prvy ulozeny paket uz dalej nema predchodcu
	packetBufferFilled->first->prev = NULL;
}

int iri_iif_packet_buffer_packet_save( iri_iif_hash_table_item_t *input_hashTableRecord, iri_iif_offsets_t *offsets, ini3payload_t *packet, packetBufferItem_ptr * bufferItemOutput )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: iri_iif_packet_buffer_packet_save()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	packetBufferItem_ptr bufferItem; // ukazatel na vytiahnuty volny prvok
	
	// v ramci zivotneho cyklu uz niekto rovnaky paket ukladal a preto sa vrati ukazatel na rovnaky ulozeny paket
	if( pointerToSavedPacket != NULL )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
			printf("%d %s %d: pouziti zalohy %p\n", (unsigned int)pthread_self(), __FILE__, __LINE__, pointerToSavedPacket);
		#endif
		// vratenie zalohovaneho odkazu
		*bufferItemOutput = pointerToSavedPacket;
		
		// dany paket teraz pouziva viacero blokov
		pointerToSavedPacket->uses = pointerToSavedPacket->uses + 1;
		return IRI_IIF_RETURN_OK;
	}
	
	// ziskam volny prvok
	if( iri_iif_packet_buffer_free_get( &bufferItem ) == IRI_IIF_RETURN_NO_SPACE )
	{
		// nie je ziadny volny prvok k dispozicii
		iri_iif_packet_buffer_packet_free_first( &bufferItem );
	}
	
	// inicializacia prvku
	iri_iif_packet_buffer_item_init( input_hashTableRecord, bufferItem, offsets, packet );
	
	// doterajsi posledny prvek bude ukazovat na novo pridany prvek
	if( packetBufferFilled->last != NULL )
		packetBufferFilled->last->next = bufferItem;
	
	// pripojim prvek na konec zeznamu
	packetBufferFilled->last = bufferItem;
	
	// ak byl zeznam prazdny, bude prvek zaroven prvym prvkem
	if( packetBufferFilled->first == NULL )
		packetBufferFilled->first = bufferItem;
	
	// ulozenie zalohy na ulozeny paket v pameti
	pointerToSavedPacket = bufferItem;
	
	// zvysim pocitadlo alokovanych bajtov
	input_hashTableRecord->additionalData->pb_storedBytes += DEFAULT_PACKET_SIZE;
	
	// kontrola ci sa tok nestal "velkym" tokom (obsahuje vela ulozenych paketov)
	if( input_hashTableRecord->additionalData->pb_storedBytes > PACKET_BUFFER_BIG_FLOW_LEVEL )
	{
		iri_iif_packet_buffer_big_flow( input_hashTableRecord );
	}
	
	// vratim ukazatel na ulozeny prvek
	*bufferItemOutput = bufferItem;
	
	return IRI_IIF_RETURN_OK;
}

void iri_iif_packet_buffer_delete_first_n_packets_from_flow( iri_iif_hash_table_item_t * input_hashTableRecord, unsigned int input_deletePackets )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: iri_iif_packet_buffer_delete_first_n_packets_from_flow()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	unsigned int deleted; // pocitadlo kolko paketov bolo vymazanych
	packetBufferItem_ptr packet = NULL; // ukazatel na ulozeny paket
	packetBufferItem_ptr temp = NULL; // pomocny ukazatel na nasledujuci paket
	
	// zacnem mazat od prveho ADB paketu
	packet = input_hashTableRecord->additionalData->adb_first;
	
	for( deleted = 0; deleted < input_deletePackets && packet != NULL; deleted++ )
	{
		// zalohovanie ukazatela na nasledujuci prvok
		temp = packet->adb_next;
		
		// odstranit data z TCPr
		if( packet->tcpr_prev != NULL )
		{
			packet->tcpr_prev->tcpr_next = packet->tcpr_next;
		}
		if( packet->tcpr_next != NULL )
		{
			packet->tcpr_next->tcpr_prev = packet->tcpr_prev;
		}
		
		// odstranit paket
		packet->uses = 0;
		iri_iif_packet_buffer_packet_free( input_hashTableRecord, packet );
		
		// prejdem na dalsi prvok
		packet = temp;
		
	}
		
	if( packet == NULL)
	{
		// aktualizujem ukazatel na posledny paket v ADB
		input_hashTableRecord->additionalData->adb_last = NULL;
	}
	
	// aktualizujem ukazatel na prvy paket v ADB
	input_hashTableRecord->additionalData->adb_first = temp;
}

void iri_iif_packet_buffer_big_flow( iri_iif_hash_table_item_t * input_hashTableRecord)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: iri_iif_packet_buffer_big_flow()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// packet buffer uz je skoro plny, odeslem TCP data a zmazem ADB data
	if( packetBufferFree->length <= PACKET_BUFFER_ALMOST_FULL_LEVEL_2)
	{
		
		// odstranim prvych N paketu z TCPr
		iri_iif_tcp_reassembling_delete_first_n_packets_from_flow( input_hashTableRecord, PACKET_BUFFER_DELETE_PACKETS );
		
		// ak viem ze sa data ukladaju do ADB viem, ze prvych N paketov z ADB je urcite starsich nez prvych N z TCP reassemblingu
		if( input_hashTableRecord->additionalData->adb_store == TRUE )
		{
			// pre dany tok odstranim prvych N paketov
			iri_iif_packet_buffer_delete_first_n_packets_from_flow( input_hashTableRecord, PACKET_BUFFER_DELETE_PACKETS );
		}
		
	}
	// packet buffer sa zacina nebezpecne zaplnovat, odeslem TCP data
	else if( packetBufferFree->length <= PACKET_BUFFER_ALMOST_FULL_LEVEL_1)
	{
		iri_iif_tcp_reassembling_send_data_with_gaps( input_hashTableRecord );
	}
}

int iri_iif_packet_buffer_packet_free( iri_iif_hash_table_item_t *input_hashTableRecord, packetBufferItem_ptr bufferItem )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: iri_iif_packet_buffer_packet_free()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: aktualna zaloha - %p()\n", (unsigned int)pthread_self(), __FILE__, __LINE__, pointerToSavedPacket);
	#endif
	
	// dany packet vyuziva vic nez jeden blok ktory ho momentalne maze
	if( bufferItem->uses > 1 )
	{
		// znizim pocitadlo pouziti
		bufferItem->uses = bufferItem->uses - 1;
		
		return IRI_IIF_RETURN_OK;
	}
	else
	{
		if( bufferItem == pointerToSavedPacket )
		{
			pointerToSavedPacket = NULL;
		}
	}
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: mazem prvok %p\n", (unsigned int)pthread_self(), __FILE__, __LINE__, bufferItem);
	#endif
	
	// odlinkovanie prvku zo zoznamu ulozenych paketov
	
	// kontrola ci nemazem prvy paket v zozname
	if( packetBufferFilled->first == bufferItem )
		packetBufferFilled->first = bufferItem->next;
	
	// kontrola ci nemazem posledny paket v zozname
	if( packetBufferFilled->last == bufferItem )
		packetBufferFilled->last = bufferItem->prev;
	
	// ak ma prvok predchodcu, prelinkujem ukazatele
	if( bufferItem->prev != NULL )
		bufferItem->prev->next = bufferItem->next;
	
	// ak ma prvok nasledovnika, prelinkujem ukazatele
	if( bufferItem->next != NULL )
		bufferItem->next->prev = bufferItem->prev;
	
	// nie je nutne inicializovat hodnoty, pretoze to sa deje pri ukladani noveho prvku
	
	
	// znizim pocitadlo alokovanych bajtov
	input_hashTableRecord->additionalData->pb_storedBytes -= DEFAULT_PACKET_SIZE;
	
	// pripojenie prvku do zoznamu volnych paketov
	
	// vkladany prvok bude ukazovat na prvok co bol doteraz prvy v zozname
	bufferItem->next = packetBufferFree->last;
	
	// vlozenie prvku na vrchol zasobniku
	packetBufferFree->last = bufferItem;
	
	// aktualizujem pocitadlo volnych paketov
	packetBufferFree->length = packetBufferFree->length + 1;
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_PACKET_BUFFER == 1
		printf("%d %s %d: packetBufferFree->length = %u, packetBufferFilled->length = %u\n", (unsigned int)pthread_self(), __FILE__, __LINE__, packetBufferFree->length, packet_delete_me());
	#endif
	return IRI_IIF_RETURN_OK;
}

// pomocna funkcia na vypis pouzitych paketu
void iri_iif_packet_buffer_print()
{
	printf("%d %s %d: iri_iif_packet_buffer_print()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	
	packetBufferItem_ptr temp;
	temp = packetBufferFilled->first;
	unsigned int i = 0;
	
	printf("%d %s %d: packetBufferFree->length = %u, packetBufferFilled->length = %u\n", (unsigned int)pthread_self(), __FILE__, __LINE__, packetBufferFree->length, packet_delete_me());
	iri_iif_hash_table_item_t *flowRecord;
    char key_ip_1_str[INET6_ADDRSTRLEN];
    char key_ip_2_str[INET6_ADDRSTRLEN];
	
	while( temp != NULL)
	{
        if(temp->key.items.ipver)
        {
            inet_ntop(AF_INET, &(temp->key.items.ip_1.v4_long.data.d8), key_ip_1_str, INET6_ADDRSTRLEN);
            inet_ntop(AF_INET, &(temp->key.items.ip_2.v4_long.data.d8), key_ip_2_str, INET6_ADDRSTRLEN);
        }
        else
        {
            inet_ntop(AF_INET6, &(temp->key.items.ip_1.v6.d8), key_ip_1_str, INET6_ADDRSTRLEN);
            inet_ntop(AF_INET6, &(temp->key.items.ip_2.v6.d8), key_ip_2_str, INET6_ADDRSTRLEN);

        }
        
		printf("IPv%u, protocol-%u, port1-%u, port2-%u, IP1-%s, IP2-%s\n",temp->key.items.ipver,temp->key.items.proto,temp->key.items.port_1,temp->key.items.port_2,key_ip_1_str,key_ip_2_str);
	
		temp = temp->next;
		i++;
		if(i >50)
			break;
	}
}
