#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <stdbool.h>

#include "sip.h"


/**
 * Init flow info.
 */
void initUser(userInfoPtr newUser, char * ip1, char * ip2, char * uri1) {
    newUser->ipAddress1 = (char *) malloc(strlen(ip1) + 1);
    strcpy(newUser->ipAddress1, ip1);
    newUser->ipAddress2 = (char *) malloc(strlen(ip2) + 1);
    strcpy(newUser->ipAddress2, ip2);
    newUser->sipUri1 = (char *) malloc(strlen(uri1) + 1);
    strcpy(newUser->sipUri1, uri1);
    newUser->registerStatus = NO_IRI_SENT;
    newUser->callStatus = NO_IRI_SENT;
    // init first transaction in list of transactions
    newUser->transactions = NULL;
}

/**
 * Insert new transaction.
 */
void insertNewTransaction(userInfoPtr newUser, char * uri2, char * callId, short unsigned int seqNum, methodType method){
    transactionInfo * currentTransaction = newUser->transactions;
    transactionInfo * prevTransaction = NULL;
    // find end of list
    while(currentTransaction != NULL){
      prevTransaction = currentTransaction;
      currentTransaction = currentTransaction->next;
    }
    // create new transaction
    transactionInfo * newTransaction = (transactionInfo *)malloc(sizeof(transactionInfo));
    newTransaction->uri2 = (char *) malloc(strlen(uri2) + 1);
    strcpy(newTransaction->uri2, uri2);
    newTransaction->callId = (char *) malloc(strlen(callId) + 1);
    strcpy(newTransaction->callId, callId);
    newTransaction->sequenceNumber = seqNum;
    newTransaction->method = method;
    newTransaction->next = NULL;
    // add transaction to the list
    if(prevTransaction == NULL)
      newUser->transactions = newTransaction;
    else
      prevTransaction->next = newTransaction;
}

/**
 * Find transaction.
 */
transactionInfo * findTransaction(userInfoPtr newUser, char * callId, short unsigned int seqNum, methodType method){
  transactionInfo * currentTransaction = newUser->transactions;
  while(currentTransaction != NULL){
    if(strcmp(currentTransaction->callId, callId) == 0 && currentTransaction->sequenceNumber == seqNum && currentTransaction->method == method)
      return(currentTransaction);
    currentTransaction = currentTransaction->next;
  }
  return(NULL);
}

/**
 * Find and delete transaction.
 */
void findAndDeleteTransaction(userInfoPtr newUser, char * callId, short unsigned int seqNum, methodType method){
  transactionInfo * currentTransaction = newUser->transactions;
  transactionInfo * prevTransaction = NULL;
  while(currentTransaction != NULL){
    if(strcmp(currentTransaction->callId, callId) == 0 && currentTransaction->sequenceNumber == seqNum && currentTransaction->method == method){
      break;
    }
    prevTransaction = currentTransaction;
    currentTransaction = currentTransaction->next;
  }
  if(currentTransaction == NULL)
    return;
  if(prevTransaction == NULL)
    newUser->transactions = currentTransaction->next;
  else
    prevTransaction->next = currentTransaction->next;
  
  free(currentTransaction->uri2);
  free(currentTransaction->callId);
  free(currentTransaction);
  //printf("transaction deleted\n");
}

/**
 * Delete first transaction.
 */
void deleteFirstTransaction(userInfoPtr newUser){
  transactionInfo * deleteTransaction = newUser->transactions;
  newUser->transactions = newUser->transactions->next;

  free(deleteTransaction->uri2);
  free(deleteTransaction->callId);
  free(deleteTransaction);
  //printf("transaction deleted\n");
}

/**
 * Count transactions.
 */
int countTransactions(userInfoPtr newUser){
  int cnt = 0;
  transactionInfo * currentTransaction = newUser->transactions;
  while(currentTransaction != NULL){
    cnt++;
    currentTransaction = currentTransaction->next;
  }
  return(cnt);
}

/**
 * Print transactions.
 */
void printTransactions(userInfoPtr newUser){
  printf("======================\n");
  transactionInfo * currentTransaction = newUser->transactions;
  while(currentTransaction != NULL){
    printf("cseq: %d\n", currentTransaction->sequenceNumber);
    printf("id: %s\n", currentTransaction->callId);
    printf("method: %d\n", currentTransaction->method);
    printf("to: %s\n", currentTransaction->uri2);
    currentTransaction = currentTransaction->next;
  }
  printf("======================\n");
}

/**
 * Get source SIP URI.
 */
char * getSrcSipUri(char * sipMsg) {
    char * begin = strstr(sipMsg, "From:");
    char * end = NULL;
    // SIP URI was not found in message
    if (begin == NULL) {
        //printf("Address not found.\n");
        return(NULL);
    }
    // cut out SIP URI between <>
    else {
        if(strstr(begin, "sip:") != NULL){
            begin = strstr(begin, "sip:");
        }
        else if(strstr(begin, "tel:") != NULL){
            begin = strstr(begin, "tel:");
        }
        else {
            return(NULL);
        }
        begin = begin + 4;
        if(strstr(begin, ";") != NULL)
            end = strstr(begin, ";");  
        if(strstr(begin, ">") != NULL)
            end = strstr(begin, ">");
    }
    char * res = malloc(end - begin + 1);
    memcpy(res, begin, end - begin);
    res[end - begin] = '\0';
    //printf("FROM: %s\n", res);
    return res;
}

/**
 * Get call ID.
 */
char * getCallId(char * sipMsg) {
    char * begin = strstr(sipMsg, "Call-Id:");
    char * end = NULL;

    if(strstr(sipMsg, "Call-Id:") != NULL){
        begin = strstr(sipMsg, "Call-Id:");
    }
    else if(strstr(sipMsg, "Call-ID:") != NULL){
        begin = strstr(sipMsg, "Call-ID:");
    }
    if (begin == NULL) {
        //printf("Call ID not found.\n");
        return (NULL);
    } else {
        begin = begin + 9;
        end = strstr(begin, "\n");
    }
    char * res = malloc(end - begin + 1);
    memcpy(res, begin, end - begin);
    res[end - begin] = '\0';
    //printf("CALL-ID: %s\n", res);
    return res;
}

/**
 * Get destination SIP URI.
 */
char * getDstSipUri(char * sipMsg) {
    char * begin = strstr(sipMsg, "To:");
    char * end = NULL;
    // SIP URI was not found in message
    if (begin == NULL) {
        //printf("Address not found.\n");
        return (NULL);
    }
    if(strstr(begin, "sip:") != NULL){
        begin = strstr(begin, "sip:");
    }
    else if(strstr(begin, "tel:") != NULL){
        begin = strstr(begin, "tel:");
    }
    else{
        return(NULL);
    }
    begin = begin + 4;
    if(strstr(begin, ";") != NULL){
        end = strstr(begin, ";");
        end--;
    }
    if(strstr(begin, ">") != NULL){
        end = strstr(begin, ">");
    }
    if (end == NULL) {
        end = strstr(begin, "\n");
        if(end == NULL)
            return(NULL);
    }
    char * res = malloc(end - begin + 1);
    memcpy(res, begin, end - begin);
    res[end - begin] = '\0';
    //printf("TO: %s\n", res);
    return res;
}

/**
 * Get RTP port(s) - "audio", "video", "text", "application", and "message"
 */
void getMediaPort(char * sipMsg, short unsigned int * portList){
    char * begin;
    char * end = NULL;
    char * res;
    const char *ports[5];
    ports[0] = "m=audio";
    ports[1] = "m=video";
    ports[2] = "m=text";
    ports[3] = "m=application";
    ports[4] = "m=message";
    
    memset(portList, 0, 5);
    
    for(int i = 0; i < 5; i++){
      begin = strstr(sipMsg, ports[i]);
      if(begin != NULL){
        begin = begin + strlen(ports[i]);
        end = strstr(begin, "\n");
        res = malloc(end - begin + 1);
        memcpy(res, begin, end - begin);
        res[end - begin] = '\0';
        portList[i] = atoi(res);
        free(res);
      }
    }
}

/**
 * Get request sequence number.
 */
unsigned short int getSeqNumber(char * sipMsg){
    char * begin = strstr(sipMsg, "CSeq: ");
    char * end = NULL;
    
    if (begin == NULL) {
        printf("CSeq not found.\n");
        return(0);
    }
    else {
        begin = begin + strlen("CSeq: ");
        end = strstr(begin, " ");
    }
    char * res = malloc(end - begin + 1);
    memcpy(res, begin, end - begin);
    res[end - begin] = '\0';
    int seqNum = atoi(res);
    free(res);
    return(seqNum);
}


/**
 * Check expitation in REGISTER message (if 0, message type is UNREGISTER)
 */
char * checkUnregister(char * sipMsg) {
    char * found = NULL;
    found = strstr(sipMsg, "Expires: 0");
    if(found == NULL)
      found = strstr(sipMsg, ";expires=0");
    return (found);
}

/**
 * Find message type substring.
 */
bool checkMsgType(char * sipMsg, char * substring) {
    char * firstLine;
    if(strstr(sipMsg, "\n") != NULL){
      firstLine = strstr(sipMsg, "\n");
      char * type = strstr(sipMsg, substring);
        if (type != NULL && firstLine - type > 0)
          return (true);
    }
    else {
      if(strstr(sipMsg, substring) != NULL)
        return(true);
    }
    return (false);
}

/**
 * Get message type.
 */
int getMessageType(char * sipMsg) {
    if (checkMsgType(sipMsg, "REGISTER")) {
        if (checkUnregister(sipMsg) != NULL) {
            //printf("UNREGISTER\n");
            return (REQUEST_UNREGISTER);
        }
        //printf("REGISTER\n");
        return (REQUEST_REGISTER);
    }
    if (checkMsgType(sipMsg, "INVITE")) {
        //printf("INVITE\n");
        return (REQUEST_INVITE);
    }
    if (checkMsgType(sipMsg, "PRACK")) {
        //printf("PRACK\n");
        return (REQUEST_PRACK);
    }
    if (checkMsgType(sipMsg, "ACK")) {
        //printf("ACK\n");
        return (REQUEST_ACK);
    }
    if (checkMsgType(sipMsg, "CANCEL")) {
        //printf("CANCEL\n");
        return (REQUEST_CANCEL);
    }
    if (checkMsgType(sipMsg, "BYE")) {
        //printf("BYE\n");
        return (REQUEST_BYE);
    }
    if (checkMsgType(sipMsg, "OPTIONS")) {
        return (REQUEST_OPTIONS);
    }
    if (checkMsgType(sipMsg, "SUBSCRIBE")) {
        return (REQUEST_SUBSCRIBE);
    }
    if (checkMsgType(sipMsg, "NOTIFY")) {
        return (REQUEST_NOTIFY);
    }
    if (checkMsgType(sipMsg, "PUBLISH")) {
        return (REQUEST_PUBLISH);
    }
    if (checkMsgType(sipMsg, "SIP/2.0 183")) {
        //printf("183\n");
        return (RESPONSE_SESSION_PROGRESS);
    }
    if (checkMsgType(sipMsg, "SIP/2.0 1")) {
        //printf("1xx PROVISIONAL\n");
        return (RESPONSE_PROVISIONAL);
    }
    if (checkMsgType(sipMsg, "SIP/2.0 2")) {
        //printf("2xx OK\n");
        return (RESPONSE_SUCCESS);
    }
    if (checkMsgType(sipMsg, "SIP/2.0 3")) {
        //printf("3xx ERROR\n");
        return (RESPONSE_ERROR);
    }
    if (checkMsgType(sipMsg, "SIP/2.0 4")) {
        //printf("4xx ERROR\n");
        return (RESPONSE_ERROR);
    }
    if (checkMsgType(sipMsg, "SIP/2.0 5")) {
        //printf("5xx ERROR\n");
        return (RESPONSE_ERROR);
    }
    if (checkMsgType(sipMsg, "SIP/2.0 6")) {
        //printf("6xx ERROR\n");
        return (RESPONSE_ERROR);
    }
    return(-1);
}

methodType getSeqMethod(char * sipMsg){
    char * begin = strstr(sipMsg, "CSeq: ");
    char * end = NULL;
    
    if (begin == NULL) {
        printf("CSeq not found.\n");
        return(0);
    }
    else {
        begin = begin + strlen("CSeq: ");
        begin = strstr(begin, " ");
        begin++;
        end = strstr(begin, "\n");
    }
    char * res = malloc(end - begin + 1);
    memcpy(res, begin, end - begin);
    res[end - begin] = '\0';
    int type = getMessageType(res);
    free(res);
    return(type);
}

/**
 * Send register message to IRI Core
 */
void printIriMessageRegister(char * type, char * ipAddress, char * sipUri, iri_iif_hash_table_item_t *table_item) {
    char * msg = (char *) malloc(strlen(type) + strlen(ipAddress) + strlen(sipUri) + 254);
    char addressType[5];
    // check address type
    if (strchr(ipAddress, ':') == NULL)
        sprintf(addressType, "IPv4");
    else
        sprintf(addressType, "IPv6");
    
    struct timeval tv;
    double time_in_micros;
    gettimeofday(&tv,NULL);
    time_in_micros = tv.tv_sec + 1000000.0 / tv.tv_usec;
    // create IRI message
    sprintf(msg, "('SIP', %.2f, '%s', '', [('UDP', ('%d.%d.%d.%d', %d, '%d.%d.%d.%d', %d)), ('SIP URI', '%s')], [], []", time_in_micros, type, table_item->key.items.ip_2.v4_long.data.d8[0], table_item->key.items.ip_2.v4_long.data.d8[1], table_item->key.items.ip_2.v4_long.data.d8[2], table_item->key.items.ip_2.v4_long.data.d8[3], table_item->key.items.port_2, table_item->key.items.ip_1.v4_long.data.d8[0], table_item->key.items.ip_1.v4_long.data.d8[1], table_item->key.items.ip_1.v4_long.data.d8[2], table_item->key.items.ip_1.v4_long.data.d8[3], table_item->key.items.port_1, sipUri);
    printf("%s\n", msg);
    if(strcmp(type, "END") == 0)
        iri_iif_modules_send_iri_message(msg, 0, table_item, 4);
    else
        iri_iif_modules_send_iri_message(msg, 1, table_item, 4);
    free(msg);
    printf("%d \n", __LINE__);
}

/**
 * Send call message to IRI Core
 */
void printIriMessageCall(char * type, char * message, userInfoPtr curr, iri_iif_hash_table_item_t *table_item) {
    printf("IRI zprava %s\n", type);
    char * msg = (char *) malloc(strlen(type) + strlen(message) + strlen(curr->ipAddress1) + strlen(curr->sipUri1)
    + strlen(curr->ipAddress2) + strlen(curr->sipUri2) + 512);
    char srcAddressType[5];
    char dstAddressType[5];
    // check source address type
    if (strchr(curr->ipAddress1, ':') == NULL)
        sprintf(srcAddressType, "IPv4");
    else
        sprintf(srcAddressType, "IPv6");

    // check destination address type
    if (strchr(curr->ipAddress2, ':') == NULL)
        sprintf(dstAddressType, "IPv4");
    else
        sprintf(dstAddressType, "IPv6");
    
    struct timeval tv;
    double time_in_micros;
    gettimeofday(&tv,NULL);
    time_in_micros = tv.tv_sec + 1000000.0 / tv.tv_usec;
    // create IRI message
    if(strcmp(type, "REPORT") == 0){
        sprintf(msg, "('SIP', %.2f, '%s', '%s', [('SIP URI', '%s'), ('SIP URI', '%s')], [], []", 
            time_in_micros, type, message, curr->sipUri1, curr->sipUri2);
    }
    else {
        for(int i = 0; i < 5; i++){
            if(curr->rtpPort1[i] != 0 && curr->rtpPort2 != 0)
                sprintf(msg, "('SIP', %d, '%s', '%s', [('UDP', ('%s', %d, '%s', %d)), ('SIP URI', '%s'), ('SIP URI', '%s')], [], []", 
                    (int)time(NULL), type, message, curr->ipAddress1 , curr->rtpPort1[i], curr->ipAddress2, curr->rtpPort2[i], curr->sipUri1, curr->sipUri2);
        }
    }
    printf("%s\n", msg);
    iri_iif_modules_send_iri_message(msg, 0, table_item, 4);
    printf("%d\n", __LINE__);
    free(msg);
    printf("%d\n", __LINE__);
}

void processSip(char * srcAddress, char * dstAddress, char * sipMsg, void * extPointer, iri_iif_hash_table_item_t *table_item) {
    char * uriFrom = getSrcSipUri(sipMsg);
    char * uriTo = getDstSipUri(sipMsg);
    char * id = getCallId(sipMsg);
    int messageType = getMessageType(sipMsg);
    
    // probably not SIP packet
    if (uriFrom == NULL || id == NULL)
        return;

    userInfoPtr * dataPointer = (userInfoPtr *) extPointer;
    
    // flow unknown, create record
    if (*dataPointer == NULL) {
        userInfoPtr newUser = (userInfoPtr) malloc(sizeof (struct userInfoPtr));
        initUser(newUser, srcAddress, dstAddress, uriFrom);
        * dataPointer = newUser;
    }

    userInfoPtr currentUser = *dataPointer;
    transactionInfo * trans = NULL;
    
    // new message is REQUEST and there is too much old crap -> delete oldest crap
    if(messageType < RESPONSE_SESSION_PROGRESS && countTransactions(currentUser) >= 10){
      deleteFirstTransaction(currentUser);
    }
    
    // update destionation SIP URI
    if((messageType == REQUEST_INVITE || messageType == RESPONSE_SESSION_PROGRESS) && currentUser->sipUri2 == NULL){
      currentUser->sipUri2 = (char *) malloc(strlen(uriTo) + 1);
      strcpy(currentUser->sipUri2, uriTo);
    }
    
    // get RTP ports
    if(messageType == REQUEST_INVITE){
      getMediaPort(sipMsg, currentUser->rtpPort1);
    }
    
    if(messageType == REQUEST_BYE){
      printIriMessageCall("REPORT", "BYE was sent", currentUser, table_item);
    }
    
    /// register, invite, bye, options
    if(messageType == REQUEST_REGISTER || messageType == REQUEST_INVITE || messageType == REQUEST_BYE || messageType == REQUEST_OPTIONS){
      if(findTransaction(currentUser, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg)) == NULL){
        insertNewTransaction(currentUser, uriTo, id, getSeqNumber(sipMsg), messageType);
        //printf("transaction inserted, cnt: %d\n", countTransactions(currentUser));
      }
    }
    /// unregister
    else if(messageType == REQUEST_UNREGISTER){
      if(findTransaction(currentUser, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg)) == NULL){
        insertNewTransaction(currentUser, uriFrom, id, getSeqNumber(sipMsg), REQUEST_REGISTER);
        //printf("transaction inserted\n");
      }
    }
    /// 183 SESSION PROGRESS (RTP already running)
    else if(messageType == RESPONSE_SESSION_PROGRESS){
      trans = findTransaction(currentUser, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg));
      if(trans != NULL){
        getMediaPort(sipMsg, currentUser->rtpPort2);
        if(trans->method == REQUEST_INVITE && (currentUser->callStatus == NO_IRI_SENT || currentUser->callStatus == IRI_END_SENT)){
          printIriMessageCall("BEGIN", "183 Session progress", currentUser, table_item);
            currentUser->callStatus = IRI_BEGIN_SENT;
        }
      }
    }
    /// 200 OK
    else if(messageType == RESPONSE_SUCCESS){
      trans = findTransaction(currentUser, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg));
      if(trans != NULL){
        if(trans->method == REQUEST_REGISTER){
          if(currentUser->registerStatus == NO_IRI_SENT || currentUser->registerStatus == IRI_END_SENT){
            printIriMessageRegister("BEGIN", currentUser->ipAddress1, currentUser->sipUri1, table_item);
            currentUser->registerStatus = IRI_BEGIN_SENT;
          }
        }
        else if(trans->method == REQUEST_INVITE){
          getMediaPort(sipMsg, currentUser->rtpPort2);
          if(currentUser->sipUri2 == NULL){
            currentUser->sipUri2 = (char *) malloc(strlen(trans->uri2) + 1);
            strcpy(currentUser->sipUri2, trans->uri2);
          }
          if(currentUser->callStatus != IRI_BEGIN_SENT){
            printIriMessageCall("BEGIN", "200 OK", currentUser, table_item);
            currentUser->callStatus = IRI_BEGIN_SENT;
          }
          else{
            printIriMessageCall("CONTINUE", "200 OK", currentUser, table_item);
          }
        }
        else if(trans->method == REQUEST_BYE){
          if(currentUser->callStatus == IRI_BEGIN_SENT){
            printIriMessageCall("END", "", currentUser, table_item);
            currentUser->callStatus = IRI_END_SENT;
           }
         }
        else if(trans->method == REQUEST_OPTIONS){
          printIriMessageRegister("REPORT", currentUser->ipAddress1, currentUser->sipUri1, table_item);
        }
        else if(trans->method == REQUEST_UNREGISTER){
          if(currentUser->registerStatus == IRI_BEGIN_SENT){
            printIriMessageRegister("END", currentUser->ipAddress1, currentUser->sipUri1, table_item);
            currentUser->registerStatus = IRI_END_SENT;
          }
        }
        findAndDeleteTransaction(currentUser, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg));
      }
    }
    /// 2xx, 3xx, 4xx and 5xx errors
    else if(messageType == RESPONSE_ERROR){
      trans = findTransaction(currentUser, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg));
      if(trans != NULL){
        if(trans->method == REQUEST_REGISTER && currentUser->registerStatus == IRI_BEGIN_SENT){
          printIriMessageRegister("END", currentUser->ipAddress1, currentUser->sipUri1, table_item);
          currentUser->registerStatus = IRI_END_SENT;
        }
      }
    findAndDeleteTransaction(currentUser, id, getSeqNumber(sipMsg), getSeqMethod(sipMsg));
    }
    else{
      //printf("ignoring msg\n");
    }
    //printf("---------------------------------------------------------------------------\n");
    free(uriFrom);
    free(uriTo);
    free(id);
}
/**
 * Process packet
 */
int sip_process_packet(iri_iif_hash_table_item_t *table_item, iri_iif_offsets_t *offsets, unsigned char *packet, uint16_t packet_size,  void * input_dataPointer)
{
  // Allocate space for an address
  char src_address[ADDRESS_SIZE];
  char dst_address[ADDRESS_SIZE];
  // UDP header
  const struct udphdr *udp = NULL;
  static int cnt = 0;
  //
  char L3_type[4];
  u_int16_t type_link_proto;
  // Sizes
  int size_link_proto = 0;
  int size_ip = 0;
  int size_udp = sizeof(struct udphdr);
  
  /// Ethernet
  const struct ether_header * ether = (struct ether_header*) packet;
  size_link_proto = sizeof (struct ether_header);
  type_link_proto = ntohs(ether->ether_type);
  
  /// IPv4
  if (type_link_proto == 0x0800) {
    const struct ip *ip = (struct ip*) (packet + size_link_proto);
    size_ip = sizeof (struct ip);
    /// Check if the packet is UDP or TCP
    if (ip->ip_p == IPPROTO_UDP) {
      strcpy(L3_type, "udp");
      udp = (struct udphdr*) (packet + size_link_proto + size_ip);
    }
    else
      return 0;
    /// get IP address
    if (inet_ntop(AF_INET, &(ip->ip_src), src_address, 64) == NULL) {
      fprintf(stderr, "Cannot get source IP address\n");
      return 0;
    }
    if (inet_ntop(AF_INET, &(ip->ip_dst), dst_address, 64) == NULL) {
	  fprintf(stderr, "Cannot get destination IP address\n");
	  return 0;
    }
  }
   /// IPv6
  else if (type_link_proto == 0x86dd) {
    const struct ip6_hdr *ip = (struct ip6_hdr*) (packet + size_link_proto);
    size_ip = sizeof (struct ip6_hdr);
    /// Check if the packet is UDP
    if (ip->ip6_ctlun.ip6_un1.ip6_un1_nxt == IPPROTO_UDP){
	  strcpy(L3_type, "udp");
      udp = (struct udphdr*) (packet + size_link_proto + size_ip);
	}
	else
	  return 0;
    
    if (inet_ntop(AF_INET6, &(ip->ip6_src), src_address, 64) == NULL) {
      fprintf(stderr, "Cannot get IP address\n");
      return 0;
	}
  }
  else {
    fprintf(stderr, "Unknown Ethernet type\n");
    return 0;
  }
  char * sipRequest = NULL;
  
  if (strcmp(L3_type, "udp") == 0) {
    cnt = cnt + 1;
    sipRequest = (char *) udp + size_udp;
  }
  processSip(src_address, dst_address, sipRequest, input_dataPointer, table_item);
  return 0;
}

int sip_destroy_data(iri_iif_hash_table_item_t *table_item, void * data){
  userInfoPtr * dataPointer = (userInfoPtr *) data;
  // nothing to delete
  if (*dataPointer == NULL) {
    return;
  }
  // delete all transactions
  userInfoPtr currentUser = *dataPointer;
  while(currentUser->transactions != NULL){
    transactionInfo * deleteTransaction = currentUser->transactions;
    currentUser->transactions = currentUser->transactions->next;

    free(deleteTransaction->callId);
    free(deleteTransaction->uri2);
    free(deleteTransaction);
  }
  if(currentUser->callStatus == IRI_BEGIN_SENT){
    printIriMessageCall("END", "SDM timeout", currentUser, table_item);
  }
  if(currentUser->registerStatus == IRI_BEGIN_SENT){
    printIriMessageRegister("END", currentUser->ipAddress1, currentUser->sipUri1, table_item);
  }
      
  // delete flow info
  free(currentUser->ipAddress1);
  free(currentUser->ipAddress2);
  free(currentUser->sipUri1);
  free(currentUser->sipUri2);
  free(*dataPointer);
  *dataPointer = NULL;
  return(0);
}
