
#include "smtp.h"

// v pakete vyhlada emailovu adresu odosielatela
int get_mail(const u_char * packet, unsigned int dataStart, unsigned int length, char * result)
{
    char /** begin, * end, */state = 0;
    unsigned int beginIndex = 0, endIndex = 0, i;
    
    
    // najprv si najdem pozicie zaciatku a konca e-mailu
    for(i = dataStart; i < dataStart + length; i++)
    {
        // hladam zaciatok e-mailu
		if(state == 0 && packet[i] == '<')
		{
			beginIndex = i + 1;
			state = 1;
		}
        // hladam koniec e-mailu
		else if(state == 1 && packet[i] == '>')
		{
			endIndex = i;
			state = 2;
			break;
		}
	}
    
    // ak som nenasiel znaky < a > , tak radsej sa nepokusam nic rozparsovat
    if( beginIndex * endIndex == 0) {
        sprintf(result, "unknown");
        return 0;
    } else {
        sprintf(result, "%.*s", endIndex - beginIndex, &packet[beginIndex]);
        return endIndex - beginIndex;
    }
}

// vypocita sa hodnota ACK (v TCP hlavicke), ktora bude potvrdzovat aktualne prijaty paket
unsigned int calc_ack_from_seq(unsigned int positionSeqNum, unsigned int packetLength, unsigned int positionData)
{
	return positionSeqNum + (packetLength - positionData);
    //return convert_seqack(packet, positionSeqNum) + (packetLength - positionData);
}

// najdenie e-mailovej adresy v sprave a ulozenie k aktualnemu toku
email_ptr email_create(const u_char * packet, unsigned int positionData, unsigned int dataLength, unsigned int ackNum)
{
    char * emailAddressString;
	unsigned short int length;
	email_ptr mail;
	
	emailAddressString = calloc( dataLength - 9, sizeof(char) ); //  = strlen("RCPT TO" + "<" + ">") alebo strlen("MAIL FROM" + "<" + ">")
    length = get_mail(packet, positionData, dataLength, emailAddressString);
    
    mail = calloc(1, sizeof(struct email));
    mail->length = length;
    mail->emailString = emailAddressString;
    mail->ackNum = ackNum;
    mail->next = NULL;
    mail->confirmed = 0;
    
    //free(emailAddressString);
    
    return mail;
}

// vyhladanie ci bolo ACK uz prijate
char save_find_ack(iri_iif_module_smtp_data_ptr save, unsigned int ackNum)
{
    ack_ptr tmp = save->acks;
    
    while(tmp != NULL)
    {
        if(tmp->value == ackNum)
            return 1;
        tmp = tmp->next;
    }
    
    return 0;
}

// ulozit e-mail odosielatela
void save_set_sender (iri_iif_module_smtp_data_ptr save, const u_char * packet, unsigned int positionSeqNum, unsigned int dataLength, unsigned int positionData)
{
    email_ptr mail;
    unsigned int ackNum; 
    
    ackNum = calc_ack_from_seq(positionSeqNum, dataLength, positionData);
    
    // vytvorenie struktury s e-mailom
    mail = email_create(packet, positionData, dataLength, ackNum);
    
    // v pripade, ze som prijal ACK pred samotnym e-mailom, potvrdim ho
    if(save_find_ack(save, ackNum) == 1)
        mail->confirmed = 1;
    
    // ak uz je nejaky e-mail ulozeny, vymazem ho
    if(save->mail != NULL)
        free(save->mail);
    
    // vlozim do uloziska e-mail
    save->mail = mail;
}

void save_add_receiver (iri_iif_module_smtp_data_ptr save, const u_char * packet, unsigned int positionSeqNum, unsigned int dataLength, unsigned int positionData)
{
    email_ptr mail, tmp;
    unsigned int ackNum; 
    
    ackNum = calc_ack_from_seq(positionSeqNum, dataLength, positionData);
    
    // vytvorenie struktury s e-mailom
    mail = email_create(packet, positionData, dataLength, ackNum);
    
	// ak este nebol ulozeny ziadny e-mail prijimatela
    if(save->rcpts == NULL)
    {
        if(save_find_ack(save, ackNum) == 1)
            (save->mail)->confirmed = 1;
        save->rcpts = mail;
    }
	// nejedna sa o prveho prijimatela, pridam na koniec zoznamu
    else
    {
        tmp = save->rcpts;
        while(tmp != NULL)
        {
	// rovnaky e-mail prijimatela uz mam ulozeny
            if(strcmp(tmp->emailString, mail->emailString) == 0) {
				free(mail);
                return;
			}
            if(tmp->next == NULL)
            {
                if(save_find_ack(save, ackNum) == 1)
                    (save->mail)->confirmed = 1;
                tmp->next = mail;
                return;
            }
            tmp = tmp->next;
        }
    }
}

// pridam nove ACK cislo do zoznamu prijatych ACK sprav
void check_ack_list(iri_iif_module_smtp_data_ptr save, unsigned int ackNum)
{
	// prijate ACK nepatri k uz prijatemu e-mailu odosielatela ani prijimatela
        ack_ptr ackTemp, ackLast;
        ackTemp = save->acks;
        // skontrolujem zoznam uz prijatych ACK sprav
        while(ackTemp != NULL)
        {
                // rovnake ACK uz som prijal v minulosti
            if(ackTemp->value == ackNum)
                return;
            ackLast = ackTemp;
            ackTemp = ackTemp->next;
        }

        // vlozeni novej polozky do zoznamu prijatych ACK sprav
        ack_ptr ackStruct = malloc(sizeof(struct ack));
        ackStruct->value = ackNum;
        ackStruct->next = NULL;

        // jedna sa o prvu ukladanu ACK spravu
        if(save->acks == NULL)
            save->acks = ackStruct;
        // ulozenie na koniec zoznamu
        else
            ackLast->next = ackStruct;

}

// ulozim si ACKnum cislo k spojeniu (ak ACK pride pred samotnou spravou, ktoru potvrdzuje)
void save_add_ack (iri_iif_module_smtp_data_ptr save, unsigned int ackNum)
{
	// potvrdenie e-mailu odosielatela
    if(save->mail != NULL && (save->mail)->ackNum == ackNum)
    {
        (save->mail)->confirmed = 1;
	return;
    }
    
    // potvrdenie e-mailu prijimatela
            email_ptr tmp = save->rcpts;
		// hladam e-mail, ktory chcem potvrdit
            while(tmp != NULL)
            {
                if(tmp->ackNum == ackNum)
                {
			// potvrdim dany e-mail
                    tmp->confirmed = 1;
                    return;
                }
                tmp = tmp->next;
            } 
       
	// vlozim ACK do zoznamu
	check_ack_list(save, ackNum);
}

// odstranim zoznam e-mailov
void delete_email_list( email_ptr input_email )
{
    email_ptr mail, mailTemp;
    mail = input_email;
    
    while(mail != NULL)
    {
		mailTemp = mail->next;
		
		free( mail->emailString );
		free( mail );
		
		mail = mailTemp;
	}
}

// vymazanie informacii o aktualnom spojeni (napr pri resete)
void save_delete_inner_structures (iri_iif_module_smtp_data_ptr input_save, char acksToo)
{
    ack_ptr ackItem, ackItemNext;
    
	// odstranim vsetky ulozene e-maily
    delete_email_list( input_save->mail );
    delete_email_list( input_save->rcpts );
    
	// odstranim aj zoznam ulozenych ACKnum
    if(acksToo == 1)
    {
		ackItem = input_save->acks;
		
		while(ackItem != NULL)
		{
			ackItemNext = ackItem->next;
			free( ackItem );
			ackItem = ackItemNext;
		}
	}
}



// vyhladavnie substringu v stringu
char str_match(uint8_t * input_packet, unsigned int packetLength, short int data, const char * string)
{
    char code;
    short int length = (short int) strlen(string);
	const char * packet = (const char *) input_packet;
	
    // velkost dat v pakete je mensi ako string, ktory v nom vyhladavame
	if( packetLength < length )
	{
		return 0;
	}
	
    // vytvorienie substringu so shodnou velkostou ako hladany retazec
    char * dataCopy = strndup(((char *) packet) + data, length);
    code = strcmp( dataCopy, string);
    free(dataCopy);
	
	return code == 0 ? 1 : 0;
}


// kontrola ci paket obsahuje data v base64 podobe
char base64_data(const u_char * packet, unsigned int packetLength, short int data)
{
    int i;
    char ascii;
    
    packetLength -= 2; // posledne dva znaky su \r \n
    if (packetLength % 3 != 0)
    {
        for (i = data; i < packetLength; i++)
        {
            ascii = packet[i];
            if ( ( ascii < '0' || ascii > '9' )
             && ( ascii < 'a' || ascii > 'z' )
             && ( ascii < 'A' || ascii > 'Z' )
             && ascii != '+' && ascii != '/' && ascii != '=' )
                return 0;
        }
        return 1;
    }
    else
        return 0;
}
/*
unsigned int convert_seqack(const u_char * packet, unsigned int position)
{
    short int i;
    unsigned int result;
    
    result = 0;
    
    for(i = 0; i < 4; i++)
    {
        result += (int) packet[position + i];
        result *= 256;
    }
    
    return result;
}
*/
// zkonvertovanie IP adresy do textovej podoby
char * convert_binary_to_ipv4(uint8_t ip[4])
{
	char * ipv4;
	ipv4 = calloc (20, sizeof(char));
    unsigned int x, i;
    char ipv4Part[5];
    
    //strcpy(ipv4, "");
    
    for(i = 0; i < 4; i++)
    {
        x = ip[i];
        if(x > 4294967040)
            x -= 4294967040;
        
        if(i != 0)
            strcat(ipv4, ".");
        sprintf(ipv4Part, "%u", x);
        strcat(ipv4, ipv4Part);
    }
    
    return ipv4;
}

// zkonvertovanie portu do textovej podoby
char * convert_binary_to_port(uint16_t input)
{
    char * port;
	port = calloc (8, sizeof(char));
    
    //strcpy(port, "");
    sprintf(port, "%u", input);
    return port;
}

// IRI sprava je ukladana do pola, ktoreho velkost sa moze dynamicky zvacsovat
void extend_array_if_necessary(char * array, unsigned int * actualSize, unsigned int * maximumSize, unsigned int newDataLength, unsigned int incrementSize)
{
    *actualSize += newDataLength;
    if(*actualSize > *maximumSize)
    {
        *maximumSize = (*actualSize) + incrementSize;
        array = realloc(array, sizeof(char) * (*maximumSize) );
    }
}


// z e-mailovych adries ulozenych v ramci spojenia sa vytvori parameter IRI spravy
void create_smtp_nid(iri_iif_module_smtp_data_ptr save, char * message)
{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
    unsigned int bytesAvailable = 10000, bytesTaken = 7;
    
    strcpy(message, "");
    
    strcat(message, ", ('E-mail message'");
    
    if( save->mail != NULL)
    {
        extend_array_if_necessary(message, &bytesTaken, &bytesAvailable, 24 + ( save->mail)->length, 100);
        strcat(message, ", ( ('E-mail address', '");
//printf("%s %u: email - %s\n", __FILE__, __LINE__, (save->mail)->emailString);
        strcat(message, ( save->mail)->emailString );
        strcat(message, "')");
    }
    else
    {
        bytesTaken += 31;
        strcat(message, ", ( ('E-mail address', 'unknown')");
    }
    extend_array_if_necessary(message, &bytesTaken, &bytesAvailable, 3, 100);
    strcat(message, ", [");
    
    email_ptr tmp = save->rcpts;
    while(tmp != NULL)
    {
        extend_array_if_necessary(message, &bytesTaken, &bytesAvailable, 24 + tmp->length, 100);
        if( tmp != save->rcpts )
			strcat(message, ", ");
        strcat(message, "('E-mail address', '");
        strcat(message, tmp->emailString );
        strcat(message, "')");
        tmp = tmp->next;
    }
    
    extend_array_if_necessary(message, &bytesTaken, &bytesAvailable, 36, 100);
    
    strcat(message, "]");
    strcat(message, ", ('E-mail Message-ID', 'unknown')");
    strcat(message, ") )");
    
    //return message;
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
}


// vytvorenie IRI spravy
void send_to_socket(const char * iriType, const char * text, iri_iif_module_smtp_data_ptr * input_dataPointer, iri_iif_hash_table_item_t *table_item, iri_iif_module_smtp_data_ptr input_save)
{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
    unsigned int bytesAvailable = 20000;//, bytesTaken = 37;
    char * message;
    message = calloc(bytesAvailable, sizeof(char) );
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
    char timeArray[20];
    struct timeval tv;
    double time_in_micros;
    
    
    gettimeofday(&tv,NULL);
    time_in_micros = tv.tv_sec + 1000000.0 / tv.tv_usec;
    sprintf(timeArray, "%.2f", time_in_micros);
    
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
    strcpy(message, "");
    strcat(message, "('smtp', ");
    strcat(message, timeArray);
    strcat(message, ", '");
    strcat(message, iriType);
    strcat(message, "', '");
    
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
    
	char * ip1 = NULL, *ip2 = NULL, *port1 = NULL, *port2 = NULL;
	
    // User IP:port 
    strcat(message, "User ");
	if( (*input_dataPointer)->clientServer == 0 )
	{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		ip1 = convert_binary_to_ipv4( table_item->key.items.ip_1.v4_long.data.d8 );
		strcat(message, ip1);
		strcat(message, ":");
		port1 = convert_binary_to_port( table_item->key.items.port_1 );
		strcat(message, port1);
	}
	else
	{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		ip2 = convert_binary_to_ipv4( table_item->key.items.ip_2.v4_long.data.d8 );
		strcat(message, ip2);
		strcat(message, ":");
		port2 = convert_binary_to_port( table_item->key.items.port_2 );
		strcat(message, port2);
	}
    strcat(message, " ");
    
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
    //extend_array_if_necessary(message, &bytesTaken, &bytesAvailable, strlen(text), 400);
    strcat(message, text);
    //extend_array_if_necessary(message, &bytesTaken, &bytesAvailable, 205, 220);
    
	if( memcmp(iriType, "CONTINUE", 8) == 0 )
	{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		strcat(message, " from ");
		
		if( input_save->mail != NULL)
		{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
			strcat(message, ( input_save->mail)->emailString );
		}
		else
		{
			strcat(message, "unknown");
		}
		
		strcat(message, " to ");
		
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		email_ptr tmp = input_save->rcpts;
		while(tmp != NULL)
		{
			if( tmp != input_save->rcpts )
				strcat(message, ", ");
			strcat(message, tmp->emailString );
			tmp = tmp->next;
		}
		
		strcat(message, " on SMTP");
	}
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
	
	
    // server IP:port 
    strcat(message, " server ");
	if( (*input_dataPointer)->clientServer == 0 )
	{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		ip2 = convert_binary_to_ipv4( table_item->key.items.ip_2.v4_long.data.d8 );
		strcat(message, ip2);
		strcat(message, ":");
		port2 = convert_binary_to_port( table_item->key.items.port_2 );
		strcat(message, port2);
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
	}
	else
	{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		ip1 = convert_binary_to_ipv4( table_item->key.items.ip_1.v4_long.data.d8 );
		strcat(message, ip1);
		strcat(message, ":");
		port1 = convert_binary_to_port( table_item->key.items.port_1 );
		strcat(message, port1);
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
	}
	
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
	
	strcat(message, "', [");
	
	if( memcmp(iriType, "END", 3) != 0 )
	{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		if( (*input_dataPointer)->clientServer == 0 )
		{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
			strcat(message, "('TCP', ('");
			ip1 = convert_binary_to_ipv4( table_item->key.items.ip_1.v4_long.data.d8 );
			strcat(message, ip1);
			
			strcat(message, "', ");
			port1 = convert_binary_to_port( table_item->key.items.port_1 );
			strcat(message, port1);
			
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
			strcat(message, ", '");
			ip2 = convert_binary_to_ipv4( table_item->key.items.ip_2.v4_long.data.d8 );
			strcat(message, ip2);
			
			strcat(message, "', ");
			port2 = convert_binary_to_port( table_item->key.items.port_2 );
			strcat(message, port2);
			
			strcat(message, "))");
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		}
		else
		{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
			strcat(message, "('TCP', ('");
			ip2 = convert_binary_to_ipv4( table_item->key.items.ip_2.v4_long.data.d8 );
			strcat(message, ip2);
			
			strcat(message, "', ");
			port2 = convert_binary_to_port( table_item->key.items.port_2 );
			strcat(message, port2);
			
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
			strcat(message, ", '");
			ip1 = convert_binary_to_ipv4( table_item->key.items.ip_1.v4_long.data.d8 );
			strcat(message, ip1);
			
			strcat(message, "', ");
			port1 = convert_binary_to_port( table_item->key.items.port_1 );
			strcat(message, port1);
			
			strcat(message, "))");
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		}
	}
    
	
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
	
	free(ip1);
	free(ip2);
	free(port1);
	free(port2);
	
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
    if(input_save != NULL)
    {
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		char * aditionalData = calloc(10000, sizeof(char));
    
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		create_smtp_nid(input_save, aditionalData);
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
        //extend_array_if_necessary(message, &bytesTaken, &bytesAvailable, strlen(aditionalData), 10);
        strcat(message, aditionalData);
        
        free(aditionalData);
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
    }
	strcat(message, "]");
    
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
    //extend_array_if_necessary(message, &bytesTaken, &bytesAvailable, 3, 1);
    
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_APP_MODULES == 1
		printf("%s %d: Modul odosiela IRI spravu - %s\n", __FILE__, __LINE__, message);
	#endif
	
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
	if( strcmp( (char *) iriType, "CONTINUE") == 0)
	{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		iri_iif_modules_send_iri_message(message, 1, table_item, 4); // 4 = IPv4
	}
	else
	{
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
		iri_iif_modules_send_iri_message(message, 0, table_item, 4); // 4 = IPv4
	}
	
	free( message );
//fprintf(stderr, "%s %d\n", __FILE__, __LINE__);
}


// ulozenie zaznamu o stave toku do SDM
void save_create(iri_iif_module_smtp_data_ptr * input_dataPointer, char state, char clientServer)
{
    *input_dataPointer = calloc( 1, sizeof(struct iri_iif_module_smtp_data));
    (*input_dataPointer)->state = state;
    (*input_dataPointer)->clientServer = clientServer;
    (*input_dataPointer)->mail = NULL;
    (*input_dataPointer)->rcpts = NULL;
    (*input_dataPointer)->acks = NULL;
}

// funkcia obsahuje konecny automat pre analyzu protokolu SMTP
int smtp_process_packet(iri_iif_hash_table_item_t *table_item, iri_iif_offsets_t *offsets, unsigned char *packet, uint16_t packet_size, void * input_dataPointer)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_APP_MODULES == 1
		printf("%s %d: smtp_process_packet(", __FILE__, __LINE__);
		
		// vypis obsahu paketu
		int i;
		for(i = 0; i < packet_size; i++)
		{
			if( *(packet + i) > 32 && *(packet + i) < 127 )
				printf("%c", *(packet + i));
			else
				printf(".");
		}
		printf(")\n");
	#endif
	//return MODULE_WANTS_TO_ANALYZE_THIS_FLOW;
	// prekovertovanie typu ukazatela
	iri_iif_module_smtp_data_ptr * dataPointer = (iri_iif_module_smtp_data_ptr *) input_dataPointer;
	
	// pretypovani na tcp strukturu pro dalsi zpracovani
	unsigned char *tcphdr_start = (packet + offsets->l4_start);
	struct tcphdr * tcpHeader;
	tcpHeader = (struct tcphdr *) tcphdr_start;
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_APP_MODULES == 1
		printf("%s %d: Stav spojenia pred spracovanim paketu - ", __FILE__, __LINE__);
		if( *dataPointer == NULL )
		{
			printf("0");
		}
		else
		{
			printf("%u", (*dataPointer)->state);
		}
		printf("\n");
	#endif
	
	// k danemu toku este nie su ulozene ziadne data
    if(*dataPointer == NULL)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "220"))
        {
			// ulozenie stavovej informacie pre novy (aktualny) tok
            save_create(dataPointer, 2, TRAFFIC_FROM_SERVER);
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "EHLO")
        ||
        str_match(packet, offsets->data_length, offsets->data_offset, "HELO"))
        {
			// ulozenie stavovej informacie pre novy (aktualny) tok
            save_create(dataPointer, 3, TRAFFIC_FROM_CLIENT);
        }
		else if( offsets->data_length != 0 )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_APP_MODULES == 1
				printf("%s %d: Spracovavany tok neobsahuje SMTP protokol\n", __FILE__, __LINE__);
			#endif
			
			// podla prveho paketu nesuceho data som nedetekoval SMTP protokol
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
		}
    }
    else if( (*dataPointer)->state == 2)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "250"))
        {
            send_to_socket("REPORT", "has connected to the SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 4;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "EHLO") || str_match(packet, offsets->data_length, offsets->data_offset, "HELO"))
        {
            (*dataPointer)->state = 3;
        }
    }
    else if( (*dataPointer)->state == 3)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "250"))
        {
            send_to_socket("REPORT", "has connected to the SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 4;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "AUTH"))
        {
            send_to_socket("REPORT", "has connected to the SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 5;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "MAIL")
         || str_match(packet, offsets->data_length, offsets->data_offset, "Mail"))
        {
            send_to_socket("REPORT", "has connected to the SMTP", dataPointer, table_item, NULL);
            
            // v prijatom pakete vyhladam adresu odosielatela a ulozim si ju
            save_set_sender(*dataPointer, packet, tcpHeader->th_seq, offsets->data_length, offsets->data_offset);
            (*dataPointer)->state = 8;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "5"))
        {
            send_to_socket("REPORT", "unsuccessfully tried connect to SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 2;
        }
        else if( ( tcpHeader->th_flags & 1 ) > 0
         || str_match(packet, offsets->data_length, offsets->data_offset, "221")
         || str_match(packet, offsets->data_length, offsets->data_offset, "QUIT") )
        {
            send_to_socket("REPORT", "has disconnected from the SMTP", dataPointer, table_item, NULL);
            
            // odstranenie vsetkych dat, ktore si modul uklada pre dany tok
            smtp_destroy_data( table_item, dataPointer );
            
            // po detekcie ukonceni spojenia uz modul nema zaujem o dalsie pakety
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "STARTTLS"))
        {
            (*dataPointer)->state = 11;
        }
    }
    else if((*dataPointer)->state == 4)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "AUTH"))
        {
            (*dataPointer)->state = 5;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "MAIL")
         || str_match(packet, offsets->data_length, offsets->data_offset, "Mail"))
        {
            // v prijatom pakete vyhladam adresu odosielatela a ulozim si ju
            save_set_sender(*dataPointer, packet, tcpHeader->th_seq, offsets->data_length, offsets->data_offset);
            (*dataPointer)->state = 8;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "250"))
        {
			// ulozim si sekvencne cislo spravy, ktorej prijem sa prave potvrdil
            save_add_ack (*dataPointer, tcpHeader->th_ack);
            (*dataPointer)->state = 8;
        }
        else if( ( tcpHeader->th_flags & 1 ) > 0
         || str_match(packet, offsets->data_length, offsets->data_offset, "221")
         || str_match(packet, offsets->data_length, offsets->data_offset, "QUIT") )
        {
            send_to_socket("REPORT", "has disconnected from the SMTP", dataPointer, table_item, NULL);
            
            // odstranenie vsetkych dat, ktore si modul uklada pre dany tok
            smtp_destroy_data( table_item, dataPointer );
            
            // po detekcie ukonceni spojenia uz modul nema zaujem o dalsie pakety
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "STARTTLS"))
        {
            (*dataPointer)->state = 11;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "334")
         || base64_data(packet, offsets->data_length, offsets->data_offset))
        {
            (*dataPointer)->state = 6;
        }
    }
    
    else if( (*dataPointer)->state == 5)
    {
        if( ( tcpHeader->th_flags & 1 ) > 0
         || str_match(packet, offsets->data_length, offsets->data_offset, "221")
         || str_match(packet, offsets->data_length, offsets->data_offset, "QUIT") )
        {
            send_to_socket("REPORT", "has disconnected from the SMTP", dataPointer, table_item, NULL);
            
            // odstranenie vsetkych dat, ktore si modul uklada pre dany tok
            smtp_destroy_data( table_item, dataPointer );
            
            // po detekcie ukonceni spojenia uz modul nema zaujem o dalsie pakety
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "334")
         || base64_data(packet, offsets->data_length, offsets->data_offset))
        {
            (*dataPointer)->state = 6;
        }
    }
    else if((*dataPointer)->state == 6)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "AUTH"))
        {
            send_to_socket("REPORT", "was unsuccessful in authentication on SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 5;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "MAIL")
         || str_match(packet, offsets->data_length, offsets->data_offset, "Mail"))
        {
            send_to_socket("BEGIN", "was successfully authenticated on SMTP", dataPointer, table_item, NULL);
            // v prijatom pakete vyhladam adresu odosielatela a ulozim si ju
            save_set_sender(*dataPointer, packet, tcpHeader->th_seq, offsets->data_length, offsets->data_offset);
            (*dataPointer)->state = 8;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "235"))
        {
            send_to_socket("BEGIN", "was successfully authenticated on SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 8;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "535"))
        {
            send_to_socket("REPORT", "was unsuccessful in authentication on SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 7;
        }
        else if( ( tcpHeader->th_flags & 1 ) > 0
         || str_match(packet, offsets->data_length, offsets->data_offset, "221")
         || str_match(packet, offsets->data_length, offsets->data_offset, "QUIT") )
        {
            send_to_socket("REPORT", "has disconnected from the SMTP", dataPointer, table_item, NULL);
            
            // odstranenie vsetkych dat, ktore si modul uklada pre dany tok
            smtp_destroy_data( table_item, dataPointer );
            
            // po detekcie ukonceni spojenia uz modul nema zaujem o dalsie pakety
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
        }
    }
    else if((*dataPointer)->state == 7)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "AUTH"))
        {
            (*dataPointer)->state = 5;
        }
        else if( ( tcpHeader->th_flags & 1 ) > 0
         || str_match(packet, offsets->data_length, offsets->data_offset, "221")
         || str_match(packet, offsets->data_length, offsets->data_offset, "QUIT") )
        {
            send_to_socket("REPORT", "has disconnected from the SMTP", dataPointer, table_item, NULL);
            
            // odstranenie vsetkych dat, ktore si modul uklada pre dany tok
            smtp_destroy_data( table_item, dataPointer );
            
            // po detekcie ukonceni spojenia uz modul nema zaujem o dalsie pakety
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "334")
         || base64_data(packet, offsets->data_length, offsets->data_offset))
        {
            (*dataPointer)->state = 6;
        }
    }
    else if((*dataPointer)->state == 8)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "250"))
        {
			// ulozim si sekvencne cislo spravy, ktorej prijem sa prave potvrdil
            save_add_ack (*dataPointer, tcpHeader->th_ack);
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "MAIL")
         || str_match(packet, offsets->data_length, offsets->data_offset, "Mail"))
        {
            // v prijatom pakete vyhladam adresu odosielatela a ulozim si ju
            save_set_sender(*dataPointer, packet, tcpHeader->th_seq, offsets->data_length, offsets->data_offset);
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "RCPT")
         || str_match(packet, offsets->data_length, offsets->data_offset, "Rcpt"))
        {
			
            // v prijatom pakete vyhladam adresu odosielatela a ulozim si ju
            save_add_receiver(*dataPointer, packet, tcpHeader->th_seq, offsets->data_length, offsets->data_offset);
	
			// ulozim si sekvencne cislo spravy, ktorej prijem sa prave potvrdil
            save_add_ack (*dataPointer, tcpHeader->th_ack);
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "EHLO")
         || str_match(packet, offsets->data_length, offsets->data_offset, "HELO"))
        {
			// z dovodu resetovania spojenia odstranim ulozene emailove adresy
            save_delete_inner_structures(*dataPointer, 0);
            (*dataPointer)->state = 3;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "DATA"))
        {
			// ulozim si sekvencne cislo spravy, ktorej prijem sa prave potvrdil
            save_add_ack (*dataPointer, tcpHeader->th_ack);
            (*dataPointer)->state = 9;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "354"))
        {
            (*dataPointer)->state = 9;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "RSET"))
        {
			// z dovodu resetovania spojenia odstranim ulozene emailove adresy
            save_delete_inner_structures(*dataPointer, 0);
            (*dataPointer)->state = 8;
        }
        else if( ( tcpHeader->th_flags & 1 ) > 0
         || str_match(packet, offsets->data_length, offsets->data_offset, "221")
         || str_match(packet, offsets->data_length, offsets->data_offset, "QUIT") )
        {
            send_to_socket("END", "has disconnected from the SMTP", dataPointer, table_item, NULL);
            
            // odstranenie vsetkych dat, ktore si modul uklada pre dany tok
            smtp_destroy_data( table_item, dataPointer );
            
            // po detekcie ukonceni spojenia uz modul nema zaujem o dalsie pakety
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
        }
    }
    else if((*dataPointer)->state == 9)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "EHLO")
         || str_match(packet, offsets->data_length, offsets->data_offset, "HELO"))
        {
			// z dovodu resetovania spojenia odstranim ulozene emailove adresy
            save_delete_inner_structures(*dataPointer, 0);
            (*dataPointer)->state = 3;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "RSET"))
        {
			// z dovodu resetovania spojenia odstranim ulozene emailove adresy
            save_delete_inner_structures(*dataPointer, 0);
            (*dataPointer)->state = 8;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "250"))
        {
            send_to_socket("CONTINUE", "successfully sent an e-mail", dataPointer, table_item, *dataPointer);
            (*dataPointer)->state = 10;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "4")
         || str_match(packet, offsets->data_length, offsets->data_offset, "5"))
        {
            send_to_socket("CONTINUE", "unsuccessfully tried send an e-mail", dataPointer, table_item, *dataPointer);
            (*dataPointer)->state = 10;
        }
        else if( ( tcpHeader->th_flags & 1 ) > 0
         || str_match(packet, offsets->data_length, offsets->data_offset, "221")
         || str_match(packet, offsets->data_length, offsets->data_offset, "QUIT") )
        {
            send_to_socket("END", "has disconnected from the SMTP", dataPointer, table_item, NULL);
            
            // odstranenie vsetkych dat, ktore si modul uklada pre dany tok
            smtp_destroy_data( table_item, dataPointer );
            
            // po detekcie ukonceni spojenia uz modul nema zaujem o dalsie pakety
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
        }
    }
    else if((*dataPointer)->state == 10)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "EHLO")
         || str_match(packet, offsets->data_length, offsets->data_offset, "HELO"))
        {
			// z dovodu resetovania spojenia odstranim ulozene emailove adresy
            save_delete_inner_structures(*dataPointer, 0);
            (*dataPointer)->state = 3;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "MAIL")
         || str_match(packet, offsets->data_length, offsets->data_offset, "Mail"))
        {
			// z dovodu resetovania spojenia odstranim ulozene emailove adresy
            save_delete_inner_structures(*dataPointer, 0);
            
            // v prijatom pakete vyhladam adresu odosielatela a ulozim si ju
            save_set_sender(*dataPointer, packet, tcpHeader->th_seq, offsets->data_length, offsets->data_offset);
            (*dataPointer)->state = 8;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "RSET"))
        {
			// z dovodu resetovania spojenia odstranim ulozene emailove adresy
            save_delete_inner_structures(*dataPointer, 0);
            (*dataPointer)->state = 8;
        }
        else if( ( tcpHeader->th_flags & 1 ) > 0
         || str_match(packet, offsets->data_length, offsets->data_offset, "221")
         || str_match(packet, offsets->data_length, offsets->data_offset, "QUIT") )
        {
            send_to_socket("END", "has disconnected from the SMTP", dataPointer, table_item, NULL);
            
            // odstranenie vsetkych dat, ktore si modul uklada pre dany tok
            smtp_destroy_data( table_item, dataPointer );
            
            // po detekcie ukonceni spojenia uz modul nema zaujem o dalsie pakety
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
        }
    }
    else if((*dataPointer)->state == 11)
    {
        if(str_match(packet, offsets->data_length, offsets->data_offset, "220") || packet[offsets->data_offset] < 25 ) // v pripade zacati TLS je na prvej pozicii asci hodnota 20-24 reprezentujuca typ TLS obsahu
        {
            send_to_socket("CONTINUE", "has initiated a SSL connection on SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 10;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "4")
         || str_match(packet, offsets->data_length, offsets->data_offset, "5"))
        {
            send_to_socket("REPORT", "has connected to the SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 4;
        }
        else if(str_match(packet, offsets->data_length, offsets->data_offset, "AUTH"))
        {
            send_to_socket("REPORT", "has connected to the SMTP", dataPointer, table_item, NULL);
            (*dataPointer)->state = 5;
        }
        else if( ( tcpHeader->th_flags & 1 ) > 0
         || str_match(packet, offsets->data_length, offsets->data_offset, "221")
         || str_match(packet, offsets->data_length, offsets->data_offset, "QUIT") )
        {
            send_to_socket("END", "has disconnected from the SMTP", dataPointer, table_item, NULL);
            
            // odstranenie vsetkych dat, ktore si modul uklada pre dany tok
            smtp_destroy_data( table_item, dataPointer );
            
            // po detekcie ukonceni spojenia uz modul nema zaujem o dalsie pakety
			return MODULE_WANTS_TO_FILTER_THIS_FLOW;
        }
    }
    
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_APP_MODULES == 1
		printf("%s %d: Stav spojenia po spracovani paketu - ", __FILE__, __LINE__);
		if( *dataPointer == NULL )
		{
			printf("0");
		}
		else
		{
			printf("%u", (*dataPointer)->state);
		}
		printf("\n");
	#endif
	
	// modul chce pokracovat v prijimani paketov
    return MODULE_WANTS_TO_ANALYZE_THIS_FLOW;
}

void smtp_process_end_of_flow( iri_iif_hash_table_item_t *table_item, void * input_dataPointer )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_APP_MODULES == 1
		printf("%s %d: smtp_process_end_of_flow()\n", __FILE__, __LINE__);
	#endif
	
	iri_iif_module_smtp_data_ptr * dataPointer = (iri_iif_module_smtp_data_ptr *) input_dataPointer;
	
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_APP_MODULES == 1
		printf("%s %d: Stav spojenia pred spracovanim spravy - ", __FILE__, __LINE__);
		if( *dataPointer == NULL )
		{
			printf("0");
		}
		else
		{
			printf("%u", (*dataPointer)->state);
		}
		printf("\n");
	#endif
	
	if((*dataPointer)->state == 9)
	{
		send_to_socket("CONTINUE", "did not finish sending an e-mail", dataPointer, table_item, *dataPointer);
	}
	
	send_to_socket("END", "has disconnected from the SMTP", dataPointer, table_item, NULL);
	
	// odstranenie vsetkych dat, ktore si modul uklada pre dany tok
	smtp_destroy_data( table_item, dataPointer );
}

int smtp_destroy_data( iri_iif_hash_table_item_t *table_item, void * input_dataPointer )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_APP_MODULES == 1
		printf("%s %d: smtp_destroy_data()\n", __FILE__, __LINE__);
	#endif
	
	// prekovertovanie typu ukazatela
	iri_iif_module_smtp_data_ptr * dataPointer = (iri_iif_module_smtp_data_ptr *) input_dataPointer;
	
	// odstranenie alokovanych struktur dynamicky alokovanych modulom
    save_delete_inner_structures(*dataPointer, 1);
	
	// odstranenie celkvej struktury
	free( *dataPointer );
	*dataPointer = NULL;
	
	// navratova hodnota nema zadny vyznam, len utisuje prekladac
	return TRUE;
}
