/*
 * Soubor:  iri-iif-tcp-reassembling.c
 * Datum:   27.05.2014
 * Autor:   Stanislav Bárta, xbarta29@stud.fit.vutbr.cz
 * Projekt: IRI-IIF pro SDM
 * Popis:   soubor obsahujici funkce pro skladani TCP toku
 */

#include "tcp-reassembling.h"


	/*
	printf("1 = %d\n", (*input_packet)->header.blob_size );
	printf("2 = %d\n", (*input_packet)->header.sze_int );
	printf("3 = %d\n", (*input_packet)->header.reserved );
	printf("4 = %d\n", (*input_packet)->header.timestamp_sec );
	printf("5 = %d\n", (*input_packet)->header.timestamp_ns );
    
	printf("6 = %d\n", tcpHeader->seq );
	printf("7 = %d\n", tcpHeader->ack_seq );
	printf("8 = %d\n", tcpHeader->source );
	printf("9 = %d\n", tcpHeader->dest );
	
	printf("A = %d\n", tcpHeader->syn );
	printf("B = %d\n", tcpHeader->fin );
	printf("C = %d\n", tcpHeader->rst );
	printf("D = %d\n", tcpHeader->ack );
    */

int iri_iif_tcp_reassembling_check_state_from_packet(struct tcphdr *tcpHeader, char * state, char * state2)
{
	// uspesne ukonceny threeway handshake
	if( *state == TCP_STATE_BOTH_SEQ_VALID )
	{
		if( tcpHeader->fin || tcpHeader->rst )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
				printf("%d %s %d: jedna strana ukoncila spojeni\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			*state = TCP_STATE_THIS_SIDE_CLOSED;
			*state2 = TCP_STATE_OTHER_SIDE_CLOSED;
			
			// pakety z ukoncovania spojenia nie je nutne odoslat modulom
			return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
		}
		
		//nasleduje bezne spracovanie paketu za switchom
		return TCP_REASSEMBLING_SEND_TO_MODULES;
	}
	// stanica odosielajuca tento paket uz odoslala paket FIN/RST
	else if( *state == TCP_STATE_THIS_SIDE_CLOSED )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
			printf("%d %s %d: ignorujem paket lebo sem prijal FIN\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
	}
	// stanica prijimajuca tento paket uz odoslala paket FIN/RST
	else if( *state == TCP_STATE_OTHER_SIDE_CLOSED )
	{
		if( tcpHeader->fin || tcpHeader->rst)
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
				printf("%d %s %d: obidve strany ukoncili spojeni\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			*state = TCP_STATE_CLOSED;
			*state2 = TCP_STATE_CLOSED;
			
			// pakety z ukoncovania spojenia nie je nutne odoslat modulom
			return TCP_REASSEMBLING_END_OF_THE_FLOW;
		}
		
		//nasleduje bezne spracovanie paketu za switchom
		return TCP_REASSEMBLING_SEND_TO_MODULES;
	}
	// stav kedy klient aj server ukoncili spojenie (tento stav by mal byt zbytecny)
	else
	{
		return TCP_REASSEMBLING_END_OF_THE_FLOW;
	}
}

char iri_iif_tcp_reassembling_process_new_packet( iri_iif_hash_table_item_t *input_hashTableRecord, iri_iif_offsets_t *input_offsets, ini3payload_t **input_packet, packetBufferItem_ptr * input_deleteItem, char * output_flowDirection )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
		printf("%d %s %d: iri_iif_tcp_reassembling_process_new_packet()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	int ipAddrLength; // velkost IP adresy
	struct tcphdr *tcpHeader; // struktura pre TCP hlavicku
	char direction; // smer do ktoreho dany paket patri
	struct packetBufferItem ** firstPacket; // ukazatel na prvy paket ulozeny v TCP reassemblingu
	struct packetBufferItem ** lastPacket; // ukazatel na posledny paket ulozeny v TCP reassemblingu
	uint32_t * expectedSeqNum; // sekvencne cislo dalsieho paketu v danom smere toku
	uint32_t * expectedSeqNum2; // sekvencne cislo dalsieho paketu v opacnom smere toku
	char * state; // stav reassemblingu v aktualnom toku
	char * state2; // stav reassemblingu v opacnom toku
	uint32_t * timestampSec; // cas prveho paketu prijateho pri detekovani diery
	
	// nastavenie dlzky IP adresy podla verzie IP protokolu
	ipAddrLength = ( input_offsets->ip_ver == 4 ) ? 4 : 16;
	
	// pretypovani na tcp strukturu pro dalsi zpracovani
	tcpHeader = (struct tcphdr *) ((*input_packet)->parts.data + input_offsets->l4_start);
	
	// zistim si do ktoreho smeru toku patri paket
	// porovnanie IP adres
    // TODO: podla mna blbost, zle indexy chyba tam +12 a +16
	int returnCode = memcmp( (*input_packet)->parts.data + input_offsets->ip_start , (*input_packet)->parts.data + input_offsets->ip_start + ipAddrLength , ipAddrLength );
	if( returnCode < 0 )
	{
		direction = FLOW_DIRECTION_SMALLER;
	}
	else if( returnCode > 0 )
	{
		direction = FLOW_DIRECTION_BIGGER;
	}
	else
	{
		// pri zhode IP adres porovnam porty
		if( *((uint16_t *) ((*input_packet)->parts.data + input_offsets->l4_start)) < *(((uint16_t *) ((*input_packet)->parts.data + input_offsets->l4_start)) + 2) )
		{
			direction = FLOW_DIRECTION_SMALLER;
		}
		else
		{
			direction = FLOW_DIRECTION_BIGGER;
		}
	}
	
	// za zaklade smeru toku si nastavim ukazatele z hash table s ktoryma budem dalej pracovat
	if( direction == FLOW_DIRECTION_SMALLER )
	{
		firstPacket = &(input_hashTableRecord->additionalData)->tcpr_firstPacketS;
		lastPacket = &(input_hashTableRecord->additionalData->tcpr_lastPacketS);
		expectedSeqNum = &(input_hashTableRecord->additionalData->tcpr_seqNumS);
		state = &(input_hashTableRecord->additionalData->tcpr_stateS);
		timestampSec = &(input_hashTableRecord->additionalData)->tcpr_timestampSecS;
		expectedSeqNum2 = &(input_hashTableRecord->additionalData->tcpr_seqNumB);
		state2 = &(input_hashTableRecord->additionalData)->tcpr_stateB;
		*output_flowDirection = direction;
	}
	else
	{
		firstPacket = &(input_hashTableRecord->additionalData->tcpr_firstPacketB);
		lastPacket = &(input_hashTableRecord->additionalData->tcpr_lastPacketB);
		expectedSeqNum = &(input_hashTableRecord->additionalData->tcpr_seqNumB);
		state = &(input_hashTableRecord->additionalData)->tcpr_stateB;
		timestampSec = &(input_hashTableRecord->additionalData)->tcpr_timestampSecB;
		expectedSeqNum2 = &(input_hashTableRecord->additionalData->tcpr_seqNumS);
		state2 = &(input_hashTableRecord->additionalData->tcpr_stateS);
		*output_flowDirection = direction;
	}
	
	// prehodeni big a little endian
	tcpHeader->seq =  htobe32( tcpHeader->seq );
	tcpHeader->ack_seq =  htobe32( tcpHeader->ack_seq );
	
	// zistit ci sa jedna o prvy paket v danem smere
	if( *state == TCP_STATE_INIT)
	{
		if( tcpHeader->ack )
		{
			if( tcpHeader->syn )
			{
				*expectedSeqNum = tcpHeader->seq + 1;
				*expectedSeqNum2 = tcpHeader->ack_seq;
				
			}
			else
			{
				*expectedSeqNum = tcpHeader->seq;
				*expectedSeqNum2 = tcpHeader->ack_seq;
				
			}
			*state = TCP_STATE_BOTH_SEQ_VALID;
			*state2 = TCP_STATE_BOTH_SEQ_VALID;
			
			// pakety z nadviazania spojenia nie je nutne odoslat modulom
			return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
		}
		else
		{
			return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
		}
	}
	
	/*
	// zistit ci sa jedna o prvy paket v danem smere
	switch( * state )
	{
		// stav ked som este neprijal ziadny paket (alebo len paket SYN - pozor na DDOS)
		case TCP_STATE_INIT:
			if( tcpHeader->ack )
			{
				if( tcpHeader->syn )
				{
					*expectedSeqNum = tcpHeader->seq + 1;
					*expectedSeqNum2 = tcpHeader->ack_seq;
					
				}
				else
				{
					*expectedSeqNum = tcpHeader->seq;
					*expectedSeqNum2 = tcpHeader->ack_seq;
					
				}
				*state = TCP_STATE_BOTH_SEQ_VALID;
				*state2 = TCP_STATE_BOTH_SEQ_VALID;
				
				// pakety z nadviazania spojenia nie je nutne odoslat modulom
				return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
			}
			else
			{
				return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
			}
		break;
		// uspesne ukonceny threeway handshake
		case TCP_STATE_BOTH_SEQ_VALID:
			if( tcpHeader->fin || tcpHeader->rst )
			{
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
					printf("%d %s %d: jedna strana ukoncila spojeni\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
				#endif
				*state = TCP_STATE_THIS_SIDE_CLOSED;
				*state2 = TCP_STATE_OTHER_SIDE_CLOSED;
				
				// pakety z ukoncovania spojenia nie je nutne odoslat modulom
				return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
			}
			
			//nasleduje bezne spracovanie paketu za switchom
		break;
		// stanica odosielajuca tento paket uz odoslala paket FIN/RST
		case TCP_STATE_THIS_SIDE_CLOSED:
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
				printf("%d %s %d: ignorujem paket lebo sem prijal FIN\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
		break;
		// stanica prijimajuca tento paket uz odoslala paket FIN/RST
		case TCP_STATE_OTHER_SIDE_CLOSED:
			if( tcpHeader->fin || tcpHeader->rst)
			{
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
					printf("%d %s %d: obidve strany ukoncili spojeni\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
				#endif
				*state = TCP_STATE_CLOSED;
				*state2 = TCP_STATE_CLOSED;
				
				// pakety z ukoncovania spojenia nie je nutne odoslat modulom
				return TCP_REASSEMBLING_END_OF_THE_FLOW;
			}
			
			//nasleduje bezne spracovanie paketu za switchom
		break;
		// stav kedy klient aj server ukoncili spojenie (tento stav by mal byt zbytecny)
		case TCP_STATE_CLOSED:
			return TCP_REASSEMBLING_END_OF_THE_FLOW;
		break;
	}
	*/
	
	// prijaty paket je v poradi
	if( tcpHeader->seq == *expectedSeqNum )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
			printf("%d %s %d: prijaty paket je v poradi\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		// updatovat dalsi ocekavane sekvencne cislo
		*expectedSeqNum = tcpHeader->seq + input_offsets->data_length;
		
		// skontrolovat ci sa jedna o vyplneni diery
		if( *firstPacket != NULL )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
				printf("%d vyplnil sem dieru\n", __LINE__);
			#endif
			// vynulovat cas diery
			*timestampSec = 0;
		}
		
		return iri_iif_tcp_reassembling_check_state_from_packet(tcpHeader, state, state2);
	}
	// duplikat jiz jednou zpracovaneho paketu
	else if( tcpHeader->seq < *expectedSeqNum )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
			printf("%d %s %d: duplikat jiz jednou zpracovaneho paketu\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		// ignorovat
		return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
	}
	// paket z buducnosti
	else
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
			printf("%d %s %d: paket z buducnosti\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		if( *timestampSec != 0 && (*input_packet)->parts.header.timestamp_sec - *timestampSec > TCP_REASSEMBLING_OLD_GAP_IN_SECONDS )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
				printf("%d %s %d: velka casova dira\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
		}
		
		if( tcpHeader->seq - *expectedSeqNum > TCP_REASSEMBLING_OLD_GAP_IN_BYTES )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
				printf("%d %s %d: velka bajtova dira\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
		}
		
		
		// kontrola ci nie je rozdiel casovej znacky aktualne spracovavaneho paketu a casu detekovanej diery prilis velky
		if( /*TODO HERE TADY SA TO ZACYKLI*/0==0 && ( ( *timestampSec != 0 && (*input_packet)->parts.header.timestamp_sec - *timestampSec > TCP_REASSEMBLING_OLD_GAP_IN_SECONDS)
		/*||
		// kontrola ci nie je rozdiel SeqNum aktualne spracovavaneho paketu a SeqNum diery prilis velky
		tcpHeader->seq - *expectedSeqNum > TCP_REASSEMBLING_OLD_GAP_IN_BYTES*/ ) )
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
				printf("%d %s %d: velka dira\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			// v pripade velkej diery sa moze stat ze nie je ulozeny ani jeden paket
			if( *firstPacket == NULL
			||
			// zistim ci prijaty paket je prvy paket za detekovanou dierou (t.j. nachadza sa pred aktualne prvym paketom)
			tcpHeader->seq < (*firstPacket)->tcpr_seqNum )
			{
				*expectedSeqNum = tcpHeader->seq + input_offsets->data_length;
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
					printf("%d %s %d: prijaty paket je prvy paket za detekovanou dierou\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
				#endif
			}
			// aktualne prijaty paket ulozim a nahradim ho prvym paketom v reassemblingu
			else
			{
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
					printf("%d %s %d: Jesus\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
				#endif
			//return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
				// ulozeni do packet bufferu
				iri_iif_tcp_reassembling_save_packet(input_hashTableRecord, input_offsets, *input_packet, firstPacket, lastPacket, timestampSec, tcpHeader->seq);
				
				*expectedSeqNum = (*firstPacket)->tcpr_seqNumNext;
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
					printf("%d %s %d: ta treti moznost\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
				#endif
				
				*input_offsets = (*firstPacket)->offsets;
				*input_packet = &(*firstPacket)->packet;
				*input_deleteItem = *firstPacket;
				
				iri_iif_packet_buffer_update_cache( *firstPacket ); 
			}
			
			
			// vynulovat cas diery (pri preskoceni diery sa system chova rovnako ako keby prisiel paket v poradi)
			*timestampSec = 0;
			
			return TCP_REASSEMBLING_SEND_TO_MODULES;
		}
		// diera este nie je stara, aktualny paket ulozim a cakam
		else
		{
			#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
				printf("%d %s %d: diera este nie je stara, aktualny paket ulozim a cakam\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
			#endif
			// ulozeni do packet bufferu
			iri_iif_tcp_reassembling_save_packet(input_hashTableRecord, input_offsets, *input_packet, firstPacket, lastPacket, timestampSec, tcpHeader->seq);
			
			if( *timestampSec == 0 )
			{
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
					printf("%d vznikla nova dira\n", __LINE__);
				#endif
				*timestampSec = (*input_packet)->parts.header.timestamp_sec;
			}
			
			return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
		}
	}
	
	return TCP_REASSEMBLING_SEND_TO_MODULES;
}

void iri_iif_tcp_reassembling_save_packet( iri_iif_hash_table_item_t *input_hashTableRecord, iri_iif_offsets_t *input_offsets, ini3payload_t *input_packet, struct packetBufferItem ** input_firstPacket, struct packetBufferItem ** input_lastPacket, uint32_t * input_timestampSec, uint32_t input_seqNum )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
		printf("%d %s %d: iri_iif_tcp_reassembling_save_packet()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	packetBufferItem_ptr bufferItem = NULL; // ukazatel na ulozeny paket
	packetBufferItem_ptr temp = NULL; // pomocny ukazatel na prvok pri hladani miesta kam vlozit prvok podla SeqNum
	
	// ulozenie paketu a ziskanie ukazatela na ulozeny paket
	iri_iif_packet_buffer_packet_save( input_hashTableRecord, input_offsets, input_packet, &bufferItem );
	
	// do ulozeneho paketu si nastavim priznak, ze TCP reassembling pouziva dany paket
	bufferItem->tcpr_inUse = TRUE;
	
	// nastavenie sekvencnych cisiel, aby som pri prechadzani TCP reassemblingu nemusel parsovat data
	bufferItem->tcpr_seqNum = input_seqNum;
	bufferItem->tcpr_seqNumNext = input_seqNum + input_offsets->data_length;
	
	// jedna sa o prvy paket ulozeny pre TCP reassembling daneho toku
	if( *input_firstPacket == NULL )
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
			printf("%d %s %d: jedna sa o prvy paket ulozeny pre TCP reassembling daneho toku\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// aktualizujem ukazatel na prvy aj posledny ulozeny paket
		*input_firstPacket = bufferItem;
		*input_lastPacket = bufferItem;
		
		// kedze sa jedna o jediny prvok, tak nema predosli ani nasledujuci
		bufferItem->tcpr_prev = NULL;
		bufferItem->tcpr_next = NULL;
		
		// tym ze sa jedna o prvy ulozeny paket, detekoval som dieru a tak si zapisem cas prijatia prveho paketu po diere
		*input_timestampSec = input_packet->parts.header.timestamp_sec;
	}
	else
	{
		#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
			printf("%d %s %d: jedna sa o dalsi paket ulozeny pre TCP reassembling daneho toku\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
		#endif
		
		// zacnem prechadzanie od posledneho ulozeneho prvku
		temp = *input_lastPacket;
		
		while( input_seqNum < temp->tcpr_seqNum )
		{
			// narazil som na zaciatok zoznamu
			if( temp == *input_firstPacket )
			{
				// prvy prvok bude ukazovat na novo vlozeny
				temp->tcpr_prev = bufferItem;
				
				// vkladany prvok ma pred sebou posledny paket a za sebou nic
				bufferItem->tcpr_prev = NULL;
				bufferItem->tcpr_next = *input_firstPacket;
				
				// aktualizujem ukazatel na posledny ulozeny paket
				*input_firstPacket = bufferItem;
				
				return;
			}
			
			temp = temp->tcpr_prev;
		}
		
		// v pripade ze uz mam ulozeny paket s rovnakym SeqNum, jedna sa o duplicitny paket
		if( input_seqNum == temp->tcpr_seqNum )
		{
			// TCP reassembling uz dany paket nepotrebuje
			bufferItem->tcpr_inUse = FALSE;
			
			// uvolnim ulozeny paket
			iri_iif_packet_buffer_packet_free( input_hashTableRecord, bufferItem );
			
			return;
		}
		
		// prvok sa vlozi na koniec zoznamu
		if( temp == *input_lastPacket )
		{
			// posledny prvok bude ukazovat na novo vlozeny
			temp->tcpr_next = bufferItem;
			
			// vkladany prvok ma pred sebou posledny paket a za sebou nic
			bufferItem->tcpr_prev = *input_lastPacket;
			bufferItem->tcpr_next = NULL;
			
			// aktualizujem ukazatel na posledny ulozeny paket
			*input_lastPacket = bufferItem;
		}
		// prvok sa vlozi do stredu zoznamu
		else
		{
			// vkladany prvok ma okolo seba prvky
			bufferItem->tcpr_prev = temp;
			bufferItem->tcpr_next = temp->tcpr_next;
			
			// predosly prvok bude ukazovat na novo vkladany prvok
			temp->tcpr_next->tcpr_prev = bufferItem;
			
			// nasledujuci prvok bude ukazovat na novo vkladany prvok
			temp->tcpr_next = bufferItem;
		}
	}
	
	/*
	// tok uz obsahuje nejake pakety pro TCP reassembling, novy paket umiestnim podla SeqNum
	else
	{
		// najprv pozriem ci sa prvok nema umiestnit na koniec zoznamu
		if( input_seqNum > (*input_lastPacket)->tcpr_seqNum )
		{
			// posledny prvok bude ukazovat na novo vlozeny
			(*input_lastPacket)->tcpr_next = bufferItem;
			
			// vkladany prvok ma pred sebou posledny paket a za sebou nic
			bufferItem->tcpr_prev = *input_lastPacket;
			bufferItem->tcpr_next = NULL;
			
			// aktualizujem ukazatel na posledny ulozeny paket
			*input_lastPacket = bufferItem;
		}
		// prvok je nutne ulozit bud na zaciatok alebo niekde do stredu zoznamu
		else
		{
			// zacnem prechadzanie od prveho ulozeneho prvku
			temp = *input_firstPacket;
			
			// hladam prvok, ktory je vacsi alebo rovny ako ten vkladany
			while( input_seqNum > temp->tcpr_seqNum )
			{
				temp = temp->tcpr_next;
			}
			
			// v pripade ze uz mam ulozeny paket s rovnakym SeqNum, jedna sa o duplicitny paket
			if( input_seqNum == temp->tcpr_seqNum )
			{
				// TCP reassembling uz dany paket nepotrebuje
				bufferItem->tcpr_inUse = FALSE;
				
				// uvolnim ulozeny paket
				iri_iif_packet_buffer_packet_free( input_hashTableRecord, bufferItem );
				return;
			}
			
			// vkladany prvok musi ukazovat na susedne prvky (v mieste kde je vkladany)
			bufferItem->tcpr_prev = temp->tcpr_prev;
			bufferItem->tcpr_next = temp;
			
			// prvok sa vklada uplne na zaciatok
			if( temp->tcpr_prev == NULL )
			{
				// aktualizujem ukazatel na prvy ulozeny paket
				*input_firstPacket = bufferItem;
			}
			// prvok sa vklada niekde do stredu zoznamu
			else
			{
				// predosly prvok bude ukazovat na novo vkladany prvok
				temp->tcpr_prev->tcpr_next = bufferItem;
			}
			
			// nasledujuci prvok bude spatnym ukazatelom ukazovat na vkladany prvok
			temp->tcpr_prev = bufferItem;
			
		}
	}
	*/
}

char iri_iif_tcp_reassembling_check_for_more_packets( iri_iif_hash_table_item_t *input_hashTableRecord, iri_iif_offsets_t *input_offsets, ini3payload_t **input_packet, packetBufferItem_ptr * input_deleteItem, char input_flowDirection )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
		printf("%d %s %d: iri_iif_tcp_reassembling_check_for_more_packets()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	struct packetBufferItem ** firstPacket; // ukazatel na prvy paket ulozeny v TCP reassemblingu
	struct packetBufferItem ** lastPacket; // ukazatel na posledny paket ulozeny v TCP reassemblingu
	uint32_t * expectedSeqNum; // sekvencne cislo dalsieho paketu v danom smere toku
	uint32_t * expectedSeqNum2; // sekvencne cislo dalsieho paketu v opacnom smere toku
	char * state; // stav reassemblingu v aktualnom toku
	char * state2; // stav reassemblingu v opacnom toku
	struct tcphdr *tcpHeader; // struktura pre TCP hlavicku
	uint32_t * timestampSec; // cas prveho paketu prijateho pri detekovani diery
	
	// pretypovani na tcp strukturu pro dalsi zpracovani
	tcpHeader = (struct tcphdr *) ((*input_packet)->parts.data + input_offsets->l4_start);
	
	// za zaklade smeru toku si nastavim ukazatele z hash table s ktoryma budem dalej pracovat
	if( input_flowDirection == FLOW_DIRECTION_SMALLER )
	{
		firstPacket = &(input_hashTableRecord->additionalData)->tcpr_firstPacketS;
		lastPacket = &(input_hashTableRecord->additionalData->tcpr_lastPacketS);
		expectedSeqNum = &(input_hashTableRecord->additionalData->tcpr_seqNumS);
		state = &(input_hashTableRecord->additionalData->tcpr_stateS);
		timestampSec = &(input_hashTableRecord->additionalData)->tcpr_timestampSecS;
		expectedSeqNum2 = &(input_hashTableRecord->additionalData->tcpr_seqNumB);
		state2 = &(input_hashTableRecord->additionalData)->tcpr_stateB;
	}
	else
	{
		firstPacket = &(input_hashTableRecord->additionalData->tcpr_firstPacketB);
		lastPacket = &(input_hashTableRecord->additionalData->tcpr_lastPacketB);
		expectedSeqNum = &(input_hashTableRecord->additionalData->tcpr_seqNumB);
		state = &(input_hashTableRecord->additionalData)->tcpr_stateB;
		timestampSec = &(input_hashTableRecord->additionalData)->tcpr_timestampSecB;
		expectedSeqNum2 = &(input_hashTableRecord->additionalData->tcpr_seqNumS);
		state2 = &(input_hashTableRecord->additionalData->tcpr_stateS);
	}
	
	// pred zavolanim funkcie som spracoval z TCP reassemblingu paket a tak ho uz mozem vymazat
	if( *input_deleteItem != NULL)
	{
		// ak prvok ma naslednika, odstranim jeho ukazatel na mazany prvok
		if( (*input_deleteItem)->tcpr_next != NULL )
		{
			(*input_deleteItem)->tcpr_next->tcpr_prev = NULL;
		}
		
		// posuniem ukazatel na prvy paket v TCP reassemblingu
		*firstPacket = (*firstPacket)->tcpr_next;
		
		// TCP reassembling uz dany paket nepotrebuje
		(*input_deleteItem)->tcpr_inUse = FALSE;
		
		iri_iif_packet_buffer_packet_free( input_hashTableRecord, *input_deleteItem );
	}
	
	// TCP reassembling neobsahuje ziadne dalsie pakety
	if( *firstPacket == NULL )
	{
		return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
	}
	
	// porovnat SeqNum ulozeneho paketu a SeqNum ocekavaneho paketu
	if( *expectedSeqNum == (*firstPacket)->tcpr_seqNum )
	{
		// vyberem paket z bufferu a vlozim ho do vystupnych ukazatelu
		*input_offsets = (*firstPacket)->offsets;
		**input_packet = (*firstPacket)->packet;
		
		*expectedSeqNum = (*firstPacket)->tcpr_seqNumNext;
		
		// po spracovani dalsieho paketu je nutne ho vymazat
		*input_deleteItem = *firstPacket;
		
		// do zalohy si ulozim ukazatel na aktualne spracovavany paket
		iri_iif_packet_buffer_update_cache( *firstPacket );
		
		return iri_iif_tcp_reassembling_check_state_from_packet(tcpHeader, state, state2);
		//return TCP_REASSEMBLING_SEND_TO_MODULES;
	}
	else
	{
		// ked nejsu SeqNum rovnake, nastavim timestampu
		*timestampSec = (*firstPacket)->packet.parts.header.timestamp_sec;
	}
	
	return TCP_REASSEMBLING_DONT_SEND_TO_MODULES;
}


void iri_iif_tcp_reassembling_free_one_flow(iri_iif_hash_table_item_t * input_hashTableRecord, packetBufferItem_ptr * input_firstPacket, packetBufferItem_ptr * input_lastPacket, char input_sendToModules)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
		printf("%d %s %d: iri_iif_tcp_reassembling_free_one_flow()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	packetBufferItem_ptr nextRecord; // pomocny ukazatel na nasledujuci prvok
	unsigned short int modulesVector; // premenna pre vektor modulov, ktore maju o dany tok zaujem
	
	// zmazem vsetky pakety
	while( *input_firstPacket != NULL )
	{
		// zalohujem si ukazatel na dalsi prvok
		nextRecord = (*input_firstPacket)->tcpr_next;
		
		// oznacim paket ako nepouzivany v TCP reassemblingu
		(*input_firstPacket)->tcpr_inUse = FALSE;
		
		// odoslanie paketu modulom v pripade potreby
		if( input_sendToModules && 1 == 0) // TODO: fix me
		{
			// odoslem paket modulom, ktore o neho maju zaujem a z navratovej hodnoty zistim novy vektor modulov
			modulesVector = iri_iif_modules_send_packet( input_hashTableRecord, &(*input_firstPacket)->offsets, &(*input_firstPacket)->packet);
			
			// vektor modulov je nulovy, nikto uz nema zaujem spracovavat dany tok
			if ( modulesVector == NOBODY_WANTS_TO_ANALYZE_THIS_FLOW )
			{
				#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
					printf("%d %s %d: Blokovanie toku, pretoze ziadny aplikacny modul o neho nema zaujem\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
				#endif
				
				// zmazat record v tabulke (prva faza)
				iri_iif_hash_table_destroy_record_phase_1( input_hashTableRecord, 1 );
				
				// dalsie pakety uz nebudem posielat modulom
				input_sendToModules = FALSE;
			}
		}
		
		// odstranim aktualny paket
		iri_iif_packet_buffer_packet_free( input_hashTableRecord, *input_firstPacket );
		
		// prejdem na zalohovany ukazatel
		*input_firstPacket = nextRecord;
	}
	
	// vynulujem ukazatel na posledny paket v TCP reassemblingu
	*input_lastPacket = NULL;
	
}

void iri_iif_tcp_reassembling_send_and_free(iri_iif_hash_table_item_t * input_hashTableRecord)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
		printf("%d %s %d: iri_iif_tcp_reassembling_send_and_free()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// odstranenie paketov z jedneho smeru toku
	iri_iif_tcp_reassembling_free_one_flow(input_hashTableRecord, &(input_hashTableRecord->additionalData->tcpr_firstPacketB), &(input_hashTableRecord->additionalData->tcpr_lastPacketB), TRUE);
	
	// odstranenie paketov z druheho smeru toku
	iri_iif_tcp_reassembling_free_one_flow(input_hashTableRecord, &(input_hashTableRecord->additionalData->tcpr_firstPacketS), &(input_hashTableRecord->additionalData->tcpr_lastPacketS), TRUE);
}

// zmazani seckych dat k nejakemu toku (ide sa mazat)
void iri_iif_tcp_reassembling_free(iri_iif_hash_table_item_t * input_hashTableRecord)
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
		printf("%d %s %d: iri_iif_tcp_reassembling_free()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	
	// odstranenie paketov z jedneho smeru toku
	iri_iif_tcp_reassembling_free_one_flow(input_hashTableRecord, &(input_hashTableRecord->additionalData->tcpr_firstPacketB), &(input_hashTableRecord->additionalData->tcpr_lastPacketB), FALSE);
	
	// odstranenie paketov z druheho smeru toku
	iri_iif_tcp_reassembling_free_one_flow(input_hashTableRecord, &(input_hashTableRecord->additionalData->tcpr_firstPacketS), &(input_hashTableRecord->additionalData->tcpr_lastPacketS), FALSE);

}

void iri_iif_tcp_reassembling_delete_first_n_packets_from_flow( iri_iif_hash_table_item_t * input_hashTableRecord, unsigned int input_deletePackets )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
		printf("%d %s %d: iri_iif_tcp_reassembling_delete_first_n_packets_from_flow()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	iri_iif_tcp_reassembling_free( input_hashTableRecord );
}

void iri_iif_tcp_reassembling_send_data_with_gaps( iri_iif_hash_table_item_t * input_hashTableRecord )
{
	#if IRI_IIF_DEBUG == 1 || IRI_IIF_DEBUG_TCP_REASSEMBLING == 1
		printf("%d %s %d: iri_iif_tcp_reassembling_send_data_with_gaps()\n", (unsigned int)pthread_self(), __FILE__, __LINE__);
	#endif
	iri_iif_tcp_reassembling_send_and_free( input_hashTableRecord );
	
}

