/**
 * \file pref_filter.c
 * \brief Prefix filter for filterd -- header file.
 * \author Roman Vrana, xvrana20@stud.fit.vutbr.cz
 * \date 2014
 */

#include <stdint.h>
#include <sdm.h>
#include "limits.h"

#ifndef PREF_FILTER_H
#define PREF_FILTER_H

/**
 * Structure for stored prefix.
 */ 
typedef struct pref_item {
    /*@{*/
    sdm_ip_t prefix; /**< IP form of the prefix. */
    uint8_t length; /**< Length of the prefix in bits. */
    uint32_t rid; /**< RID of the rule. */
    /*@}*/
} pref_item_t;

/**
 * Prefix filter structure
 */
typedef struct pref_filter_s {
    /*@{*/
    int last_item;
    pref_item_t pref_array[UINT16_MAX + 1];
    uint32_t v4_masks[33]; /**< Table with IPv4 masks. */
    uint64_t v6_masks[65]; /**< Table with IPv6 masks. */
    /*@{*/
} pref_filter_t;

/* Init function */
void pref_init(pref_filter_t *pf);

/* Function for comparing prefixes */
int pref_compare(pref_item_t *p, pref_item_t *pref);

/* Function for mathing IP to prefix */
int pref_match_v4(pref_filter_t *pf, sdm_ip_t *ip, uint8_t *m_flag, uint32_t *set_rid);
int pref_match_v6(pref_filter_t *pf, sdm_ip_t *ip, uint8_t *m_flag, uint32_t *set_rid);

/* Function for adding prefix */
void pref_add(pref_filter_t *pf, pref_item_t *pref);

/* Function for removing prefix */
void pref_rem(pref_filter_t *pf, pref_item_t *pref);

/* Function for Destroying prefix filter */
void pref_destroy(pref_filter_t *pf);

#endif /* PREF_FILTER_H */
