/*
 * File: sdm_req_queue.c -- queue for SDM requests
 * Author: Roman Vrana, xvrana20@stud.fit.vutbr.cz
 * Date: 2014
 */  

#include <string.h>
#include "sdm_req_queue.h"

int sdm_q_init(sdm_q_t *q)
{
    q->max_size = SDM_Q_SIZE - 1;
    q->head = 0;
    q->end = 0;
    
    memset(q->req_array, 0x0, SDM_Q_SIZE * sizeof(sdm_flow_identifier_t));

    return 0;
}


sdm_flow_identifier_t sdm_q_pull(sdm_q_t *q)
{
    sdm_flow_identifier_t failure;
    memset(&failure, 0x0, sizeof(sdm_flow_identifier_t));
    if (q->head == q->end) {
        failure.raw[0] = 1;
        return failure;
    }

    sdm_flow_identifier_t desired;
    memcpy(&desired, &(q->req_array[q->head]), sizeof(sdm_flow_identifier_t));
    
    q->head = (q->head + 1) & q->max_size;

    return desired;
}

static inline int sdm_q_full(sdm_q_t *q)
{
    return (((q->end + 1) & q->max_size) == q->head);
}

int sdm_q_push(sdm_q_t *q, sdm_flow_identifier_t *k)
{
    if (!sdm_q_full(q)) {
        q->req_array[q->end] = *k;
        q->end = (q->end + 1) & q->max_size;

        return 0;
    } else {
        return -1;
    }
}
