/**
 * \file ini3d_new.h
 * \brief INI3 daemon for LI HW probe -- header file
 * \author Roman Vrana, xvrana20@stud.fit.vutbr.cz
 * \date 2014
 */  

#ifndef INI3D_H
#define INI3D_H

#include <stdint.h>
#define FRAME_BUFFER_SIZE 1600

/**
 * \brief Structure of hello header of INI3d.
 */
typedef struct ini3header {
    /*@{*/
    uint8_t version; /**< Version of INI3 payload. */
    uint8_t reserved; /**< RESERVED PART OF MESSAGE */
    uint32_t probe_id; /**< ID of the probe. MUST match the ID used by CCCId. */
    /*@}*/
}__attribute__ ((__packed__)) ini3header_t;

/**
 * \brief Header of INI3 payload v1.
 */
typedef struct ini3_head {
    /*@{*/
    uint16_t blob_size; /**< Size of intercepted frame. */
    uint8_t sze_int; /**< Input interface of the intercepted frame. */
    uint8_t reserved; /**< RESERVED PART OF MESSAGE */
    uint32_t timestamp_sec; /**< Time of arrival (seconds). */
    uint32_t timestamp_ns; /**< Time of arrival (nanoseconds). */
    uint16_t rid[2]; /**< Reason IDs (source, destination). */
    /*@}*/
} ini3_head_t;

/**
 * \brief Structure of the payload (mostly for internal manipulation).
 */
typedef union ini3payload {
    /*@{*/
    struct {
        ini3_head_t header; /**< Header of the payload. */
        unsigned char data[FRAME_BUFFER_SIZE]; /**< Intercepted frame. */
    /*@}*/
    } parts;
    unsigned char raw[sizeof(ini3_head_t) + FRAME_BUFFER_SIZE];
}__attribute__ ((__packed__)) ini3payload_t;

#endif
