/**
 * \file commands.c
 * \brief 
 * \author Roman Vrana, xvrana20@stud.fit.vutbr.cz
 * \version 
 * \date 2015-02-23
 */
    
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>

#include "../../libs/error_logger/error_logger.h"
#include "probe_monitor.h"
#include "msg_types.h"
#include "commands.h"
#include "log_reader.h"

int probe_start(void *in, void *out, void *others)
{
    pid_t *pids = (pid_t *) out, temp_pid, daemon_pid;
    int pid_pipe[2];

    start_msg_t *start_info = (start_msg_t *) in;
    char int_ifc[10] = {'e','t','h'};

    FILE *pid_file;
    char pid_str[11];
    memset(pid_str, 0x0, 11);

    pipe(pid_pipe);
    // check if module is not running
    if (access(CCCID_PID_FILE, F_OK) != -1) {
        // module is already running
        pid_file = fopen(CCCID_PID_FILE, "r");
        fread(pid_str, sizeof(char), 10, pid_file);

        pids[0] = strtoul(pid_str, NULL, 0);
        fclose(pid_file);
    } else {
        // start module CCCI
        if (pids[0] == 0 || kill(pids[0], 0) == -1) {
            temp_pid = fork();

            if (temp_pid > 0) {
                // parent
                read(pid_pipe[0], &(pids[0]), 4);

                pid_file = fopen(CCCID_PID_FILE, "w");
                sprintf(pid_str, "%d", pids[0]);
                pid_str[strlen(pid_str)] = '\n';
                fwrite(pid_str, sizeof(char), 11, pid_file);

                wait(NULL);
            } else if (temp_pid < 0) {
                perror("");
                // error 
            } else {
                daemon_pid = fork();
                if (daemon_pid > 0) {               
                    write(pid_pipe[1], &daemon_pid, 4);
                    exit(EXIT_SUCCESS);
                } else if (daemon_pid == 0) {
                    setsid();
                    chdir("/");
                    for (int i = getdtablesize(); i > 2; i--)
                        close(i);

                    int null = open("/dev/null", O_WRONLY);

//                    dup2(STDIN_FILENO, null);
//                    dup2(STDOUT_FILENO, null);
//                    dup2(STDERR_FILENO, null);

                    dup2(null, STDIN_FILENO);
                    dup2(null, STDOUT_FILENO);
                    dup2(null, STDERR_FILENO);

                    if (execlp(start_info->exe_path_cccid,
                               "cccid",
                               "-H", start_info->SLIS_host,
                               "-P", start_info->SLIS_CCCI_port,
                               "-i", start_info->probe_id,
                               "-I", start_info->CCCI_TRAP_str,
                               NULL) == -1)
                    { 
                        perror("");
                        unlink(CCCID_PID_FILE);
                        exit(EXIT_FAILURE);
                    } 
                } else {
                    // error
                    exit(EXIT_FAILURE);
                }
            }
        }
    }

    // start module filterd
    memset(pid_str, 0x0, 11);
    if (access(FILTERD_PID_FILE, F_OK) != -1) {
        // module is already running
        pid_file = fopen(FILTERD_PID_FILE, "r");
        fread(pid_str, sizeof(char), 10, pid_file);

        pids[1] = strtoul(pid_str, NULL, 0);
        fclose(pid_file);
    } else {
        if (pids[1] == 0 || kill(pids[1], 0) == -1) {
            temp_pid = fork();

            if (temp_pid > 0) {
                // parent 
                read(pid_pipe[0], &(pids[1]), 4);

                pid_file = fopen(FILTERD_PID_FILE, "w");
                sprintf(pid_str, "%d", pids[1]);
                pid_str[strlen(pid_str)] = '\n';
                fwrite(pid_str, sizeof(char), 11, pid_file);

                wait(NULL);
            } else if (temp_pid < 0) {
                // error 
            } else {
                daemon_pid = fork();
                if (daemon_pid > 0) {               
                    write(pid_pipe[1], &daemon_pid, 4);
                    exit(EXIT_SUCCESS);
                } else if (daemon_pid == 0) {
                    setsid();
                    chdir("/");
                    for (int i = getdtablesize(); i > 2; i--)
                        close(i);

                    close(STDIN_FILENO);
                    close(STDOUT_FILENO);
                    close(STDERR_FILENO);

                    if (start_info->intercepted_ifc != 254) {
                        if (start_info->intercepted_ifc == 255) {
                            int_ifc[0] = 'a';
                            int_ifc[1] = 'n';
                            int_ifc[2] = 'y';
                        } else {
                            sprintf(int_ifc + 3, "%hhu", start_info->intercepted_ifc);
                        }
                        if (execlp(start_info->exe_path_filterd_pcap,
                                   "filterd",
                                   "-i", start_info->filterd_TRAP_str,
                                   "-d", int_ifc,
                                   NULL) == -1) 
                        {
                            perror("");
                            unlink(FILTERD_PID_FILE);
                            exit(EXIT_FAILURE);
                        }
                    } else {
                        if (execlp(start_info->exe_path_filterd,
                                   "filterd",
                                   "-i", start_info->filterd_TRAP_str,
                                   NULL) == -1) 
                        {
                            perror("");
                            unlink(FILTERD_PID_FILE);
                            exit(EXIT_FAILURE);
                        }
                    } 
                } else {
                    // error
                    exit(EXIT_FAILURE);
                }
            }
        }
    }

    // start module INI3
    memset(pid_str, 0x0, 11);
    if (access(INI3D_PID_FILE, F_OK) != -1) {
        // module is already running
        pid_file = fopen(INI3D_PID_FILE, "r");
        fread(pid_str, sizeof(char), 10, pid_file);

        pids[2] = strtoul(pid_str, NULL, 0);
        fclose(pid_file);
    } else {
        if (pids[2] == 0 || kill(pids[2], 0) == -1) {
            temp_pid = fork();

            if (temp_pid > 0) {
                // parent
                read(pid_pipe[0], &(pids[2]), 4);

                pid_file = fopen(INI3D_PID_FILE, "w");
                sprintf(pid_str, "%d", pids[2]);
                pid_str[strlen(pid_str)] = '\n';
                fwrite(pid_str, sizeof(char), 11, pid_file);

                wait(NULL);
            } else if (temp_pid < 0) {
                // error
            } else {
                daemon_pid = fork();
                if (daemon_pid > 0) {               
                    write(pid_pipe[1], &daemon_pid, 4);
                    exit(EXIT_SUCCESS);
                } else if (daemon_pid == 0) {
                    setsid();
                    chdir("/");
                    for (int i = getdtablesize(); i > 2; i--)
                        close(i);

                    close(STDIN_FILENO);
                    close(STDOUT_FILENO);
                    close(STDERR_FILENO);

                    if (execlp(start_info->exe_path_ini3d,
                               "ini3d",
                               "-H", start_info->SLIS_host,
                               "-P", start_info->SLIS_INI3_port,
                               "-e", start_info->probe_id,
                               "-i", start_info->INI3d_TRAP_str,
                               NULL)) 
                    {
                        perror("");
                        unlink(INI3D_PID_FILE);
                        exit(EXIT_FAILURE);
                    }
                } else {
                    // error
                    exit(EXIT_FAILURE);
                }
            }
        }
    }
    return 0;
}

int probe_stop(void *in, void *out, void *others)
{
    // check if module running
    pid_t *pids = (pid_t *) in;
    unsigned char *states = (unsigned char *) out;
    // check if module is running
    for (int i = 0; i < MOD_COUNT; i++) {
        if (pids[i] != 0 && kill(pids[i], SIGTERM) != -1) {
            sleep(2);
            if (kill(pids[i], 0) == -1) {
               states[i] = MOD_DOWN;
            } else {
               kill(pids[i], SIGKILL);
               states[i] = MOD_KILL;
            }
        } else if (pids[i] == 0) {
            states[i] = MOD_DOWN;
        }
    }
    unlink(CCCID_PID_FILE);
    unlink(FILTERD_PID_FILE);
    unlink(INI3D_PID_FILE);

    return 0;
}

int probe_restart(void *in, void *out, void *others)
{
    int ret;

    ret = probe_stop(out, NULL, NULL);
    ret = probe_start(in, out, NULL);

    return ret;
} 

int probe_report(void *in, void *out, void *others)
{
    pid_t *pids = (pid_t *) in;
    unsigned char *states = (unsigned char *) out;
    log_reader_t *log_rd = (log_reader_t *) others;
    // check if module is running
    for (int i = 0; i < MOD_COUNT; i++) {
        if (pids[i] == 0 || kill(pids[i], 0) == -1) {
            states[i] = MOD_DOWN << 4;
        } else {
            // if running, check last change of error log
            if (log_rd->fds[i] > 0)
                states[i] = reader_report(log_rd, i);
            else
                states[i] = MOD_OK << 4;
        }
    }

    return 0;
}

void populate_commands(cmd_t *vector)
{
    vector[START_MSG] = probe_start;
    vector[STOP_MSG] = probe_stop;
    vector[REPORT_MSG] = probe_report;
}
