/**
* \file log_reader.c
* \brief 
* \author Roman Vrana, xvrana20@stud.fit.vutbr.cz
* \date 2015-06-10
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <time.h>
#include <syslog.h>

#include <unistd.h>
#include <wait.h>
#include <fcntl.h>
#include <sys/stat.h>

#include "log_reader.h"
#include "../../libs/error_logger/error_logger.h"

#define PATH_LEN 1024

/* helpers */
static int parse_log(log_reader_t *rd, char *log, int mod);

static const char *LOG_NAMES[] =
{
    [0] = "cccid.log",
    [1] = "filterd.log",
    [2] = "ini3d.log"
};

int reader_init(log_reader_t *rd, const char *path, int min_severity)
{
    // set timestamps to now
    time_t t = time(NULL);
    char p[PATH_LEN];
    for (int i = 0; i < MOD_COUNT; i++) {
        rd->last_logs_ts[i] = t;
        memset(p, 0x0, PATH_LEN);
        // open logs
        if (path != NULL)
            strcat(p, path);
        else
            strcat(p, MODULES_LOG_PATH);

        strcat(p, LOG_NAMES[i]);

        rd->fds[i] = open(p, O_RDONLY);
    }

    rd->min_severity = min_severity;

    return 0;
}

int reader_report(log_reader_t *rd, int module)
{
    // check logs
    struct stat fileinfo;

    if (fstat(rd->fds[module], &fileinfo) == 0) {
        if (rd->last_logs_ts[module] <= fileinfo.st_ctime) {
            rd->last_logs_ts[module] = fileinfo.st_ctime;
            return MOD_LOG << 4;
        } else {
            return MOD_OK << 4;
        }
    } else {
        return MOD_OK << 4;
    }
}

void reader_set_min_severity(log_reader_t *rd, int severity)
{
    if (severity >= LOG_EMERG && severity <= LOG_DEBUG)
        rd->min_severity = severity;
}


void reader_destroy(log_reader_t *rd)
{
    // close all logs and set timestamps to 0
    for (int i = 0; i < MOD_COUNT; i++) {
        close(rd->fds[i]);
        rd->last_logs_ts[i] = 0x0;
    }
}
