/**
 * \file msg_parser.c
 * \brief 
 * \author Roman Vrana, xvrana20@stud.fit.vutbr.cz
 * \version 
 * \date 2015-02-17
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "msg_parser.h"

/* HELPERS */
int fill_start_msg(unsigned char *data, void **parsed);
int fill_setting_value(unsigned char *data, void **parsed);
/* HELPERS*/


/**
 * \brief 
 *
 * \param buffer
 *
 * \return 
 */
int parse_header(unsigned char *buffer, msg_t *message)
{
    // assign first 8 bytes from buffer to header
    message->header.type = buffer[0];
    memcpy(&(message->header.length),
           (uint32_t *) &(buffer[1]),
           sizeof(uint32_t));

    // check the header if everything is ok and return state
    if (message->header.type < STOP_MSG 
        ||(message->header.type > MONITOR_MSG
        && message->header.type < TRANS_END))
    {
        /* UNKNOWN HEADER TYPE -- BARK */
        return UNKNOWN_HEADER;
    }

    return PARSE_OK;
}

/**
 * \brief 
 *
 * \param message
 *
 * \return 
 */
int parse_message(msg_t *message, unsigned char *data)
{
    int state = UNKNOWN_DATA;
    switch (message->header.type) {
        case START_MSG:
            // separate data part of message to appropriate field
            state = fill_start_msg(data, &(message->data));
        break;

        case STOP_MSG:
        case REPORT_MSG:
            state = PARSE_OK;
        break;

        case MONITOR_MSG:
        case LOG_MSG:
            state = fill_setting_value(data, &(message->data));
        break;


        default:
            state = UNKNOWN_DATA;
        break;
            
    }
    
    return state;
}

/**
 * \brief 
 *
 * \param data
 * \param parsed
 *
 * \return 
 */
int fill_start_msg(unsigned char *data, void **parsed)
{
    start_msg_t *content = calloc(1, sizeof(start_msg_t));
    if (content == NULL)
        return ERROR_MEMORY;

    uint16_t *port_ptr = (uint16_t *) data;
    uint8_t *byte_ptr = data + sizeof(uint64_t);
    uint8_t *str_ptr = data + 2 * sizeof(uint64_t);

    sprintf(content->SLIS_CCCI_port, "%hu", *(port_ptr));
    sprintf(content->SLIS_INI3_port, "%hu", *(port_ptr + 1));
    sprintf(content->probe_id, "%u", *((uint32_t *) (port_ptr + 2)));

    content->SLIS_host_len = *(byte_ptr);
    content->CCCI_TRAP_len = *(byte_ptr + 1);
    content->filterd_TRAP_len = *(byte_ptr + 2);
    content->INI3_TRAP_len = *(byte_ptr + 3);

    content->exe_path_len = *(byte_ptr + 4);
    content->intercepted_ifc = *(byte_ptr + 5);

    content->exe_path_cccid = calloc(content->exe_path_len + strlen("cccid") + 1, sizeof(char));
    if (content->exe_path_cccid == NULL) {
        free(content);
        return ERROR_MEMORY;
    }

    content->exe_path_filterd = calloc(content->exe_path_len + strlen("filterd") + 1, sizeof(char));
    if (content->exe_path_filterd == NULL) {
        free(content->exe_path_cccid);
        free(content);
        return ERROR_MEMORY;
    }

    content->exe_path_filterd_pcap = calloc(content->exe_path_len + strlen("filterd-pcap") + 1, sizeof(char));
    if (content->exe_path_filterd_pcap == NULL) {
        free(content->exe_path_filterd);
        free(content->exe_path_cccid);
        free(content);
        return ERROR_MEMORY;
    }
    
    content->exe_path_ini3d = calloc(content->exe_path_len + strlen("ini3d") + 1, sizeof(char));
    if (content->exe_path_ini3d == NULL) {
        free(content->exe_path_filterd);
        free(content->exe_path_cccid);
        free(content);
        return ERROR_MEMORY;
    }

    content->SLIS_host = calloc(content->SLIS_host_len + 1, sizeof(char));
    if (content->SLIS_host == NULL) {
        free(content->exe_path_ini3d);
        free(content->exe_path_filterd);
        free(content->exe_path_cccid);
        free(content);
        return ERROR_MEMORY;
    }

    content->CCCI_TRAP_str = calloc(content->CCCI_TRAP_len + 1, sizeof(char));
    if (content->SLIS_host == NULL) {
        free(content->SLIS_host);
        free(content->exe_path_ini3d);
        free(content->exe_path_filterd);
        free(content->exe_path_cccid);
        free(content);
        return ERROR_MEMORY;
    }

    content->filterd_TRAP_str = calloc(content->filterd_TRAP_len + 1, sizeof(char));
    if (content->SLIS_host == NULL) {
        free(content->CCCI_TRAP_str);
        free(content->SLIS_host);
        free(content->exe_path_ini3d);
        free(content->exe_path_filterd);
        free(content->exe_path_cccid);
        free(content);
        return ERROR_MEMORY;
    }

    content->INI3d_TRAP_str = calloc(content->INI3_TRAP_len + 1, sizeof(char));
    if (content->SLIS_host == NULL) {
        free(content->filterd_TRAP_str);
        free(content->CCCI_TRAP_str);
        free(content->SLIS_host);
        free(content->exe_path_ini3d);
        free(content->exe_path_filterd);
        free(content->exe_path_cccid);
        free(content);
        return ERROR_MEMORY;
    }
        
    memcpy(content->exe_path_cccid, str_ptr, content->exe_path_len);
    strcat(content->exe_path_cccid, "cccid");

    memcpy(content->exe_path_filterd, str_ptr, content->exe_path_len);
    strcat(content->exe_path_filterd, "filterd");

    memcpy(content->exe_path_filterd_pcap, str_ptr, content->exe_path_len);
    strcat(content->exe_path_filterd_pcap, "filterd_pcap");

    memcpy(content->exe_path_ini3d, str_ptr, content->exe_path_len);
    strcat(content->exe_path_ini3d, "ini3d");

    str_ptr += content->exe_path_len;
    memcpy(content->SLIS_host, str_ptr, content->SLIS_host_len);

    str_ptr += content->SLIS_host_len;
    memcpy(content->CCCI_TRAP_str, str_ptr , content->CCCI_TRAP_len);

    str_ptr += content->CCCI_TRAP_len;
    memcpy(content->filterd_TRAP_str, str_ptr, content->filterd_TRAP_len);

    str_ptr += content->filterd_TRAP_len;
    memcpy(content->INI3d_TRAP_str, str_ptr, content->INI3_TRAP_len);

    *parsed = content;
    content = NULL;

    return PARSE_OK; 
}

void clear_start_msg(start_msg_t *msg)
{
    free(msg->SLIS_host);
    free(msg->CCCI_TRAP_str);
    free(msg->filterd_TRAP_str);
    free(msg->INI3d_TRAP_str);
    free(msg->exe_path_cccid);
    free(msg->exe_path_filterd);
    free(msg->exe_path_filterd_pcap);
    free(msg->exe_path_ini3d);
}

int fill_setting_value(unsigned char *data, void **parsed)
{
    uint32_t *content = calloc(1, sizeof(uint32_t));
    if (content == NULL)
        return ERROR_MEMORY;

    *content = *((uint32_t *) data);

    *parsed = content;
    return PARSE_OK;
}

int serialize(msg_t *message, unsigned char *buffer)
{
    memcpy(buffer, &(message->header), 8);
    memcpy(buffer + 8, message->data, message->header.length);

    return SERIALIZE_OK;
}

void error_response(unsigned char *response, unsigned probe_id,
                    unsigned int err_num)
{
    response[0] = ERROR_RES;
    *((uint32_t *) (response + 1)) = probe_id;
    response[6] = 0x0;
    *((int *) (response + 8)) = err_num;
}

void state_response(unsigned char *response, uint32_t probe_id, char timed,
                    unsigned char states[MOD_COUNT])
{
    response[0] = STATE_RES;
    *((uint32_t *) (response + 1)) = probe_id;
    response[6] = timed;
    for (int i = 0; i < MOD_COUNT; i++) {
        response[8 + i] = (states[i] & 0xF0) >> 4;
        response[8 + i + MOD_COUNT] = (states[i] & 0x0F);
    }
}

void set_response(unsigned char *response, uint32_t probe_id)
{
    response[0] = SET_RES;
    *((uint32_t *) (response + 1)) = probe_id;
    response[6] = 0x0;
    response[9] = 0x1;
}

void create_response(unsigned char *response, int type, uint32_t probe_id,
                     int err_num, void *opt)
{
    switch (type) {
        case ERROR_RES:
            error_response(response, probe_id, err_num);
        break;

        case STATE_RES:
            state_response(response, probe_id,
                           (char) err_num, (unsigned char *) opt);
        break;

        case SET_RES:
            set_response(response, probe_id);
        break;

        default:
            break;
    }
}
