#!/bin/sh

PRINTF_LEN=40

STATE_INSTALLED='\e[1m[  \e[1;32mINSTALLED\e[39m  \e[1m]\e[0m'
STATE_INSATLLED_DEV='\e[1m[  \e[1;32mINSTALLED+DEVEL\e[39m  \e[1m]\e[0m'
STATE_PRESENT='\e[1m[  \e[1;32mAVAILABLE\e[39m  \e[1m]\e[0m'
STATE_MISSING='\e[1m[  \e[1;31mMISSING\e[39m  \e[1m]\e[0m'

STATE_LOADED='\e[1m[  \e[1;32mLOADED\e[39m  \e[1m]\e[0m'
STATE_UNLOADED='\e[1m[  \e[1;31mUNLOADED\e[39m  \e[1m]\e[0m'

DRIVERS='szedata2 szedata2-cv3 combo6core combov3'
LIBS_DEP='libtrap libsdm libsze2 libcombo libcommlbr libnetconf'
SWTOOLS='combod crctest csboot csbus csbus-i2cms cscard-ident 
         csid c10g4ctl dmactl ethctl hfexctl hgenctl ibufctl i2cctl obufctl 
         sze2counter sze2fastwrite sze2loopback sze2machinegun sze2multiread 
         sze2pcap sze2read sze2tap sze2test sze2write tsuctl'

CARD_OK=1
CARD_TYPE='NONE'
DRIVERS_OK=1
DRIVERS_LOAD=1
LIBS_OK=1
SW_OK=1

MISSING_PKG=''
UNLOADED_DRIVERS=''

KERNEL=`uname -r`

echo 'Checking installed ComboV3 card ...'
lspci -d '18ec:fb40' -n | grep '18ec:fb40' > /dev/null && CARD_TYPE='COMBO40'
lspci -d '18ec:fbc1' -n | grep '18ec:fbc1' > /dev/null && CARD_TYPE='COMBO100'

if [[ $CARD_TYPE == 'NONE' ]]; then
    echo 'No compatible ComboV3 card installed. You have to install compatible card first.'
    CARD_OK=0
elif [[ $CARD_TYPE == 'COMBO40' ]]; then
    printf "%-""$PRINTF_LEN""s" 'COMBO-40G2/COMBO-10G8'
    echo -e $STATE_INSTALLED
elif [[ $CARD_TYPE == 'COMBO100' ]]; then
    printf "%-""$PRINTF_LEN""s" 'COMBO-100G1/COMBO-10G10'
    echo -e $STATE_INSTALLED
fi
echo ''



echo 'Checking installed drivers ...'

for dr in $DRIVERS; do
    printf "%-""$PRINTF_LEN""s" $dr 

    if [[ -n `find /lib/modules/"$KERNEL"/extra/ -iname "$dr.ko" 2> /dev/null` ]]; then
        echo -e $STATE_INSTALLED
    else
        echo -e $STATE_MISSING
        DRIVERS_OK=0
    fi
done

echo ''
if [ $DRIVERS_OK -eq 0 ]; then
    echo 'Some of drivers are missing from system. These are critical for correct'
    echo 'LI probe functinallity. '
    echo ''
elif [ $DRIVERS_OK -eq 1 ]; then
    echo 'All drivers are installed in system.'
fi
echo ''

echo 'Checking presence of drivers in kernel ...'
for dr in $DRIVERS; do
    printf "%-""$PRINTF_LEN""s" $dr

    if [[ -n `lsmod | grep $dr 2> /dev/null` ]]; then
        echo -e $STATE_LOADED
    else
        echo -e $STATE_UNLOADED
        UNLOADED_DRIVERS="$UNLOADED_DRIVERS $dr"
        DRIVERS_LOAD=0
    fi
done
echo ''

if [ $DRIVERS_LOAD -eq 0 ]; then
    echo 'Some drivers are not loaded in currently running kernel. If driver is installed'
    echo 'use "modprobe <driver_name>" command to load driver to kernel. If it fails check'
    echo 'driver installation.'
    echo ''
    echo "Drivers missing in kernel:$UNLOADED_DRIVERS"
elif [ $DRIVERS_OK -eq 1 ]; then
    echo 'All drivers are loaded.'
fi
echo ''

echo 'Checking libraries ...'
for l in $LIBS_DEP; do
    printf "%-""$PRINTF_LEN""s" $l
    
    if [[ -n `find /usr -iname "$l.so" 2> /dev/null` ]]; then
        if [[ -n `find /usr -iname "$l.h" 2> /dev/null` 
           || -n `find /usr -iname "\`sed s/lib// <<< $l\`.h" 2> /dev/null` ]]; then
            echo -e $STATE_INSATLLED_DEV
        else
            echo -e $STATE_INSTALLED
        fi
    else
        echo -e $STATE_MISSING
        LIBS_OK=0
    fi
done
echo ''

echo 'Checking SW tools ...'
for s in $SWTOOLS; do
    printf "%-""$PRINTF_LEN""s" $s
    
    if [[ -n `find /usr/bin -iname "$s" 2> /dev/null` 
       || -n `find /usr/local/bin -iname "$s" 2> /dev/null` ]]; then
        echo -e $STATE_INSTALLED
    else
        echo -e $STATE_MISSING
        SW_OK=0
    fi
done
echo ''

if [[ $LIBS_OK -eq 1 && $SW_OK -eq 1 && $CARD_OK -eq 1 ]]; then
    echo 'Checking firmware availability ...'
    printf "%-""$PRINTF_LEN""s" 'COMBO-40G2'
    if [[ $CARD_TYPE == 'COMBO40' ]]; then
        # check 40G firmware
        FW=`find $FW_PATH -iname combo40g2_sdm.mcs -printf '%h\n'`
        if [[ -n $FW ]]; then
            echo -e $STATE_PRESENT
        else
            echo -e $STATE_MISSING
        fi
        printf "%-""$PRINTF_LEN""s" 'COMBO-10G8'
        FW=`find $FW_PATH -iname combo8g10_sdm.mcs -printf '%h\n'`
        if [[ -n $FW ]]; then
            echo -e $STATE_PRESENT
        else
            echo -e $STATE_MISSING
        fi
    fi
    
    if [[ $CARD_TYPE == 'COMBO100' ]]; then
        #check for 10x10G firmware, then 1x100G
        printf "%-""$PRINTF_LEN""s" 'COMBO-10G10'
        FW=`find $FW_PATH -iname combo10g10_sdm.mcs -printf '%h\n'`
        if [[ -n $FW ]]; then
            echo -e $STATE_PRESENT
        else
            echo -e $STATE_MISSING
        fi
        printf "%-""$PRINTF_LEN""s" 'COMBO-100G1'
        FW=`find $FW_PATH -iname combo100g1_sdm.mcs -printf '%h\n'`
        if [[ -n $FW ]]; then
            echo -e $STATE_PRESENT
        else
            echo -e $STATE_MISSING
        fi
    fi
else
    echo 'No compatible ComboV3 card installed. Firmware check cannot be performed.'
fi

echo ''

PYTHON_OK=1
echo 'Checking python version ...'
printf "%-""$PRINTF_LEN""s" 'Python 2.X'
if env python2 --version > /dev/null 2>&1; then
    echo -e $STATE_INSTALLED "\e[1m`env python2 --version 2>&1 | sed 's/Python //'`\e[0m"
else
    echo -e $STATE_MISSING
    PYTHON_OK=0
fi

printf "%-""$PRINTF_LEN""s" 'Python 3.X'
if env python3 --version > /dev/null 2>&1; then
    echo -e $STATE_INSTALLED "\e[1m`env python3 --version 2>&1 | sed 's/Python //'`\e[0m"
else
    echo -e $STATE_MISSING
    PYTHON_OK=0
fi

echo ''

if [ $PYTHON_OK -eq 1 ]; then
    echo 'Python interpreter installed in correct versions. Configuration interface will be'
    echo 'available without any problems.'
    exit 0
else
    echo 'Python interpreter is missing in one of the versions or entirely.'
    echo 'You need to install or update your python instalation to use configuration'
    echo 'interface provided with li probe software.'
    exit 1
fi


