#!/bin/sh
FW_PATH=/usr/share/mcs/sdm/

# error reporting
function bark 
{
    echo "Error: $1" >&2
}


# check FW and load if needed
CARD_TYPE='NONE'
lspci -d '18ec:fb40' -n | grep '18ec:fb40' > /dev/null && CARD_TYPE='COMBO40'
lspci -d '18ec:fbc1' -n | grep '18ec:fbc1' > /dev/null && CARD_TYPE='COMBO100'

if [[ $CARD_TYPE == 'COMBO40' ]]; then
    # load 40G firmware
    FW=`find $FW_PATH -iname combo40g2_sdm.mcs -printf '%h\n'`
    if [[ -n $FW ]]; then
        csboot -d "$FW/design.xml" -f 100 "$FW/combo10g10_sdm.mcs"
    else
        FW=`find $FW_PATH -iname combo8g10_sdm.mcs -printf '%h\n'`
        if [[ -n $FW ]]; then
            csboot -d "$FW/design.xml" -f 100 "$FW/combo8g10_sdm.mcs"
        fi
    fi
    ibufctl -Ae1
elif [[ $CARD_TYPE == 'COMBO100' ]]; then
    #check for 10x10G firmware, then 1x100G and at last 8x10G
    FW=`find $FW_PATH -iname combo10g10_sdm.mcs -printf '%h\n'`
    if [[ -n $FW ]]; then
        csboot -d "$FW/design.xml" -f 100 "$FW/combo10g10_sdm.mcs"
    else
        FW=`find $FW_PATH -iname combo100g1_sdm.mcs -printf '%h\n'`
        if [[ -n $FW ]]; then
            csboot -d "$FW/design.xml" -f 100 "$FW/combo100g1_sdm.mcs"
    fi
    ibufctl -Ae1
else
    #no valid firmware
    bark 'No valid SDM firmware detected. Reinstall the probe software package or check the firmware paths.'
fi

# check if SLIS running (if supposed to run on probe)
# test connection to SLIS server
if [[ `ps aux | grep -e 'af' -e 'mf' -e 'iri' | grep -v 'grep' | wc -l` -eq 0 ]]; then
    if [[ -f /opt/slis/lilt.sh ]]; then
        /opt/slis/lilt.sh startcd
    else
        bark 'SLIS is not installed on this system. Make sure you have connection to your LI system.'
    fi
elif [[ `ps aux | grep -e 'af' -e 'mf' -e 'iri' | grep -v 'grep' | wc -l` -ne 3 ]]; then
    if [[ -f /opt/slis/lilt.sh ]]; then
        bark 'SLIS is not running properly. Shutdown the LI system and start it again.'
    else
        bark 'SLIS is not installed on this system. Make sure you have connection to your LI system.'
    fi
fi

# run probe_monitor daemon
echo "$0: Starting probe_monitor daemon ..."
probe_monitor

if [ -z `pidof probe_monitor` ]; then
    bark "probe_monitor is not running. Cannot start li probe software."
    exit 1
fi


echo "$0: Starting SDM SW daemon ..."
sdm_daemon
if [ -z `pidof sdm_daemon` ]; then
    bark "sdm_daemon is not running. Cannot start li probe software."
    exit 1
fi

echo "$0: Starting TimeStamp SW unit ..."
tsuctl
if [ -z `pidof tsuctl` ]; then
    bark "tsuctl is not running. Probe will not work properly with LI system."
    exit 1
fi
